/*
================================ COPYRIGHT ================================
The contents of this file are subject to the VTech Informations Ltd. License
of VT-OS Ver. 1.1 operating system (the "License"); you may not use this 
file except in compliance with the License.  
 
Software distributed under the License is distributed on an "AS IS" basis, 
WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
for the specific language governing rights and limitations under the License.
 
The Original Code is VT-OS Ver. 1.1 Operating System, released 
on October 1st, 1999

The Initial Developer of the Original Code is VTech Informations Ltd.  All 
codes are Copyright (C) VTech Informations Ltd. 1999.  All Rights Reserved.
===========================================================================
*/

/*
===========================================================================
File        :   ex_rate.c
Author(s)   :   Henry Fok
Company     :   VTech Informations Ltd.
Project     :   Helio 
Date:	    :   October 1st, 1999
Purpose:	:   Exchange rate
Revision    :   1.1
Note        :   None
===========================================================================
*/              


#include "stdafx.h"
#include "include\exrate.h"
#include "include\math.h"
#include "include\app.h"

BYTE        country_name[NUM_OF_COUNTRIES][4] = {"DZD", "ARP", "AUD", "ATS", "BSD",
    	    									 "BBD", "BEF", "BMD", "BRR", "BGL",
        										 "CAD", "CLP", "CNY", "CYP", "CSK",        									 
        										 "DKK", "EGP", "EUR", "FIM", "FJD",        									 
        										 "FRF", "DEM", "GRD", "HKD", "HUF",
        										 "ISK", "INR", "IDR", "IEP", "ILS",
        										 "ITL", "JMD", "JPY", "JOD", "LBP",
        										 "LUF", "MYR", "MXP", "NLG", "NZD",
        										 "NOK", "PKR", "PHP", "PLZ", "PTE",
        										 "ROL", "RUR", "SAR", "SGD", "SKK",
        										 "ZAR", "KRW", "ESP", "SDD", "SEK",
        										 "CHF", "TWD", "THB", "TTD", "TRL",
        									 	 "USD", "GBP", "VEB", "ZMK"};
        									 	
//double        exchange_rate_array[NUM_OF_COUNTRIES][NUM_OF_COUNTRIES];
EXCHANGERATE    *ExchangeRate;
BOOLEAN         ex_rate_db_init = FALSE;

/********************************************************
 * Function:	ExchangeInitExchangeRateDatabase
 * Purpose: 	This function is called to initialise 
				the exchange rate table 				
 * Scope:		application/internal
 * Input:		None
 * Output:		None
 * Return:		None
 * Comment: 	None
 *********************************************************/
void ExchangeInitExchangeRateDatabase()
{
	
    #define R00      1.82674
    #define R01      115.384
    #define R02      74.1114
    #define R03      9.45275
    #define R04      115.177
    #define R05      57.2722
    #define R06      3.22464
    #define R07      115.202
    #define R08      60.3153
    #define R09      0.0669481
    #define R10      77.0402
    #define R11      0.23389
    #define R12      13.9213
    #define R13      225.482
    #define R14      3.41518
    #define R15      17.5155
    #define R16      33.8487
    #define R17      130.148
    #define R18      21.8917
    #define R19      59.3603
    #define R20      19.8431
    #define R21      66.551
    #define R22      0.404899
    #define R23      14.8772
    #define R24      0.523955
    #define R25      1.63946
    #define R26      2.71469
    #define R27      0.013424
    #define R28      165.296
    #define R29      28.2708
    #define R30      0.0672349
    #define R31      3.05767
    #define R32      1
    #define R33      162.726
    #define R34      0.0765297
    #define R35      3.22858
    #define R36      30.3454
    #define R37      11.459
    #define R38      59.0504
    #define R39      63.672
    #define R40      15.0736
    #define R41      2.27785
    #define R42      2.94993
    #define R43      30.7893
    #define R44      0.649248
    #define R45      0.010236
    #define R46      5.00504
    #define R47      30.7488
    #define R48      68.1378
    #define R49      3.0495
    #define R50      18.8113
    #define R51      0.097924
    #define R52      0.782385
    #define R53      0.540982
    #define R54      14.6272
    #define R55      81.3567
    #define R56      3.57026
    #define R57      3.10961
    #define R58      18.6713
    #define R59      0.00034
    #define R60      115.157
    #define R61      188.398
    #define R62      0.199691
    #define R63      0.0525628

	double 		rate;
	USHORT 		x, y;
    double 		exchange_rate[NUM_OF_COUNTRIES];

    exchange_rate[0] = R00;
    exchange_rate[1] = R01;
    exchange_rate[2] = R02;
    exchange_rate[3] = R03;
    exchange_rate[4] = R04;
    exchange_rate[5] = R05;
    exchange_rate[6] = R06;
    exchange_rate[7] = R07;
    exchange_rate[8] = R08;
    exchange_rate[9] = R09;

    exchange_rate[10] = R10;
    exchange_rate[11] = R11;
    exchange_rate[12] = R12;
    exchange_rate[13] = R13;
    exchange_rate[14] = R14;
    exchange_rate[15] = R15;
    exchange_rate[16] = R16;
    exchange_rate[17] = R17;
    exchange_rate[18] = R18;
    exchange_rate[19] = R19;
    exchange_rate[20] = R20;

    exchange_rate[21] = R21;
    exchange_rate[22] = R22;
    exchange_rate[23] = R23;
    exchange_rate[24] = R24;
    exchange_rate[25] = R25;
    exchange_rate[26] = R26;
    exchange_rate[27] = R27;
    exchange_rate[28] = R28;
    exchange_rate[29] = R29;
    exchange_rate[30] = R30;

    exchange_rate[31] = R31;
    exchange_rate[32] = R32;
    exchange_rate[33] = R33;
    exchange_rate[34] = R34;
    exchange_rate[35] = R35;
    exchange_rate[36] = R36;
    exchange_rate[37] = R37;
    exchange_rate[38] = R38;
    exchange_rate[39] = R39;
    exchange_rate[40] = R40;

    exchange_rate[41] = R41;
    exchange_rate[42] = R42;
    exchange_rate[43] = R43;
    exchange_rate[44] = R44;
    exchange_rate[45] = R45;
    exchange_rate[46] = R46;
    exchange_rate[47] = R47;
    exchange_rate[48] = R48;
    exchange_rate[49] = R49;
    exchange_rate[50] = R50;

    exchange_rate[51] = R51;
    exchange_rate[52] = R52;
    exchange_rate[53] = R53;
    exchange_rate[54] = R54;
    exchange_rate[55] = R55;
    exchange_rate[56] = R56;
    exchange_rate[57] = R57;
    exchange_rate[58] = R58;
    exchange_rate[59] = R59;
    exchange_rate[60] = R60;

    exchange_rate[61] = R61;
    exchange_rate[62] = R62;
    exchange_rate[63] = R63;

    if (!ex_rate_db_init)
    {
        ExchangeRate = (EXCHANGERATE*)qmalloc(sizeof(EXCHANGERATE));
    	for (x = 1; x < NUM_OF_COUNTRIES; x++)
        {
            for (y = 0; y < x; y++)
            {
                rate = exchange_rate[x]/exchange_rate[y];
                ExchangeRate->exchange_rate_array[x][y] = rate;
            }
        }
        DataNewRecordWithID(maths_dbid, EXCHANGE_RATE, 0, 1);
        DataWriteField(maths_dbid, EXCHANGE_RATE, 0, sizeof(EXCHANGERATE), (BYTE*)ExchangeRate);
    }
          
    ex_rate_db_init = TRUE;
    DataWriteField(maths_dbid, EXCHANGE_INFO, 6, sizeof(BOOLEAN), (BYTE*)&ex_rate_db_init);
}

/********************************************************
 * Function:	ExchangeGetExchangeRate
 * Purpose: 	set the text of exchange rate
 * Scope:	application/internal
 * Input:	None
 * Output	an exchange_rate
 * Return:	None
 * Comment: 	None
 *********************************************************/
double ExchangeGetExchangeRate(USHORT first_country, USHORT second_country)
{
	if (first_country > second_country)
        return (ExchangeRate->exchange_rate_array[first_country][second_country]);
	else if (first_country < second_country)
        return (1/(ExchangeRate->exchange_rate_array[second_country][first_country]));
	else 
		return 1.0;
}

/********************************************************
 * Function:	ExchangeUpdateDatabase
 * Purpose: 	To update exchange rate database with new
 				exchange rates
 * Scope:		application
 * Input:		None
 * Output		None
 * Return:		None
 * Comment: 	None
 *********************************************************/
void ExchangeUpdateDatabase()
{
    DataWriteField(maths_dbid, EXCHANGE_RATE, 0, sizeof(EXCHANGERATE), (BYTE*)ExchangeRate);
    DataWriteField(maths_dbid, EXCHANGE_INFO, 6, sizeof(BOOLEAN), (BYTE*)&ex_rate_db_init);
}
