/***************************************************************************
 *                                                                         *
 * Project:                                                                *
 *              PDA                                                        *
 *                                                                         *
 * Filename:                                                               *
 *              PenMgr.h                                                   *
 *                                                                         *
 * Description:                                                            *
 *              Pen Manager for touch panel input device                   *
 *                                                                         *
 * Version:                                                                *
 *              1.0                                                        *
 *                                                                         *
 * History:                                                                *
 *              23/09/98   Prepared by Andrew Hui (AH)                     *
 *              23/09/98   V1.0 release (AH)                               *
 *                                                                         *
 ***************************************************************************/
#include "platform.h"
#ifdef PR31700

#include "include\penmgr.h"
#include "include\dev_pen.h"
#include "include\datatype.h"

#if 1
WORD pu_x, pu_y;
#endif

PenBufType	PenEvntQ[PEN_EVNT_Q_SIZE];	 /* X, Y, PenStatus = PEN_UP/PEN_MOVE */

WORD PenMgrStatus=0;  /* PEN_DOWN / PEN_MOVE */

WORD PenMgrMode=0;   /* PEN_RECOGNITION, PEN_CALIBRATION, PEN_NORMAL */


/* pen calibration information */

WORD PenMinX=196;
WORD PenMinY=221;

WORD PenMaxX=659;
WORD PenMaxY=625;

WORD PenEvntPtr  = 0;

WORD PenFilterMask [PEN_FILTER_SAMPLE] = {1,2,2};   /* sum >2 = pen move */

int Iabs(int a)
{
    if (a<0)
      return (-a);
    else return a;
}

void PenMgr(MsgType *Pmsg)
{
    WORD weight;
    WORD x,y;
    WORD PMcount;
    static  WORD PenMgrWorking=0;
    static  WORD PM_state=PM_IDLE;
    static  WORD Pen_msg_ignore=0;

    WORD i;

    if (PenMgrMode==PEN_NORMAL)
    {
//        if ((WORD)(Pmsg->msgPBP)<PenMsgSerial)  /* serial no */
//        {
            switch((WORD)(Pmsg->msgID))  /* only PEN_MOVE have x,y information */
			{
				case PEN_DOWN ://  printf(" Md ");
								break;

				case PEN_MOVE :// printf(" Mm ");
								PenEvntQ[PenEvntPtr%PEN_EVNT_Q_SIZE].x		= Pmsg->para1; /* x_ADC_value; */
								PenEvntQ[PenEvntPtr%PEN_EVNT_Q_SIZE].y		= Pmsg->para2; /* y_ADC_value; */
								PenEvntQ[PenEvntPtr%PEN_EVNT_Q_SIZE].status = PEN_MOVE;
								PenEvntPtr++;

								break;

				case PEN_UP   ://	printf(" Mu ");
                                if (PenEvntPtr< PEN_FILTER_SAMPLE-1)  /* U, M U */
								{
//                                    IntDisableGlobal();
									DIS_GBL_INT;
									PenEvntPtr=0; /* reset */
                                    PenTmrAction = PEN_TIMER_STOP;

//                                    if (PenTmrHandle!=0)
//                                    {
//                                        TmrIntDisable(PenTmrHandle);  /* disable timer interrupt */
//                                        printf("tR");
//                                        PenTmrHandle=0;
//                                    }
//                                    PenLastSerial=PenMsgSerial;
//                                    PenMsgSerial=0;
//                                    IntEnableGlobal();
										ENA_GBL_INT;
//                                    PenDownIntEnable();
                                }
								else
								{
									//printf(" Mu ");
									PenEvntQ[PenEvntPtr%PEN_EVNT_Q_SIZE].status = PEN_UP;
									PenEvntPtr++;
								}

								break;
			}

 			weight=0;

			/* classify Pen UP/DN/MV */
            if ((PenEvntPtr >= PEN_FILTER_SAMPLE))  /* enough samples to process */
			{
				for (i=0; i<PEN_FILTER_SAMPLE; i++)
					weight += PenFilterMask[i] * PenEvntQ[(PenEvntPtr - PEN_FILTER_SAMPLE+i)%PEN_EVNT_Q_SIZE].status;
			}

			switch (PM_state)
			{
				case PM_IDLE:   //printf("i");
								x		=0;
								y		=0;
								PMcount =0;

								if (weight > PEN_MOVE_THRESHOLD)  /* Pen_Moving */
								{
									for (i=0; i<PEN_FILTER_SAMPLE; i++) /* find averaged x, y */
									{
										if (PenEvntQ[(PenEvntPtr-PEN_FILTER_SAMPLE+i)%PEN_EVNT_Q_SIZE].status ==PEN_MOVE)
										{
											x += PenEvntQ [(PenEvntPtr -PEN_FILTER_SAMPLE+i)%PEN_EVNT_Q_SIZE].x;
											y += PenEvntQ [(PenEvntPtr -PEN_FILTER_SAMPLE+i)%PEN_EVNT_Q_SIZE].y;
											PMcount++;
										}
									}
									x /= PMcount;
									y /= PMcount;

                                    /* convert ADC value to 0 - 159 */

                                    x= Iabs(x-PenMinX)*160/Iabs(PenMaxX-PenMinX);
                                    y= Iabs(y-PenMinY)*160/Iabs(PenMaxY-PenMinY);
#if 1
pu_x = x;
pu_y = y;
#endif
                                    EvtAppendEvt (PEN_EVENT, PEN_DOWN, x, y, 0 );
									PM_state=PM_PM;
								}
								else
								{
									/* else stay in idle mode */
									if (PenEvntPtr > PEN_FILTER_SAMPLE-1)
									PenEvntPtr;
								}
								break;

				case PM_PM	:   //printf("m");
								x		=0;
								y		=0;
								PMcount =0;

								if (weight > PEN_MOVE_THRESHOLD)  /* Pen_Moving */
								{
									for (i=0; i<PEN_FILTER_SAMPLE; i++) /* find averaged x, y */
									{
										if (PenEvntQ[(PenEvntPtr-PEN_FILTER_SAMPLE+i)%PEN_EVNT_Q_SIZE].status ==PEN_MOVE)
										{
											x += PenEvntQ [(PenEvntPtr -PEN_FILTER_SAMPLE+i)%PEN_EVNT_Q_SIZE].x;
											y += PenEvntQ [(PenEvntPtr -PEN_FILTER_SAMPLE+i)%PEN_EVNT_Q_SIZE].y;
											PMcount++;
										}
									}
									x /= PMcount;
									y /= PMcount;

                                    /* convert ADC value to 0 - 159 */
                                    x= Iabs(x-PenMinX)*160/Iabs(PenMaxX-PenMinX);
                                    y= Iabs(y-PenMinY)*160/Iabs(PenMaxY-PenMinY);
#if 1
pu_x = x;
pu_y = y;
#endif
                                    EvtAppendEvt (PEN_EVENT, PEN_MOVE, x, y, 0 );
								}
								else /* Pen_Up, go back to idle mode */
								{
									printf("TC2");
//                                    IntDisableGlobal();
										DIS_GBL_INT;
									printf("\x8\x8\x8");

//                                    TmrIntDisable(PenTmrHandle);  /* disable timer interrupt */
//                                    PenLastSerial=PenMsgSerial;
//                                    PenMsgSerial=0;
//                                    printf(" tR ");
                                    PenEvntPtr=0;
                                    PenTmrAction = PEN_TIMER_STOP;
//                                    IntEnableGlobal() ;
									ENA_GBL_INT;
//                                    PenDownIntEnable();

									PM_state=PM_IDLE;
#if 0
									EvtAppendEvt (PEN_EVENT, PEN_UP, 0, 0, 0 );
#endif
#if 1
									EvtAppendEvt (PEN_EVENT, PEN_UP, pu_x, pu_y, 0 );
#endif
//                                    PenTmrHandle=0;
//                                    PenState3=6;
//                                    PenGetxy4();
								}
								break;
			}
//        }
        /*   else   ignore pending message */
	}

	if (PenMgrMode==PEN_CALIBRATION)	/* to be implement */
	{
		// read message
	}


	if (PenMgrMode==PEN_RECOGNITION)	/* to be implement */
	{
		// read message


	}
		PenMgrWorking=0;
}
#endif

