/*
================================ COPYRIGHT ================================
The contents of this file are subject to the VTech Informations Ltd. License
of VT-OS Ver. 1.1 operating system (the "License"); you may not use this 
file except in compliance with the License.  
 
Software distributed under the License is distributed on an "AS IS" basis, 
WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
for the specific language governing rights and limitations under the License.
 
The Original Code is VT-OS Ver. 1.1 Operating System, released 
on October 1st, 1999

The Initial Developer of the Original Code is VTech Informations Ltd.  All 
codes are Copyright (C) VTech Informations Ltd. 1999.  All Rights Reserved.
===========================================================================
*/

/*
===========================================================================
File        :   aplaunch.c
Author(s)   :   Henry Fok
Company     :   VTech Informations Ltd.
Project     :   Helio 
Date:	    :   October 1st, 1999
Purpose:	:   application c file
Revision    :   1.1
Note        :   None
===========================================================================
*/              



#include "include\app.h"
#include "include\appfunc.h"

#define DBG_MESSAGE
#define DBG_APP_BEGIN()
#define DBG_APP_END()

//#include <emudbg.h>

extern BOOLEAN draw_view_edit_scrollbar;

#define DBG_MESSAGE

/********************************************************
 * Function:	PhonebookAppLaunch
 * Purpose: 	This function is called to launch the 
				phonebook application
 * Scope:		application/internal
 * Input:		None
 * Output:		None
 * Return:		TRUE if handled
				FALSE if not handled
 * Comment: 	None
 *********************************************************/
BOOLEAN PhonebookAppLaunch(WORD cmd, void *cmd_ptr)
{
	
	UWORD 			num_rec, count;
	UWORD 			byte_read;
	USHORT 			field_num, total_entry, entry_count = 0;
	RecordID 		rec_id;
    BYTE            *buffer;
	BYTE 			*prepare_string;
    DmFieldArray    dm_field_array;
    DmFieldBuffer   *dm_field_buffer;
	
	if (cmd == LAUNCH_CMD_GOTO_REC && DataRecordInfo(((GotoRec*)cmd_ptr)->dbid, ((GotoRec*)cmd_ptr)->rec_id, NULL, NULL, NULL, NULL, NULL) != TRUE)
	{
		if (((GotoRec*)cmd_ptr)->find_string != NULL)
			pfree(((GotoRec*)cmd_ptr)->find_string);
		pfree(cmd_ptr);
		cmd = LAUNCH_CMD_NORMAL_LAUNCH;
	}		
	
	switch(cmd)
	{
     	case LAUNCH_CMD_NORMAL_LAUNCH:
			UIApplicationInit();
			DBG_APP_BEGIN();
			DBG_MESSAGE("LAUNCH_CMD_NORMAL_LAUNCH\r\n");
			PhonebookAppRestore(FALSE);
			EventLoop();	
			if (entry_low_memory == FALSE || (entry_low_memory == TRUE && entry_old_edit == FALSE))
				PhonebookAppSave();
			DBG_MESSAGE("Exiting ...\r\n");
			DBG_APP_END();
			UIDeleteAllAppObjects();
            DataCloseDB(pb_dbid);
     		return TRUE;
		case LAUNCH_CMD_FIND:
            PhonebookDatabaseChecking();
			app_id = SysGetActiveAppID();		
			DataTotalRecord(pb_dbid, &num_rec);
			count = 0;
			while (count < num_rec)
			{
				DataOpenRecord(pb_dbid, count, &rec_id, NULL);
				DataGetAllField(pb_dbid, rec_id, &dm_field_array);
				total_entry = dm_field_array.total_entry;																			
                dm_field_buffer = dm_field_array.data_array;
				entry_count = 0;
				while (entry_count < total_entry)
				{
                    if (GlobalFindSearchText((BYTE*)cmd_ptr, (BYTE*)(dm_field_buffer[entry_count]).data) >= 0)
					{
						PhonebookPrepareFindText(pb_dbid, rec_id, &prepare_string);
#ifdef PR31700
                        GlobalFindAddItem(app_id, pb_dbid, rec_id, entry_count, prepare_string, FALSE);
#endif
						pfree(prepare_string);
						break;
					}
					entry_count ++;
				}
                DataFreeFieldArray(dm_field_array);        
				count ++;
			}
			return TRUE;			
		case LAUNCH_CMD_GOTO_REC:
			DBG_APP_BEGIN();
			DBG_MESSAGE("LAUNCH_CMD_GOTO_REC\r\n");
			gf_entry = TRUE;
			PhonebookDatabaseChecking();
			PhonebookGotoItem(((GotoRec*)cmd_ptr)->dbid, ((GotoRec*)cmd_ptr)->rec_id, ((GotoRec*)cmd_ptr)->field_num, ((GotoRec*)cmd_ptr)->find_string);
//			UIApplicationInit();
//			PhonebookAppRestore(FALSE);
			if (((GotoRec*)cmd_ptr)->find_string != NULL)
			{

//				DataRecIDtoNum(((GotoRec*)cmd_ptr)->dbid, ((GotoRec*)cmd_ptr)->rec_id, &num_rec);
				pfree(((GotoRec*)cmd_ptr)->find_string);
//				EvtAppendEvt(EVT_LIST_SELECT,0,num_rec - list_tel_list.top_item_num,0,NULL);
			}

			pfree(cmd_ptr);

			EventLoop();	
			
			if ((entry_low_memory == FALSE || (entry_low_memory == TRUE && entry_old_edit == FALSE)) 
                && (gf_entry == FALSE || (gf_entry == TRUE && entry_old_edit == FALSE)))
				PhonebookAppSave();

			DBG_MESSAGE("Exiting ...\r\n");
			DBG_APP_END();
			UIDeleteAllAppObjects();
            DataCloseDB(pb_dbid);
			return TRUE;
		case LAUNCH_CMD_ALARM_HIT:
            pfree(cmd_ptr);
            DataCloseDB(pb_dbid);
			return TRUE;
		default:
			return FALSE;
	}
	return FALSE;
}

/********************************************************
 * Function:	PhonebookPrepareFindText
 * Purpose: 	This function is called to prepare the 
				text for a record to be displayed in the global find 
				application
 * Scope:		application/internal
 * Input:		dbid
				rec_id
				prepare_string
 * Output:		None
 * Return:		None
 * Comment: 	None
 *********************************************************/
void PhonebookPrepareFindText(DatabaseID dbid, RecordID rec_id, BYTE **prepare_string)
{
	SHORT			string_name_width, string_phone_width;
	ObjectBounds	name_bounds, phone_num_bounds;
	BYTE			*name, *phone_num, *buffer;
	UWORD			byte_read, name_num_chars;

	name_bounds.xcoord	= 0;
	name_bounds.ycoord	= 0;
	name_bounds.width	= 67;
	name_bounds.height	= 9;

	phone_num_bounds.xcoord	= 0;
	phone_num_bounds.ycoord	= 0;
	phone_num_bounds.width	= 67;
	phone_num_bounds.height	= 9;

	*prepare_string = (BYTE*)pmalloc(100*sizeof(BYTE));
	DataGetField(dbid, rec_id, 2, &buffer, &byte_read);
        name = (BYTE*)pmalloc((strlen(buffer) + 10) *sizeof(BYTE));
	strcpy(name, buffer);
        qfree(buffer);
	StrChopString(&name_bounds, name, SMALL_FONT, TRUE, 0);
	string_name_width = StrGetWidth(name, SMALL_FONT);
	DataGetField(dbid, rec_id, 5, &buffer, &byte_read);
        phone_num = (BYTE*)pmalloc((strlen(buffer) + 10) *sizeof(BYTE));
	strcpy(phone_num, buffer);
	qfree(buffer);
	StrChopString(&phone_num_bounds, phone_num, SMALL_FONT, TRUE, 0);
	string_phone_width = StrGetWidth(phone_num, SMALL_FONT);
	strcpy(*prepare_string, name);
	name_num_chars = strlen(name);
	(*prepare_string)[name_num_chars] = 11;
	(*prepare_string)[name_num_chars + 1] = 139 - string_phone_width - string_name_width;
	sprintf((char*)(*prepare_string + name_num_chars + 2), "%s", phone_num);
	pfree(name);
	pfree(phone_num);
}

/********************************************************
 * Function:	PhonebookGotoItem
 * Purpose: 	This function is called to init all related variables
				and goto the specific form to display the required 
				record
 * Scope:		application/internal
 * Input:		dbid
				rec_id
				field_num
				search_string
 * Output:		None
 * Return:		None
 * Comment: 	None
 *********************************************************/
void PhonebookGotoItem(DatabaseID dbid, RecordID rec_id, USHORT field_num, BYTE *search_string)
{
    BYTE *buffer, out_string[200], object_type;
	UWORD byte_read, rec_num;
	WORD count, total_num_lines;
	USHORT item_num, temp_top_row_num = 0;
	UBYTE cat;
	WORD highlight_char_pos = 0, num_lines = 0, line_num;
	Field *field_ptr;
	BOOLEAN found = FALSE;
	EvtType temp_event;
    Scrollbar *scroll_ptr;
	Textbox *textbox_ptr;
	BOOLEAN	highlight = FALSE;
	SHORT	row_num_array[30];

	
	DBG_MESSAGE("PhonebookGotoItem\r\n");

	/* global variables initialisation */
	pb_dbid = dbid;
    pressed_object = 0;
    current_cate = NULL;
    char_start = 'A';
    temp_cate = NULL;
    save_cate = NULL;
    save_first_letter = 'A';
    table_font_size = SMALL_FONT;
    SetStatus(TEL_LIST,FALSE,FALSE,FALSE,0,FALSE);
    list_tel_list.list_id = LIST_NAME;
    list_find.list_id = LIST_FIND_RESULT_NAME;
    list_tel_list.dbid = list_find.dbid = 0;
    list_tel_list.max_num_items_display = list_find.max_num_items_display = 0;
    list_tel_list.num_items_display = list_find.num_items_display = 0;
    list_tel_list.total_num_items = list_find.total_num_items = 0;
    list_tel_list.top_item_num = list_find.top_item_num = 0;        
    list_tel_list.record_id_block = list_find.record_id_block = NULL;        
	list_tel_list.dbid = list_find.dbid = pb_dbid;

	FormInitAllFormObjects(FORM_TEL_LIST);
	FormInitAllFormObjects(FORM_VIEW_EDIT);


	DBG_MESSAGE("PhonebookGotoItem A\r\n");
    if (MemoryLow)
        entry_low_memory = TRUE;
    else 
		entry_low_memory = FALSE;
    
	DataGetField(pb_dbid, PB_STATUS, 0, &buffer, &byte_read);
    
    if (*buffer == EDITING)
		entry_old_edit = TRUE;
	else 
		entry_old_edit = FALSE;		    	

    qfree(buffer);    
	

	DBG_MESSAGE("PhonebookGotoItem B\r\n");
	/* Allocate global variable */
	field_buffers = (BYTE**)qmalloc(26*sizeof(BYTE*));
	for (count = 0; count <26; count++)
		field_buffers[count] = NULL;
	current_cate = (BYTE*)qmalloc(30*sizeof(BYTE));

//    DataOpenDB(dbid, SORT_TEXT_MODE | 2, OPEN_RW);
	DataRecIDtoNum(dbid, rec_id, &rec_num);
	DataOpenRecord(dbid, rec_num, &rec_id, NULL);
	DataRecordInfo(dbid, rec_id, NULL, &cat, NULL, NULL, NULL);
	//set up current cate
	DataCategoryName(dbid, cat, current_cate);
	//set up char start
	DataGetField(dbid, rec_id, 2, &buffer, &byte_read);
	char_start = buffer[0];
	qfree(buffer);
	
    DataGetField(pb_dbid, PB_STATUS, 2, &buffer, &byte_read);
    strcpy(find_string, buffer);
    qfree(buffer);
	

    DataGetField(pb_dbid, PB_STATUS, 3, &buffer, &byte_read);
    font_size = *buffer;
    qfree(buffer);

	DBG_MESSAGE("PhonebookGotoItem C\r\n");

	//set up FORM_TEL_LIST
    draw_scrollbar = FALSE;
	PhonebookRecordToList(pb_dbid, current_cate, char_start, NULL);
    draw_scrollbar = TRUE;
	PhonebookCateNameToPopupTrigger(pb_dbid);
	if (ControlPopupFindItemNum(POPUP_TRIGGER_CATE, current_cate, &item_num) == TRUE) 
		ControlPopupSetSelectedItem(POPUP_TRIGGER_CATE, (SHORT)item_num);
	else
		ControlPopupSetSelectedItem(POPUP_TRIGGER_CATE, 0);
	DataOpenRecord(dbid, rec_num, &rec_id, NULL);
	
	DBG_MESSAGE("PhonebookGotoItem D\r\n");
	//set up FORM_VIEW_EDIT
	/* field_buffers */
	for (count = 0; count < 26; count++)
	{
		DataGetField(pb_dbid, rec_id, (USHORT)count, &buffer, &byte_read);			
		field_buffers[count] = (BYTE*)qmalloc((strlen(buffer) + 1)*sizeof(BYTE));
		strcpy(field_buffers[count], buffer);
		qfree(buffer);
	}
	// temp_cate
	temp_cate = (BYTE*)qmalloc(30*sizeof(BYTE));
	strcpy(temp_cate, current_cate);

	DBG_MESSAGE("PhonebookGotoItem E\r\n");
	/* app_status */
	app_status.status = VIEW;						
	app_status.key_in = FALSE;
	app_status.rec_id = rec_id;
	app_status.new_record = FALSE;
	app_status.company = FALSE;
	app_status.home = FALSE;

	/* STRING_NAME */
	DataGetField(pb_dbid, rec_id, 2, &buffer, &byte_read);			
	StringSetText(STRING_NAME, buffer);
	qfree(buffer);
	/* STRING_LAST_NAME and TEXTBOX_LAST_NAME*/
	DataGetField(pb_dbid, rec_id, 1, &buffer, &byte_read);			
	TextboxSetText(TEXTBOX_LAST_NAME, buffer);	
	qfree(buffer);
	/* STRING_FIRST_NAME and TEXTBOX_FIRST_NAME*/
	DataGetField(pb_dbid, rec_id, 0, &buffer, &byte_read);			
	TextboxSetText(TEXTBOX_FIRST_NAME, buffer);
	qfree(buffer);

	/* STRING_VIEW_EDIT_LARGE_NAME && STRING_VIEW_EDIT_LARGE_TITLE &&
	   STRING_VIEW_EDIT_LARGE_COMPANY*/
	PhonebookPrepareFirstLastName(field_buffers[0], field_buffers[1], out_string);
	StringSetText(STRING_NAME, out_string);
	StringSetText(STRING_VIEW_EDIT_LARGE_TITLE,		field_buffers[4]);
	StringSetText(STRING_VIEW_EDIT_LARGE_COMPANY,	field_buffers[3]);

	DBG_MESSAGE("PhonebookGotoItem F\r\n");


//	if (field_num >= 0 && field_num <= 14)
//		app_status.company = TRUE;
//	else if (field_num >= 15 && field_num <= 25)
		app_status.home = TRUE;
		
	/* Setup FORM_VIEW_EDIT */
	//PhonebookChangeViewInfo();
	
	DBG_MESSAGE("PhonebookGotoItem G\r\n");

	PhonebookRecordToFieldBuffers(app_status.rec_id);
	PhonebookFieldBuffersToField();
	PhonebookViewEditSetupScreen();
	PhonebookGetMappedRowNum(row_num_array);
	PhonebookCateNameToPopupTrigger(pb_dbid);
	ControlPopupDeleteItem(POPUP_TRIGGER_CATE, 0);
	if (ControlPopupFindItemNum(POPUP_TRIGGER_CATE, temp_cate, &item_num) == TRUE) 
		ControlPopupSetSelectedItem(POPUP_TRIGGER_CATE, (SHORT)item_num);
	else
		ControlPopupSetSelectedItem(POPUP_TRIGGER_CATE, 0);	
	if (save_cate != NULL)
		qfree(save_cate);
	save_cate = (BYTE*)qmalloc(30*sizeof(BYTE));
	strcpy (save_cate, temp_cate);
	
	DBG_MESSAGE("PhonebookGotoItem H\r\n");

//	if (field_num >= 5 && field_num <= 14)
//	{
//		app_status.company = TRUE;
//		app_status.home    = FALSE;
//		TableSetTopRowNum((ObjectID)(TABLE_VIEW_EDIT), row_num_array[field_num]);
//		FormGetObjectPointer((ObjectID)(row_num_array[field_num] + FIELD_ROW0), &object_type, (void**)&field_ptr);
//		highlight = TRUE;
//	}
//	else if (field_num >= 15 && field_num <= 25)
//	{	
//		app_status.home 	= TRUE;
//		app_status.company	= FALSE;
//		TableSetTopRowNum((ObjectID)(TABLE_VIEW_EDIT), row_num_array[field_num]);
//		FormGetObjectPointer((ObjectID)(row_num_array[field_num] + FIELD_ROW0), &object_type, (void**)&field_ptr);
//		highlight = TRUE;
//	}
//	else 
		highlight = FALSE;
	
	DBG_MESSAGE("PhonebookGotoItem I\r\n");
	
	PhonebookSetViewEditTable(FORM_VIEW_EDIT, TABLE_VIEW_EDIT);
    draw_view_edit_scrollbar = FALSE;
	PhonebookViewEditTableSetScrollbar(TABLE_VIEW_EDIT, SCROLLBAR_VIEW_EDIT);
    draw_view_edit_scrollbar = TRUE;

	if (highlight == FALSE)
	{
		PhonebookChangeFontSize(FORM_VIEW_EDIT, font_size);		
		FormPopupForm(FORM_VIEW_EDIT);
		return;
	}

/**	DBG_MESSAGE("PhonebookGotoItem J\r\n");
    highlight_char_pos = GlobalFindSearchText(search_string, field_ptr->field_string);
    
    
	total_num_lines = field_ptr->field_total_num_lines;
	for (line_num = 0; line_num < total_num_lines; line_num++)
	{
		if ((highlight_char_pos >= field_ptr->field_lineinfo[line_num].start) &&
			(highlight_char_pos <= (field_ptr->field_lineinfo[line_num].start +
			field_ptr->field_lineinfo[line_num].length -1)))
		{
			found = TRUE;
			if (line_num > (total_num_lines - field_ptr->field_num_lines_displayed))
				line_num = (total_num_lines - field_ptr->field_num_lines_displayed);
			field_ptr->field_top_line_num = line_num;
			break;
		}
	}

	DBG_MESSAGE("PhonebookGotoItem K\r\n");

	if (!found)
		field_ptr->field_top_line_num = 0;
	FormGetObjectPointer(SCROLLBAR_VIEW_EDIT, &object_type, (void**)&scroll_ptr);
	total_num_lines = 0;
	temp_event.para1 = scroll_ptr->scrollbar_value;
	for (count = FIELD_ROW0; count < field_ptr->identification.ui_object_id; count++)
	{
		FieldGetTotalNumOfLines((ObjectID)(count), &num_lines);
		total_num_lines += num_lines;
	}
	temp_event.para2 =	total_num_lines + field_ptr->field_top_line_num;
	DBG_MESSAGE("PhonebookGotoItem L\r\n");

	if (scroll_ptr->scrollbar_attr.scrollbar_visible == TRUE)
		PhonebookScrollbarSetViewEditTable(TABLE_VIEW_EDIT, SCROLLBAR_VIEW_EDIT, &temp_event);
	field_ptr->field_highlight_start_char = highlight_char_pos;
	field_ptr->field_highlight_end_char = highlight_char_pos + strlen(search_string) - 1;
	field_ptr->field_highlight_length = strlen(search_string);
	field_ptr->field_attr.field_highlight = TRUE;
	FormSetFormActiveObject(FORM_VIEW_EDIT, field_ptr->identification.ui_object_id);
	PhonebookChangeFontSize(FORM_VIEW_EDIT, font_size);
	FormPopupForm(FORM_VIEW_EDIT);**/
}

/********************************************************
 * Function:	PhonebookGetMappedRowNum
 * Purpose: 	This function is called to mapped row_num				
 * Scope:		application/internal
 * Input:		None
 * Output:		USHORT array
 * Return:		None
 * Comment: 	None
 *********************************************************/
void PhonebookGetMappedRowNum(SHORT *array)
{
	SHORT		row_count, row_up_to;
	
	array[0]	= -1;
	array[1]	= -1;
	array[2]	= -1;
	array[3]	= -1;
	array[4]	= -1;	

	row_up_to = 0;
	for (row_count = 5; row_count <= 9; row_count++)
	{
		if (strlen(field_buffers[row_count]))
		{
			array[row_count] = row_up_to;
			row_up_to ++;
		}
		else array[row_count] = -1;
	}	

	if (strlen(field_buffers[10]) || strlen(field_buffers[11]) ||
		strlen(field_buffers[12]) || strlen(field_buffers[13]) ||
		strlen(field_buffers[14]))	
		array[10] = array[11] = array[12] = array[13] = array[14] = row_up_to;
	else array[10] = array[11] = array[12] = array[13] = array[14] = -1;
		
	
	row_up_to = 0;
	for (row_count = 15; row_count <= 16; row_count++)
	{
		if (strlen(field_buffers[row_count]))
		{
			array[row_count] = row_up_to;
			row_up_to ++;
		}
		else array[row_count] = -1;
	}	

	if (strlen(field_buffers[17]) || strlen(field_buffers[18]) ||
		strlen(field_buffers[19]) || strlen(field_buffers[20]) ||
		strlen(field_buffers[21]))	
	{
		array[17] = array[18] = array[19] = array[20] = array[21] = row_up_to;
		row_up_to ++;
	}
	else array[17] = array[18] = array[19] = array[20] = array[21] = -1;

	for (row_count = 22; row_count <= 25; row_count++)
	{
		if (strlen(field_buffers[row_count]))
		{
			array[row_count] = row_up_to;
			row_up_to ++;
		}
		else array[row_count] = -1;
	}	
}
