/*
================================ COPYRIGHT ================================
The contents of this file are subject to the VTech Informations Ltd. License
of VT-OS Ver. 1.1 operating system (the "License"); you may not use this 
file except in compliance with the License.  
 
Software distributed under the License is distributed on an "AS IS" basis, 
WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
for the specific language governing rights and limitations under the License.
 
The Original Code is VT-OS Ver. 1.1 Operating System, released 
on October 1st, 1999

The Initial Developer of the Original Code is VTech Informations Ltd.  All 
codes are Copyright (C) VTech Informations Ltd. 1999.  All Rights Reserved.
===========================================================================
*/

/*
===========================================================================
File        :   main.c
Author(s)   :   Kenny Ng
Company     :   VTech Informations Ltd.
Project     :   Helio 
Date:	    :   October 1st, 1999
Purpose:	:   application c file
Revision    :   1.1
Note        :   None
===========================================================================
*/              




#include "stdafx.h"
#include "include\app.h"
#include "include\dev_pwr.h"
#include "include\system.h"
#include "include\lcddebug.h"

/* ==================================================================== */
/* ========================= Function Prototypes ====================== */
/* ==================================================================== */
void EventLoop();
static BOOLEAN ApplicationHandleEvent(EvtType *Event);

BOOLEAN (*FormDispatchEvent)(EvtType *Event);


/* ==================================================================== */
/* ========================= Global Defines =========================== */
/* ==================================================================== */

/* ==================================================================== */
/* ========================= Global Variables ========================= */
/* ==================================================================== */

/********************************************************
 * Function:	ApplicationHandleEvent
 * Purpose: 	This function is used to load the specific data of
				form and set event handler of the form
 * Scope:		application/internal
 * Input:		Event		received event
 * Output:		None
 * Return:		TRUE if handled
				FALSE if not handled
 * Comment: 	None
 *********************************************************/
static BOOLEAN ApplicationHandleEvent(EvtType *Event)
{
	Form *form_ptr;
	ObjectID form_id;
	BYTE object_type;
	Err Error;
        char buf[50];

	if (Event->eventType == EVT_FORM_LOAD)
	{
	  form_id = (ObjectID)Event->eventID;
	  Error = UISearchForAddress(form_id,&object_type,(void**)&form_ptr);
	  if (Event->para1 == 1 || Error != TRUE)
	  {
		FormInitForm(form_id);
	  }
	  if (!UISearchForAddress(form_id,&object_type,(void**)&form_ptr)) return FALSE;
	  switch (form_id)
          {
			case FORM_PC_SYNC:
                        FormSetEventHandler(FORM_PC_SYNC, (void**)&FormDispatchEvent, (void*) PCSyncMain);
                             break;
			case FORM_SYNC_LOG:
                        FormSetEventHandler(FORM_SYNC_LOG, (void**)&FormDispatchEvent, (void*) PCSyncLog);
                             break;
	  }
	  FormSetActiveForm(form_id);
	  return TRUE;
	}
	return FALSE;
}

/********************************************************
 * Function:	EventLoop
 * Purpose: 	This function is a loop to get events from event
				manager and then passes the event to different
				event handler
 * Scope:		application
 * Input:		None
 * Output:		None
 * Return:		None
 * Comment: 	None
 *********************************************************/
void EventLoop(void)
{
	EvtType event;
	do
	{
	  EvtGetEvent (&event);
	  if (!SystemHandleEvent(&event))
          {
		if (!MenuHandleEvent(&event))
                {
		  if (!ApplicationHandleEvent(&event))
                  {
			FormDispatchEvent(&event);
                  }
                }
          }
          }
		#ifdef PC_SIM
		while (event.eventType != EVT_APP_STOP && m_bIsExit == FALSE);
		#else
		while (event.eventType != EVT_APP_STOP);
		#endif
}

#ifdef xxxxxxxx
void Test()
{
    DatabaseID dbid;
    RecordID recid;
    UWORD fs;
    int i;
    char *buf;

    SysEnableDebug();

    buf = (char*) qmalloc(33000);

    DataNewDB((BYTE*)"asdf", 0, (BYTE*)"asdf", &dbid);
    DataOpenDB(dbid, 0, OPEN_RW);
    DataNewRecord(dbid, 0, 4, &recid);

    DataWriteField(dbid, recid, 0, 1, (BYTE*) buf);
    DataWriteField(dbid, recid, 1, 20, (BYTE*)buf);
    DataWriteField(dbid, recid, 2, 2001, (BYTE*)buf);
    DataWriteField(dbid, recid, 3, 2440, (BYTE*)buf);


    DataNewRecord(dbid, 0, 4, &recid);
    DataWriteField(dbid, recid, 0, 1, (BYTE*) buf);
    DataWriteField(dbid, recid, 1, 20, (BYTE*)buf);
    DataWriteField(dbid, recid, 2, 21001, (BYTE*)buf);
    DataWriteField(dbid, recid, 2, 25201, (BYTE*)buf);
    DataWriteField(dbid, recid, 3, 2440, (BYTE*)buf);


    DataNewRecord(dbid, 0, 4, &recid);
    DataWriteField(dbid, recid, 0, 1, (BYTE*) buf);
    DataWriteField(dbid, recid, 1, 20, (BYTE*)buf);
    DataWriteField(dbid, recid, 2, 7000, (BYTE*)buf);
    DataWriteField(dbid, recid, 3, 2440, (BYTE*)buf);


    DataNewRecord(dbid, 0, 4, &recid);
    DataWriteField(dbid, recid, 0, 1, (BYTE*) buf);
    DataWriteField(dbid, recid, 1, 20, (BYTE*)buf);
    DataWriteField(dbid, recid, 2, 7000, (BYTE*)buf);


    for(i=0;i<4;i++)
    {
        DataFieldSize(dbid, recid, i, &fs);
        printf("\nFS: %d %d ", i, fs);
    }
    DataCloseRecord(dbid, recid);
    DataCloseDB(dbid);

    DataOpenDB(dbid, 0, OPEN_RW);
    DataTotalRecord(dbid, &fs);

    printf("\nTotalRec %d ", fs);

    while(1);
}

#endif

/********************************************************
 * Function:	main
 * Purpose: 	main procedure of the application
 * Scope:		Application
 * Input:		None
 * Output:		None
 * Return:		None
 * Comment: 	None
 *********************************************************/
__main(WORD cmd, void *cmd_ptr)
{
//    Test();
    ClearUartBuffer(UART_A);
    SyncAppLaunch( cmd,cmd_ptr);
    SySetupRestoreSettingsAfterSync();    
//    SysEnableDebug();
//    UartInit(UART_A, UART_BAUD115200, UART_DATABIT8|UART_PARITY_NONE|UART_STOPBIT1);
}

