/*
================================ COPYRIGHT ================================
The contents of this file are subject to the VTech Informations Ltd. License
of VT-OS Ver. 1.1 operating system (the "License"); you may not use this 
file except in compliance with the License.  
 
Software distributed under the License is distributed on an "AS IS" basis, 
WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
for the specific language governing rights and limitations under the License.
 
The Original Code is VT-OS Ver. 1.1 Operating System, released 
on October 1st, 1999

The Initial Developer of the Original Code is VTech Informations Ltd.  All 
codes are Copyright (C) VTech Informations Ltd. 1999.  All Rights Reserved.
===========================================================================
*/

/*
===========================================================================
File        :   aplaunch.c
Author(s)   :   Henry Fok
Company     :   VTech Informations Ltd.
Project     :   Helio 
Date:	    :   October 1st, 1999
Purpose:	:   application c file
Revision    :   1.1
Note        :   None
===========================================================================
*/              

#include "stdafx.h"
#include "include\app.h"
#include "include\appfunc.h"

/********************************************************
 * Function:	SchedulerAppLaunch
 * Purpose: 	This function is called to launch the 
				Scheduler application
 * Scope:		application/internal
 * Input:		None
 * Output:		None
 * Return:		TRUE if handled
				FALSE if not handled
 * Comment: 	None
 *********************************************************/
BOOLEAN SchedulerAppLaunch(WORD cmd, void *cmd_ptr)
{
	
	UWORD			num_rec, count, byte_read;
	RecordID		rec_id;
	BYTE			*buffer, *prepare_string, **string;
	WORD			num_appmt, appmt_num;
    UWORD           total_rec;
	BYTE			output;
	BOOLEAN			start_deleted, end_deleted;
	USHORT			start_row, end_row;
	ObjectID        active_form_id;

    SchedulerDatabaseChecking();

    if (cmd == LAUNCH_CMD_GOTO_REC && DataRecordInfo(sch_appmt_dbid, ((GotoRec*)cmd_ptr)->rec_id, NULL, NULL, NULL, NULL, NULL) != TRUE)
	{
		if (((GotoRec*)cmd_ptr)->find_string != NULL)
			pfree(((GotoRec*)cmd_ptr)->find_string);
		pfree(cmd_ptr); 
		cmd = LAUNCH_CMD_NORMAL_LAUNCH;
    }

	switch(cmd)
	{
		case LAUNCH_CMD_TODAY_SCH:
        case LAUNCH_CMD_NORMAL_LAUNCH:
            virtual_table.vt_num_rows = 0;
			UIApplicationInit();
			
			if (cmd == LAUNCH_CMD_TODAY_SCH)
            	SchedulerAppRestore(FALSE, TRUE, cmd_ptr);
			else 
				SchedulerAppRestore(FALSE, FALSE, cmd_ptr);            	
			EventLoop();
			

/* =========================================================================================== */
            FormGetActiveFormID(&active_form_id);
            if (active_form_id == FORM_SCH_DAY_VIEW && virtual_table.vt_active_row != -1)
            {
                output = SchedulerDayViewNormal(&virtual_table, TABLE_SCH_DAY_VIEW, &start_row,
                                                &end_row, &start_deleted, &end_deleted);
				if (virtual_table.vt_active_row != -1 && new_appointment == TRUE)
				{
					new_appointment = FALSE;
					virtual_table.vt_items[virtual_table.vt_active_row]->appointment = -1;
				}
				virtual_table.vt_active_row = -1;
                SchedulerTimeSettingsDeleteAllEmptyAppmts(&virtual_table);
                SchedulerTimeSettingsAddAppmts(&virtual_table);
			}
			
/* =========================================================================================== */		
			
			SchedulerAppSave();
            UIDeleteAllAppObjects();
            DataCloseDB(sch_appmt_dbid);
            DataCloseDB(sch_voice_dbid);
    		return TRUE;
		case LAUNCH_CMD_FIND:
			app_id = SysGetActiveAppID();
			/* sch_appmt_dbid */
			DataTotalRecord(sch_appmt_dbid, &num_rec);
			for (count = 0; count < num_rec; count++)
			{
				DataOpenRecord(sch_appmt_dbid, count, &rec_id, NULL);
				DataGetField(sch_appmt_dbid, rec_id, 2, &buffer, &byte_read);
				SchedulerDBGetAppmtString(buffer, byte_read, &string, &num_appmt);
				qfree(buffer);
				for (appmt_num = 0; appmt_num < num_appmt; appmt_num++)
				{
					if (GlobalFindSearchText((BYTE*)cmd_ptr, string[appmt_num]) >= 0)
					{
						SchedulerPrepareFindText(sch_appmt_dbid, rec_id, appmt_num, &prepare_string);
#ifdef PR31700
                        GlobalFindAddItem(app_id, sch_appmt_dbid, rec_id, (USHORT)appmt_num, prepare_string, FALSE);
#endif
                        qfree(buffer);
						pfree(prepare_string);
						break;
					}
				}
				if (num_appmt != 0)
				{
					for (appmt_num = 0; appmt_num < num_appmt; appmt_num++)
						qfree(string[appmt_num]);
					qfree(string);
				}
				DataCloseRecord(sch_appmt_dbid, rec_id);
			}
			pfree(cmd_ptr);
			return TRUE;
		case LAUNCH_CMD_GOTO_REC:
            virtual_table.vt_num_rows = 0;
            SchedulerGotoItem(sch_appmt_dbid, ((GotoRec*)cmd_ptr)->rec_id,
              ((GotoRec*)cmd_ptr)->field_num, ((GotoRec*)cmd_ptr)->find_string);

			if (((GotoRec*)cmd_ptr)->find_string != NULL)
				pfree(((GotoRec*)cmd_ptr)->find_string);
			pfree(cmd_ptr);
			EventLoop();
			
/* =========================================================================================== */
            FormGetActiveFormID(&active_form_id);
            if (active_form_id == FORM_SCH_DAY_VIEW && virtual_table.vt_active_row != -1)
            {
                output = SchedulerDayViewNormal(&virtual_table, TABLE_SCH_DAY_VIEW, &start_row,
                                                &end_row, &start_deleted, &end_deleted);
				if (virtual_table.vt_active_row != -1 && new_appointment == TRUE)
				{
					new_appointment = FALSE;
					virtual_table.vt_items[virtual_table.vt_active_row]->appointment = -1;
				}
				virtual_table.vt_active_row = -1;
                SchedulerTimeSettingsDeleteAllEmptyAppmts(&virtual_table);
                SchedulerTimeSettingsAddAppmts(&virtual_table);
			}
			
/* =========================================================================================== */		
			
			SchedulerAppSave();
			UIDeleteAllAppObjects();
            DataCloseDB(sch_appmt_dbid);
            DataCloseDB(sch_voice_dbid);
     		return TRUE;
		case LAUNCH_CMD_ALARM_HIT:
            SchedulerManageAlarm((AlarmEventHit*)cmd_ptr);
			pfree(cmd_ptr);
            DataCloseDB(sch_appmt_dbid);
            DataCloseDB(sch_voice_dbid);
			return TRUE;
		default:
			return FALSE;
	}
	return FALSE;
}

/********************************************************
 * Function:	SchedulerPrepareFindText
 * Purpose: 	This function is called to prepare the find text to 
				be displayed in Global Find
 * Scope:		application/internal
 * Input:		dbid
				rec_id
				prepare_string
 * Output:		None
 * Return:		None
 * Comment: 	None
 *********************************************************/
void SchedulerPrepareFindText(DatabaseID dbid, RecordID rec_id, WORD appmt_num, BYTE **prepare_string)
{
	
	/* sch_appmt_dbid */
	SHORT			string_text_width, string_time_width;
	ObjectBounds	text_bounds, time_bounds;
	BYTE			*text, date_time[20], *buffer, *temp, sbuffer[13], time[10], count, length_sbuffer;
	UWORD			byte_read, name_num_chars;
	WORD			num_appmt;
	BYTE			**string;
	AppointmentDB	*appmt_rec;

	text_bounds.xcoord	= 0;
	text_bounds.ycoord	= 0;
	text_bounds.width	= 60;
	text_bounds.height	= 9;

	time_bounds.xcoord	= 0;
	time_bounds.ycoord	= 0;
	time_bounds.width	= 74;
	time_bounds.height	= 9;

	*prepare_string = (BYTE*)pmalloc(200*sizeof(BYTE));
	
	DataGetField(dbid, rec_id, 1, &buffer, &byte_read);
	SchedulerDBGetAppmtRec(buffer, byte_read, &appmt_rec, &num_appmt);
	qfree(buffer);
	DataGetField(dbid, rec_id, 2, &buffer, &byte_read);
	SchedulerDBGetAppmtString(buffer, byte_read, &string, &num_appmt);
	qfree(buffer);

	text = (BYTE*)pmalloc((strlen(string[appmt_num]) + 10) * sizeof(BYTE));
	strcpy(text, string[appmt_num]);
	StrChopString(&text_bounds, text, SMALL_FONT, TRUE, 0);
	string_text_width = StrGetWidth(text, SMALL_FONT);
	
	DataGetField(dbid, rec_id, 0, &buffer, &byte_read);
	SchedulerCalculateDayViewDate((SHORT)(buffer[0] + SYSETUP_YEAR_OFFSET), (SHORT)(buffer[1]), (SHORT)(buffer[2]), sbuffer);	
	qfree(buffer);
	SchedulerHourMinuteToString(appmt_rec[appmt_num].hour, appmt_rec[appmt_num].minute, time, TRUE);

	length_sbuffer = strlen(sbuffer);
	strcpy(date_time, sbuffer);
	date_time[length_sbuffer] = 32;
    for (count = 0; count <= (BYTE)(strlen(time)); count++)
		date_time[length_sbuffer + count + 1] = time[count];

	temp = (BYTE*)pmalloc((strlen(date_time) + 10) * sizeof(BYTE));
	strcpy(temp, date_time);
	StrChopString(&time_bounds, temp, SMALL_FONT, TRUE, 0);
	string_time_width = StrGetWidth(temp, SMALL_FONT);

	strcpy(*prepare_string, text);
	name_num_chars = strlen(text);
	(*prepare_string)[name_num_chars] = 11;
	(*prepare_string)[name_num_chars + 1] = 139 - string_time_width - string_text_width;
	sprintf((char*)(*prepare_string + name_num_chars + 2), "%s", temp);

	if (num_appmt != 0)
	{
		for (appmt_num = 0; appmt_num < num_appmt; appmt_num++)
			qfree(string[appmt_num]);
		qfree(string);
	}
	qfree(appmt_rec);
    pfree(temp);
    pfree(text);
}


/********************************************************
 * Function:	SchedulerGotoItem
 * Purpose: 	This function is called to init all related variables
				and goto the specific form to display the required 
				record
 * Scope:		application/internal
 * Input:		dbid
				rec_id
				field_num
				search_string
 * Output:		None
 * Return:		None
 * Comment: 	None
 *********************************************************/
void SchedulerGotoItem(DatabaseID dbid, RecordID rec_id, USHORT field_num, BYTE *search_string)
{
	BYTE			*buffer, object_type;
	UWORD			byte_read, rec_num;
	BYTE			**string;
	UBYTE			date[4];
	SHORT			wday, nth_week;
	WORD			highlight_char_pos, line_num, total_num_lines, num_lines, num_appmt;
	Field			*field_ptr;
	CountrySettings country;
	EvtType			evt;
	Scrollbar		*scroll_ptr;
	USHORT			row_to_be_drawn;
	BOOLEAN			found = FALSE;
	WORD			count;
	WORD			search_string_len;
	Form			*form_ptr;


	SchedulerDatabaseChecking();
	SySetupGetCountrySettings(&country);
	DataRecIDtoNum(dbid, rec_id, &rec_num);
	DataOpenRecord(dbid, rec_num, &rec_id, NULL);

	/* global variables initialisation */
	new_appointment = FALSE;
	day_view_old_event.eventType = 0;
	SchInitMainGlobal();
	
	DataGetField(sch_appmt_dbid, SCH_STATUS, 2, &buffer, &byte_read);
	alarm_count = *((UWORD*)buffer);
	qfree(buffer);

	DataGetField(sch_appmt_dbid, SCH_STATUS, 3, &buffer, &byte_read);
	current_appmt_num = *((UWORD*)buffer);
	qfree(buffer);
	

	FormInitAllFormObjects(FORM_SCH_DAY_VIEW);
	FormInitAllFormObjects(FORM_SCH_TIME_SETTINGS);
	FormInitAllFormObjects(FORM_SCH_ALARM);
	FormInitAllFormObjects(FORM_SCH_REC);
	FormInitAllFormObjects(FORM_SCH_REPLAY);
	FormInitAllFormObjects(FORM_SCH_DATE);
	FormInitAllFormObjects(FORM_SCH_WEEK);
	FormInitAllFormObjects(FORM_SCH_MONTH);

    DataGetField(dbid, rec_id, F_DATE, &buffer, &byte_read);
	date[0] = buffer[0];
	date[1] = buffer[1];
	date[2] = buffer[2];
	date[4] = '\0';
	qfree(buffer);
//kkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkk
	//RtcYMDToWday1((SHORT)(date[0] + SYSETUP_YEAR_OFFSET), (SHORT)(date[1] - 1), (SHORT)(date[2]), &wday, &nth_week);
	wday = Dt_Date_WDay((SHORT)(date[0] + SYSETUP_YEAR_OFFSET), (SHORT)(date[1] - 1), (SHORT)(date[2]));
//kkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkk
	
	if (country.start_of_week == SYSETUP_MON)
	{
		if (wday == 0) wday = 6;
			else wday --;
	}
	SchedulerSetDayViewDate((SHORT)(date[0] + SYSETUP_YEAR_OFFSET), (SHORT)(date[1]), (SHORT)(date[2]), wday);
	SchedulerDayViewLoadAppmtDataToScreen(&virtual_table, date);
	if (search_string != NULL)
	{
		search_string_len = strlen(search_string_len);
		FormGetObjectPointer(FORM_SCH_DAY_VIEW, &object_type, (void**)&form_ptr);
		VTSetVirtualTableLayout(&virtual_table);
		VTVirtualTableSetScrollbar(&virtual_table);
		DataOpenRecord(dbid, rec_num, &rec_id, NULL);
		DataGetField(dbid, rec_id, 2, &buffer, &byte_read);
		SchedulerDBGetAppmtString(buffer, byte_read, &string, &num_appmt);
		qfree(buffer);
		highlight_char_pos = GlobalFindSearchText(search_string, string[field_num]);
		field_ptr = virtual_table.vt_items[field_num]->ptr;
		for (line_num = 0; line_num < field_ptr->field_total_num_lines; line_num++)
		{
			if ((highlight_char_pos >= field_ptr->field_lineinfo[line_num].start) &&
				(highlight_char_pos <= (field_ptr->field_lineinfo[line_num].start +
				field_ptr->field_lineinfo[line_num].length -1)))
			{
				found = TRUE;
				if (line_num > (field_ptr->field_total_num_lines - field_ptr->field_num_lines_displayed))
					line_num = (field_ptr->field_total_num_lines - field_ptr->field_num_lines_displayed);
				field_ptr->field_top_line_num = line_num;
				break;
			}
		}
		if (!found)
			field_ptr->field_top_line_num = 0;
		FormGetObjectPointer(SCROLLBAR_SCH_DAY_VIEW, &object_type, (void**)&scroll_ptr);
		evt.para1 = scroll_ptr->scrollbar_value;
		total_num_lines = 0;
		for (count = 0; count < field_num; count++)
		{
			VTGetFieldTotalNumLines(&virtual_table, (USHORT)count, &num_lines);
			total_num_lines += num_lines;
		}
		evt.para2 =	total_num_lines + field_ptr->field_top_line_num;
		VTScrollbarSetVirtualTable(&virtual_table, &evt, &row_to_be_drawn);
		field_ptr->field_highlight_start_char = highlight_char_pos;
		field_ptr->field_highlight_end_char = highlight_char_pos + search_string_len - 1;
		field_ptr->field_highlight_length = search_string_len;
		field_ptr->field_attr.field_highlight = TRUE;
		//virtual_table.vt_active_row = -1;
        if (found)
        {
			for (count = 0; count < virtual_table.vt_num_rows; count++)
			{
				virtual_table.vt_items[count]->ptr->bounds.height = 9;
				virtual_table.vt_items[count]->ptr->screen_bounds.height = 9;
				virtual_table.vt_row_height[count] = 9; 
			}

			VTSetVirtualTableLayout(&virtual_table);
			VTVirtualTableSetScrollbar(&virtual_table);

            form_ptr->form_focus = field_num - virtual_table.vt_top_row_num + FIELD_SCH_DAY_VIEW_0;
            virtual_table.vt_active_row = field_num;
        }
        else
            virtual_table.vt_active_row = -1;		
	}
	FormPopupForm(FORM_SCH_DAY_VIEW);
	DataCloseRecord(dbid, rec_id);
}


/********************************************************
 * Function:	SchedulerManageAlarm
 * Purpose: 	This function is called to init all related variables
				and goto the specific form to display the required 
				record
 * Scope:		application/internal
 * Input:		dbid
				rec_id
				field_num
				search_string
 * Output:		None
 * Return:		None
 * Comment: 	None
 *********************************************************/
void SchedulerManageAlarm(AlarmEventHit *ptr)
{
	USHORT				row_num;
	RecordID			rec_id, temp_rec_id;
    AppointmentDB       *appmt_rec;
	BYTE				*buffer, **string, *appmt_rec_buf, *string_buf;
	UWORD				byte_read;
	WORD				num_appmt, count, num_bytes_appmt, num_bytes_string;
	BYTE				alarm_found = 0, date[4];
    UWORD               rec_num, count1, total_rec, reference;
	AlarmEvent			evtAlarm;
    BOOLEAN             found = FALSE;

    evtAlarm.type = 0;
    reference = ptr->reference;
	rec_id	= ptr->rec_id;

    DataRecIDtoNum(sch_appmt_dbid, rec_id, &rec_num);
    if (DataOpenRecord(sch_appmt_dbid, rec_num, &rec_id, NULL) == TRUE)
    {
        DataGetField(sch_appmt_dbid, rec_id, 1, &buffer, &byte_read);
        SchedulerDBGetAppmtRec(buffer, byte_read, &appmt_rec, &num_appmt);
        qfree(buffer);

        DataGetField(sch_appmt_dbid, rec_id, 2, &buffer, &byte_read);
        SchedulerDBGetAppmtString(buffer, byte_read, &string, &num_appmt);
        qfree(buffer);

        for (count1 = 0; count1 < num_appmt; count1++)
        {
            if (appmt_rec[count1].alarm == VOICE_ALARM ||
                appmt_rec[count1].alarm == TONE_ALARM)
            {
                if (appmt_rec[count1].alarm_num == reference)
                {
                    row_num = count1;
                    found = TRUE;
                    break;
                }
            }
        }

        if (found)
        {
            if (appmt_rec[row_num].alarm == VOICE_ALARM)
                appmt_rec[row_num].alarm = VOICE_NOT_ALARM;
            else appmt_rec[row_num].alarm = NO_ALARM;
        
            if (strlen(string[row_num]) == 0 && appmt_rec[row_num].alarm == NO_ALARM)
            {    
                appmt_rec[row_num].appointment = -1;
                appmt_rec[row_num].appmt_num   = 0;
			}
			else
				appmt_rec[row_num].appmt_num |= 0x80000000;

            for (count = 0; count < num_appmt; count ++)
            {
                if (appmt_rec[count].alarm != NO_ALARM &&
                    appmt_rec[count].alarm != VOICE_NOT_ALARM)
                {
                    alarm_found = 1;         
                    break;
                }
            }

            DataWriteField(sch_appmt_dbid, rec_id, 3, 1, (BYTE*)&alarm_found);
            SchedulerDBAppmtToBuffer(appmt_rec, string, num_appmt, 
                                     &appmt_rec_buf, &string_buf, &num_bytes_appmt,
                                     &num_bytes_string);
            DataWriteField(sch_appmt_dbid, rec_id, 1, num_bytes_appmt, appmt_rec_buf);
            DataWriteField(sch_appmt_dbid, rec_id, 2, num_bytes_string, string_buf);
            qfree(appmt_rec_buf);
            qfree(string_buf);

            if (num_appmt != 0)
            {
                for (count = 0; count < num_appmt; count++)
                    qfree(string[count]);
                qfree(string);
            }
            qfree(appmt_rec);
        } 
    }
    DataTotalRecord(sch_appmt_dbid, &total_rec);
	if (DataRecIDtoNum(sch_appmt_dbid, rec_id, &rec_num) != TRUE)
	    rec_num = 0;
 
	for (count1 = rec_num; count1 < total_rec; count1++)
	{
		DataOpenRecord(sch_appmt_dbid, count1, &temp_rec_id, NULL);
        DataGetField(sch_appmt_dbid, temp_rec_id, F_DATE, &buffer, &byte_read);
		if (SchedulerCheckEarliestDayWithAlarm(buffer) == TRUE)
		{
			date[0] = buffer[0];
			date[1] = buffer[1];
			date[2] = buffer[2];
			SchedulerSetAppmtWholeDayAlarm(date);
			qfree(buffer);
			DataCloseRecord(sch_appmt_dbid, temp_rec_id);
			break;
		}
		qfree(buffer);
		DataCloseRecord(sch_appmt_dbid, temp_rec_id);
	}
}
