/*
================================ COPYRIGHT ================================
The contents of this file are subject to the VTech Informations Ltd. License
of VT-OS Ver. 1.1 operating system (the "License"); you may not use this 
file except in compliance with the License.  
 
Software distributed under the License is distributed on an "AS IS" basis, 
WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
for the specific language governing rights and limitations under the License.
 
The Original Code is VT-OS Ver. 1.1 Operating System, released 
on October 1st, 1999

The Initial Developer of the Original Code is VTech Informations Ltd.  All 
codes are Copyright (C) VTech Informations Ltd. 1999.  All Rights Reserved.
===========================================================================
*/

/*
===========================================================================
File        :   schalarm.c
Author(s)   :   Henry Fok
Company     :   VTech Informations Ltd.
Project     :   Helio 
Date:	    :   October 1st, 1999
Purpose:	:   application c file
Revision    :   1.1
Note        :   None
===========================================================================
*/              


#include "stdafx.h"
#include "include\app.h"
#include "include\appfunc.h"
#include "include\date.h"


/********************************************************
 * Function:	SchedulerAlarmSave
 * Purpose: 	This function is called to save the set alarm settings
				to the record 
 * Scope:		application/internal
 * Input:		None
 * Output:		None
 * Return:		None
 * Comment: 	None
 *********************************************************/
void SchedulerAlarmSave()
{
	UBYTE date[4];
	AlarmEvent evtAlarm;
	RecordID rec_id;
	USHORT row_num;
	WORD num_appmt;
	AppointmentDB *appmt_rec;
	BYTE *buffer;
	UWORD byte_read;
	UWORD rec_num;
	UWORD 	count;

    evtAlarm.type = 0;
	if (alarm_settings.change == FALSE)
		return;

	SchedulerGetDayViewDate(date);

	if (time_settings.alarm == TONE_ALARM || time_settings.alarm == VOICE_ALARM)
	{
		if (alarm_settings.alarm == NO_ALARM || alarm_settings.alarm == VOICE_ALARM)
		{
			DataFindBinRecord(sch_appmt_dbid, F_DATE, 0, date, 3, &rec_id);
			evtAlarm.app = SysGetActiveAppID();
			AlarmSetEventClass((&evtAlarm), ALARM_ONTIME);
            evtAlarm.type = 0;
			evtAlarm.reference = time_settings.alarm_num;
			evtAlarm.rec_id = rec_id;
			AlarmDelEvent(evtAlarm);
		}
	}

	if (alarm_settings.recorded == TRUE)
	{
        if (alarm_settings.alarm != VOICE_ALARM &&
            alarm_settings.alarm != VOICE_NOT_ALARM)
		{
			DataCloseRecord(alarm_settings.dbid, alarm_settings.temp_recid);
			DataDeleteRecord(alarm_settings.dbid, alarm_settings.temp_recid, FALSE);
		}
		else
		{
            if (time_settings.alarm == VOICE_ALARM ||
                time_settings.alarm == VOICE_NOT_ALARM)
			{
                DataRecIDtoNum(sch_voice_dbid, time_settings.recid, &rec_num);
                DataOpenRecord(sch_voice_dbid, rec_num, &time_settings.recid, NULL);
				DataGetField(sch_voice_dbid, time_settings.recid, 2, &buffer, &byte_read);
                DataCloseRecord(time_settings.dbid, time_settings.recid);
				count = *((UWORD*)buffer);
				qfree(buffer);
                
				if (count == 1)						
				{				
					DataCloseRecord(time_settings.dbid, time_settings.recid);
					DataDeleteRecord(time_settings.dbid, time_settings.recid, FALSE);
				}
			}
			time_settings.dbid = alarm_settings.dbid;
			time_settings.recid = alarm_settings.temp_recid;
		}
	}
	else
	{
        if (time_settings.alarm == VOICE_ALARM ||
            time_settings.alarm == VOICE_NOT_ALARM)
		{
            DataRecIDtoNum(sch_voice_dbid, time_settings.recid, &rec_num);
            DataOpenRecord(sch_voice_dbid, rec_num, &time_settings.recid, NULL);
			DataGetField(sch_voice_dbid, time_settings.recid, 2, &buffer, &byte_read);
            DataCloseRecord(time_settings.dbid, time_settings.recid);
			count = *((UWORD*)buffer);
			qfree(buffer);
			if (count == 1)						
			{				
				DataCloseRecord(time_settings.dbid, time_settings.recid);
				DataDeleteRecord(time_settings.dbid, time_settings.recid, FALSE);
			}
		}
	}

	time_settings.alarm = alarm_settings.alarm;
	time_settings.alarm_time = alarm_settings.alarm_time;
	time_settings.alarm_num = alarm_count;
	alarm_count ++;

	virtual_table.vt_items[time_settings.row_num]->alarm = time_settings.alarm;
	virtual_table.vt_items[time_settings.row_num]->alarm_time = time_settings.alarm_time;
	virtual_table.vt_items[time_settings.row_num]->alarm_num = time_settings.alarm_num;
	virtual_table.vt_items[time_settings.row_num]->appmt_num = (virtual_table.vt_items[time_settings.row_num]->appmt_num)|0x80000000;
	virtual_table.vt_items[time_settings.row_num]->dbid = time_settings.dbid;
	virtual_table.vt_items[time_settings.row_num]->recid = time_settings.recid;

	SchedulerGetDayViewDate(date);
	if (new_appointment == TRUE && time_settings.alarm != NO_ALARM)
		new_appointment = FALSE;

	SchedulerDayViewSaveAppmtData(&virtual_table, date);

	if (SchedulerCheckEarliestDayWithAlarm(date) == TRUE)
		SchedulerSetAppmtWholeDayAlarm(date);
}

/********************************************************
 * Function:	SchedulerSetAppmtAlarmOn
 * Purpose: 	This function is called to 
				- set or reset an alarm of a particular appointment in one day
 * Scope:		application/internal
 * Input:		alarm_type
				alarm_time
				*date
				hour
				minute
				appmt_recid
				voice_recid
				row_num
 * Output:		None
 * Return:		None
 * Comment: 	None
 *********************************************************/
void SchedulerSetAppmtAlarmOn(BYTE alarm_type, BYTE alarm_time, UWORD alarm_num, UBYTE *date,
							  BYTE hour, BYTE minute, RecordID appmt_recid,
							  RecordID voice_recid, USHORT row_num)
{
	AlarmEvent evtAlarm;
	SHORT year, mon, mday;
	USHORT char_count = 0;
	BYTE *appmt_msg;
	BYTE *buffer, **string;
	UWORD byte_read;
	RecordID rec_id;
	WORD num_appmt, count;
	UWORD rec_num;

	evtAlarm.hit_time.msec	= 0;
	evtAlarm.hit_time.sec	= 0;
    evtAlarm.type = 0;
	
	if (alarm_time == MIN_15_BEFORE)
		minute -= 15;
	else if (alarm_time == MIN_30_BEFORE)
		minute -= 30;
	else if (alarm_time == MIN_60_BEFORE)
		minute -= 60;

	if (minute < 0)
	{
		minute += 60;
		hour --;
	}

	if (hour < 0)
	{
		evtAlarm.hit_time.min 	= 0;
		evtAlarm.hit_time.hour	= 0;
	}
	else 	
	{
		evtAlarm.hit_time.hour	 = (SHORT)hour;
		evtAlarm.hit_time.min	 = (SHORT)minute;
	}

	year	= (SHORT)(date[0] + SYSETUP_YEAR_OFFSET);
    mon     = (SHORT)(date[1] - 1);
	mday	= (SHORT)(date[2]);
	
	evtAlarm.hit_time.mday	= mday;
	evtAlarm.hit_time.mon	= mon;
	evtAlarm.hit_time.year	= year;
	evtAlarm.hit_time.wday	= 0;
	evtAlarm.hit_time.yday	= 0;	

	evtAlarm.app = SysGetActiveAppID();
	AlarmSetEventClass((&evtAlarm), ALARM_ONTIME);
    evtAlarm.type = 0;
	evtAlarm.reference = alarm_num;
	evtAlarm.rec_id = appmt_recid;

	DataFindBinRecord(sch_appmt_dbid, F_DATE, 0, date, 3, &rec_id);
	DataRecIDtoNum(sch_appmt_dbid, rec_id, &rec_num);
	DataOpenRecord(sch_appmt_dbid, rec_num, &rec_id, NULL);
	DataGetField(sch_appmt_dbid, rec_id, F_STRING, &buffer, &byte_read);	
	SchedulerDBGetAppmtString(buffer, byte_read, &string, &num_appmt);
	qfree(buffer);
	appmt_msg = string[row_num];

	evtAlarm.alert_msg = (BYTE*)qmalloc(201*sizeof(BYTE));
	
	while (char_count < 200 && appmt_msg[char_count] != '\0')
	{
		evtAlarm.alert_msg[char_count] = appmt_msg[char_count];
		char_count ++;
	}
	evtAlarm.alert_msg[char_count] = '\0';

	if (alarm_type == VOICE_ALARM)
	{
		(evtAlarm.extra_info) = (AlarmExtraInfo *)qmalloc(sizeof(AlarmExtraInfo));
		(evtAlarm.extra_info)->type = TYPE_VOICE;
		(evtAlarm.extra_info)->dbid = sch_voice_dbid;
		(evtAlarm.extra_info)->rec_id = voice_recid;
		(evtAlarm.extra_info)->field_num = 0;
	}
	else evtAlarm.extra_info = NULL;

	AlarmDelEvent(evtAlarm);
	AlarmAddEvent(evtAlarm);

	qfree(evtAlarm.alert_msg);
	if (alarm_type == VOICE_ALARM)
		qfree(evtAlarm.extra_info);
	for (count = 0; count < num_appmt; count++)
		qfree(string[count]);
	if (num_appmt != 0) qfree(string);

}

/********************************************************
 * Function:	SchedulerSetAppmtWholeDayAlarm
 * Purpose: 	This function is called to 
				- set or reset alarms of appointments of a day
 * Scope:		application/internal
 * Input:		date				
 * Output:		None
 * Return:		None
 * Comment: 	None
 *********************************************************/
void SchedulerSetAppmtWholeDayAlarm(UBYTE *date)
{
	AlarmEvent evtAlarm;
	USHORT row_num = 0;
	BYTE *buffer, **string;
	WORD num_appmt;
	UWORD byte_read;
	AppointmentDB *appmt_rec;
	RecordID rec_id;
	UWORD rec_num;

    evtAlarm.type = 0;
	if (DataFindBinRecord(sch_appmt_dbid, F_DATE, 0, date, 3, &rec_id) != TRUE)
		return;
	DataRecIDtoNum(sch_appmt_dbid, rec_id, &rec_num);
	DataOpenRecord(sch_appmt_dbid, rec_num, &rec_id, NULL);
	DataGetField(sch_appmt_dbid, rec_id, F_DATA, &buffer, &byte_read);
	SchedulerDBGetAppmtRec(buffer, byte_read, &appmt_rec, &num_appmt);
	qfree(buffer);
	DataGetField(sch_appmt_dbid, rec_id, F_STRING, &buffer, &byte_read);
	SchedulerDBGetAppmtString(buffer, byte_read, &string, &num_appmt);
	qfree(buffer);
	DataCloseRecord(sch_appmt_dbid, rec_id);

	AlarmDelAppEvent(SysGetActiveAppID());

	for (row_num = 0; row_num < num_appmt; row_num++)
	{
        if (appmt_rec[row_num].alarm != NO_ALARM &&
            appmt_rec[row_num].alarm != VOICE_NOT_ALARM)
		{
			DataOpenRecord(sch_appmt_dbid, rec_num, &rec_id, NULL);
			SchedulerSetAppmtAlarmOn(appmt_rec[row_num].alarm, appmt_rec[row_num].alarm_time, appmt_rec[row_num].alarm_num,
									 date, appmt_rec[row_num].hour, appmt_rec[row_num].minute,
									 rec_id, appmt_rec[row_num].recid, row_num);
			DataCloseRecord(sch_appmt_dbid, rec_id);
		}
	}
	if (num_appmt != 0) qfree(appmt_rec);	
	for (row_num = 0; row_num < num_appmt; row_num++)
		qfree(string[row_num]);
	if (num_appmt != 0) qfree(string);
}

/********************************************************
 * Function:	SchedulerCheckWholeDayAlarmOff
 * Purpose: 	this function is to check whether the date of the 
				input record id has alarm or not
 * Scope:		application/internal
 * Input:		rec_id
 * Output:		None
 * Return:		TRUE		no alarm
 				FALSE		there is at least one alarm
 * Comment: 	None
 *********************************************************/
BOOLEAN SchedulerCheckWholeDayAlarmOff(RecordID rec_id)
{
	BYTE *buffer;
	UWORD byte_read;
	UWORD rec_num;
	
	DataRecIDtoNum(sch_appmt_dbid, rec_id, &rec_num);
	DataOpenRecord(sch_appmt_dbid, rec_num, &rec_id, NULL);
	DataGetField(sch_appmt_dbid, rec_id, F_ALARM, &buffer, &byte_read);
	DataCloseRecord(sch_appmt_dbid, rec_id);
	if (*buffer == 1)
		return FALSE;
	else return TRUE;
}

/********************************************************
 * Function:	SchedulerSetAppmtAlarmOff
 * Purpose: 	This function is called to set an appointment 
				alarm off
 * Scope:		application/internal
 * Input:		current_rec_id
				row_num
 * Output:		None
 * Return:		None
 * Comment: 	None
 *********************************************************/
void SchedulerSetAppmtAlarmOff(RecordID current_rec_id, USHORT row_num)
{
	BYTE *buffer;
	UWORD byte_read;
	AppointmentDB *appmt_rec;
	WORD num_appmt;
	WORD count;
	BYTE alarm;
	RecordID rec_id;
	BYTE **string;
	BYTE *appmt_rec_buf, *appmt_string_buf;
	WORD num_bytes_appmt, num_bytes_string;
	UWORD rec_num;

	DataRecIDtoNum(sch_appmt_dbid, current_rec_id, &rec_num);
	DataOpenRecord(sch_appmt_dbid, rec_num, &rec_id, NULL);
	DataGetField(sch_appmt_dbid, current_rec_id, F_DATA, &buffer, &byte_read);
	SchedulerDBGetAppmtRec(buffer, byte_read, &appmt_rec, &num_appmt);
	qfree(buffer);
	DataGetField(sch_appmt_dbid, current_rec_id, F_STRING, &buffer, &byte_read);
	SchedulerDBGetAppmtString(buffer, byte_read, &string, &num_appmt);
	qfree(buffer);

	if (appmt_rec[row_num].alarm == VOICE_ALARM)
        appmt_rec[row_num].alarm = VOICE_NOT_ALARM;
    else appmt_rec[row_num].alarm = NO_ALARM;
	appmt_rec[row_num].alarm_time	= EXACT_TIME;
	appmt_rec[row_num].alarm_num	= 0;

	SchedulerDBAppmtToBuffer(appmt_rec, string, num_appmt, 
							 &appmt_rec_buf, &appmt_string_buf, &num_bytes_appmt,
							 &num_bytes_string);
	DataWriteField(sch_appmt_dbid, current_rec_id, F_DATA, num_bytes_appmt, appmt_rec_buf);
	DataWriteField(sch_appmt_dbid, current_rec_id, F_STRING, num_bytes_string, appmt_string_buf);
	qfree(appmt_rec_buf);
	qfree(appmt_string_buf);
	qfree(appmt_rec);
	for (count = 0; count < num_appmt; count++)
		qfree(string[count]);
	qfree(string[count]);


	for (count = 0; count < num_appmt; count++)
	{
        if (appmt_rec[count].alarm != NO_ALARM &&
            appmt_rec[count].alarm != VOICE_NOT_ALARM)
		{
			alarm = 1;
			DataWriteField(sch_appmt_dbid, current_rec_id, F_ALARM, 1, (BYTE*)&alarm);
			return ;
		}
	}
	alarm = 0;
	DataWriteField(sch_appmt_dbid, current_rec_id, F_ALARM, 1, (BYTE*)&alarm);
	DataCloseRecord(sch_appmt_dbid, current_rec_id);
}


/********************************************************
 * Function:	SchedulerGetNextRecWithAlarm
 * Purpose: 	This fucntion is called to get next record id 
				that has alarm 
 * Scope:		application/internal
 * Input:		current_rec_id				
 * Output:		next_rec_id
 * Return:		TRUE		found
				FALSE		not found
 * Comment: 	None
 *********************************************************/
BOOLEAN SchedulerGetNextRecWithAlarm(RecordID current_rec_id, RecordID *next_rec_id)
{
	UWORD current_rec_num;
	UWORD rec_num;
	UWORD total_rec;
	BYTE *buffer;
	UWORD byte_read;

	DataRecIDtoNum(sch_appmt_dbid, current_rec_id, &current_rec_num);
	DataTotalRecord(sch_appmt_dbid, &total_rec);

	for (rec_num = current_rec_num + 1; rec_num < total_rec; rec_num++)
	{
		DataOpenRecord(sch_appmt_dbid, rec_num, next_rec_id, NULL);
		DataGetField(sch_appmt_dbid, *next_rec_id, F_ALARM, &buffer, &byte_read);
		if (*buffer == 1)
		{
			qfree(buffer);
			DataCloseRecord(sch_appmt_dbid, *next_rec_id);
			return TRUE;
		}
		qfree(buffer);
		DataCloseRecord(sch_appmt_dbid, *next_rec_id);
	}
	return FALSE;
}

/********************************************************
 * Function:	SchedulerCheckEarliestDayWithAlarm
 * Purpose: 	This function is called to check whether the input date is 
				the earliest date with alarm or not
 * Scope:		application/internal
 * Input:		date				
 * Output:		None
 * Return:		TRUE		
 				FALSE		
 * Comment: 	None
 *********************************************************/
BOOLEAN SchedulerCheckEarliestDayWithAlarm(UBYTE *date)
{
	UWORD 				input_rec_num;
	RecordID 			input_rec_id;
	RTM 				time, temp_time, temp_time1;
	UBYTE 				cur_date[3], alarm_date[3];
	AppointmentDB       *appmt_rec;
	BOOLEAN 			result;
    BYTE                alarm, *buffer;
	UWORD 				byte_read;
	WORD				num_appmt, count;
	SHORT				hour, minute;


	if (DataFindBinRecord(sch_appmt_dbid, F_DATE, 0, date, 3, &input_rec_id) != TRUE)
		return FALSE;

	temp_time1.msec = 0;
	temp_time1.sec = 0;
	temp_time1.min = 0;
	temp_time1.hour = 0;
	temp_time1.mday = 0;
	temp_time1.mon = 0;
	temp_time1.year = 0;
	temp_time1.wday = 0;
	temp_time1.yday = 0;

	RtcGetTime(&time);	
	cur_date[0] = (UBYTE)(time.year - SYSETUP_YEAR_OFFSET);
	cur_date[1] = (UBYTE)(time.mon + 1);
	cur_date[2] = (UBYTE)(time.mday);

	result = AlarmGetLastAlarmTime(SysGetActiveAppID(), &time);

	alarm_date[0] = (UBYTE)(time.year - SYSETUP_YEAR_OFFSET);
	alarm_date[1] = (UBYTE)(time.mon + 1);
	alarm_date[2] = (UBYTE)(time.mday);
		
	DataRecIDtoNum(sch_appmt_dbid, input_rec_id, &input_rec_num);
	DataOpenRecord(sch_appmt_dbid, input_rec_num, &input_rec_id, NULL);
	DataGetField(sch_appmt_dbid, input_rec_id, F_ALARM, &buffer, &byte_read);
    if (*buffer == 0)
    {
		qfree(buffer);
        DataCloseRecord(sch_appmt_dbid, input_rec_id);
        return FALSE;
    }
    else if ((result == TRUE && SchedulerCompareDate(date, alarm_date) >= 1 && SchedulerCompareDate(date, cur_date) <= 1) || (result == FALSE))
	{
		qfree(buffer);
		DataGetField(sch_appmt_dbid, input_rec_id, F_DATA, &buffer, &byte_read);		
		SchedulerDBGetAppmtRec(buffer, byte_read, &appmt_rec, &num_appmt);
		qfree(buffer);
		alarm = 0;
		for (count = 0; count < num_appmt; count++)
		{
			if (appmt_rec[count].alarm == NO_ALARM || appmt_rec[count].alarm == VOICE_NOT_ALARM)
			    continue;
			temp_time = temp_time1;
			temp_time.year = (SHORT)(date[0] + SYSETUP_YEAR_OFFSET);
			temp_time.mon  = (SHORT)(date[1] - 1);
			temp_time.mday = (SHORT)(date[2]);

			temp_time.hour = (SHORT)(appmt_rec[count].hour);
            temp_time.min = (SHORT)(appmt_rec[count].minute);

			if (appmt_rec[count].alarm_time == MIN_15_BEFORE)
				temp_time.min -= 15;
			else if (appmt_rec[count].alarm_time == MIN_30_BEFORE)
                temp_time.min -= 30;
			else if (appmt_rec[count].alarm_time == MIN_60_BEFORE)
                temp_time.min -= 60;

			if (temp_time.min < 0)    
			{
				temp_time.min += 60;
				temp_time.hour --;
			}
		
			if (temp_time.hour < 0)
			{
				temp_time.hour = 0;
                temp_time.min = 0;
			}
/*			else 	
			{
				temp_time.hour = (SHORT)(appmt_rec[count].hour);
                temp_time.min = (SHORT)(appmt_rec[count].minute);
			}
*/			
			RtcGetTime(&time);
//kkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkk
            if (RtcCompareTime(&temp_time, &time) == TRUE)
            //if (Dt_CmpDate((RTM*)&temp_time, (RTM*)&time))
//kkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkkk
			{
				alarm = 1;
				break;
			}
		}
		qfree(appmt_rec);
		DataCloseRecord(sch_appmt_dbid, input_rec_id);
		if (alarm == 1)
			return TRUE;
		else return FALSE;
    }
	return FALSE;
}

/********************************************************
 * Function:	SchedulerCompareDate
 * Purpose: 	This function is called to compare two input date
 * Scope:		application/internal
 * Input:		input_date_0
 *				input_date_1
 * Output:		None
 * Return:		0			input_date_0 < input_date_1
				1			input_date_0 = input_date_1
				2			input_date_0 > input_date_1
 * Comment: 	None
 *********************************************************/
BYTE SchedulerCompareDate(UBYTE *input_date_0, UBYTE *input_date_1)
{
	USHORT count; 

	for (count = 0; count < 3; count++)
	{
		if (input_date_0[count] > input_date_1[count])
			return 0;
		else if (input_date_0[count] < input_date_1[count])
			return 2;
	}
	return 1;
}

/********************************************************
 * Function:    SchedulerSaveAndSetAlarm
 * Purpose:     
 * Scope:		application/internal
 * Input:		None
 * Output:		None
 * Return:		None
 * Comment: 	None
 *********************************************************/
void SchedulerSaveAndSetAlarm()
{
    UBYTE    date[4];

    SchedulerGetDayViewDate(date);
    SchedulerDayViewSaveAppmtData(&virtual_table, date);
	if (SchedulerCheckEarliestDayWithAlarm(date) == TRUE)
		SchedulerSetAppmtWholeDayAlarm(date);
}
    





