/*
================================ COPYRIGHT ================================
The contents of this file are subject to the VTech Informations Ltd. License
of VT-OS Ver. 1.1 operating system (the "License"); you may not use this 
file except in compliance with the License.  
 
Software distributed under the License is distributed on an "AS IS" basis, 
WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
for the specific language governing rights and limitations under the License.
 
The Original Code is VT-OS Ver. 1.1 Operating System, released 
on October 1st, 1999

The Initial Developer of the Original Code is VTech Informations Ltd.  All 
codes are Copyright (C) VTech Informations Ltd. 1999.  All Rights Reserved.
===========================================================================
*/

/*
===========================================================================
File        :   app.h
Author(s)   :   Henry Fok
Company     :   VTech Informations Ltd.
Project     :   Helio 
Date:	    :   October 1st, 1999
Purpose:	:   application header file
Revision    :   1.1
Note        :   None
===========================================================================
*/              


#ifndef __APP_H_
#define __APP_H_

#include "include\tdlann.h"
#include "include\system.h"
#include "include\SySetup.h"
#include "include\virtable.h"

/*****************************************************************
************************** Defines ********************************
******************************************************************/
#define EVT_DAY_VIEW_STRING_ENTER		17000
#define EVT_DAY_VIEW_STRING_EXIT		17001
#define EVT_DAY_VIEW_STRING_SELECT		17002

#define VT_TABLE_CHANGED			0
#define VT_TABLE_NOT_CHANGED		1
#define VT_FIELD_CHANGED			2

#define TO_LEFT						0
#define TO_RIGHT					1

#define     MemoryLow			MemoryCheckMemLow()
#define     MemoryNormal		!MemoryCheckMemLow()

#define		SetAppmtNumHighBit(a)		a|0x80000000	

/*****************************************************************
********************** Global Variables **************************
*****************************************************************/
extern AppID		app_id;
extern DatabaseID	sch_appmt_dbid;
extern DatabaseID	sch_voice_dbid;
extern VirtualTable virtual_table;
extern RecordID		rec_rec_id;
extern BOOLEAN		recorded;
extern BOOLEAN		duplicate;
extern UWORD		alarm_count;
extern UWORD		current_appmt_num;

//======================= Scheduler Database ========================
#define SCH_STATUS      0x80001000
#define SCH_DAY_VIEW	0x80001001

//======================= Scheduler Status Define (Close Application) ========================
//Field 0
#define RESTART			0
#define DAY_VIEW		1
#define WEEK_VIEW		2
#define MONTH_VIEW		3

//Field 1   -------------------------   font_size (1 byte)
//Field 2   -------------------------	alarm_count (UWORD)


//======================= Scheduler Day View Information ========================
//Field	0	-------------------------	editing date (3 bytes)

/*==================== Menu Item ===========================*/
#define MENU_ITEM_COPY			0
#define MENU_ITEM_CUT			1
#define MENU_ITEM_DUP           2
#define MENU_ITEM_PASTE         3
#define MENU_ITEM_UNDO          4
#define MENU_ITEM_PURGE         5
#define MENU_ITEM_ZOOM          6

/*****************************************************************
***************** Global Variables - FORM_SCH_DAY_VIEW ***********
*****************************************************************/
extern BOOLEAN new_appointment;
extern EvtType day_view_old_event;

/*****************************************************************
************ Global Variables - Database Structure ***********
*****************************************************************/
/* Scheduler Appointment Database Structure */
typedef struct _APPOINTMENT_DB
{
	BYTE			hour;
	BYTE			minute;
	BYTE			alarm;
	BYTE			alarm_time;
	UWORD			alarm_num;
	WORD			appointment;
	UWORD			appmt_num;
	DatabaseID		dbid;
	RecordID		recid;
}AppointmentDB;

#define F_DATE			0
#define F_DATA			1
#define F_STRING		2
#define F_ALARM			3
//#define F_APPMT_NUM		4


//Field 0 ------------------------- (BYTE)hour, (BYTE)minute, (BYTE)alarm, (BYTE)alarm_time, (WORD)appointment
//Field 1 ------------------------- AppointmentDB
//Field 2 ------------------------- string, .........
//Field 3 ------------------------- whether there is 1 or more alarm set in the date

/*****************************************************************
************ Global Variables - FORM_SCH_TIME_SETTINGS ***********
*****************************************************************/
/*==================== TIME_SETTINGS =====================*/
typedef struct _TIME_SETTINGS
{
	BOOLEAN			new_appointment;
	BOOLEAN			new_appointment_with_time;
	BYTE			hour_start;
	BYTE			hour_end;
	BYTE			minute_start;
	BYTE			minute_end;
	BYTE			alarm;
	BYTE			alarm_time;
	UWORD			alarm_num;
	DatabaseID		dbid;
	RecordID		recid;
	WORD			row_num;
}TimeSettings;
extern TimeSettings time_settings;

/*****************************************************************
************ Global Variables - FORM_SCH_ALARM_SETTINGS ***********
*****************************************************************/
/*==================== TIME_SETTINGS =====================*/
typedef struct _ALARM_SETTINGS
{
	BOOLEAN			change;
	BOOLEAN			recorded;
	DatabaseID		dbid;
	RecordID		org_recid;
	RecordID		temp_recid;
	BYTE			hour_start;
	BYTE			minute_start;
	BYTE			alarm;
	BYTE			alarm_time;
	UWORD			alarm_num;
	WORD			row_num;
}AlarmSettings;
extern AlarmSettings alarm_settings;

/*****************************************************************
************ Global Variables - FORM_SCH_REC ***********
*****************************************************************/
extern BOOLEAN		rec_stop;

/*****************************************************************
************ Global Variables - FORM_SCH_REPLAY ***********
*****************************************************************/
extern ObjectID		enter_form_id;
extern DatabaseID	voice_playing_dbid;
extern RecordID		voice_playing_recid;
extern UWORD		play_pause_frame;
extern BOOLEAN		play_stop;

/*****************************************************************
********************** OBJECT ID Defines **************************
*****************************************************************/
//==================== FORM_EMAIL_NAME_LIST ===================================
#define FORM_SCH_DAY_VIEW				0
#define POPUP_TRIGGER_APP_SELECT        1
#define STRING_SCH_STATUS               2
#define BITMAP_SCH_DAY_VIEW_ANN_ON		3
#define BITMAP_SCH_DAY_VIEW_TODO_ON		4
#define BITMAP_SCH_DAY_VIEW_ANN_OFF		152	
#define BITMAP_SCH_DAY_VIEW_TODO_OFF	153
#define BITMAP_SCH_DAY_VIEW_LEFT		5
#define BITMAP_SCH_DAY_VIEW_RIGHT		6
#define STRING_SCH_DAY_VIEW_DATE		7		
#define BUTTON_SCH_DAY_VIEW_NEW			8
#define BUTTON_SCH_DAY_VIEW_GOTO		9
#define TABLE_SCH_DAY_VIEW				10
#define STRING_SCH_DAY_VIEW_0			11
#define STRING_SCH_DAY_VIEW_1			12
#define STRING_SCH_DAY_VIEW_2			13
#define STRING_SCH_DAY_VIEW_3			14
#define STRING_SCH_DAY_VIEW_4			15
#define STRING_SCH_DAY_VIEW_5			16
#define STRING_SCH_DAY_VIEW_6			17
#define STRING_SCH_DAY_VIEW_7			18
#define STRING_SCH_DAY_VIEW_8			19
#define STRING_SCH_DAY_VIEW_9			20
#define STRING_SCH_DAY_VIEW_10			21
#define STRING_SCH_DAY_VIEW_11			22
#define FIELD_SCH_DAY_VIEW_0			23
#define FIELD_SCH_DAY_VIEW_1			24
#define FIELD_SCH_DAY_VIEW_2			25
#define FIELD_SCH_DAY_VIEW_3			26
#define FIELD_SCH_DAY_VIEW_4			27
#define FIELD_SCH_DAY_VIEW_5			28
#define FIELD_SCH_DAY_VIEW_6			29
#define FIELD_SCH_DAY_VIEW_7			30
#define FIELD_SCH_DAY_VIEW_8			31
#define FIELD_SCH_DAY_VIEW_9			32
#define FIELD_SCH_DAY_VIEW_10			33
#define FIELD_SCH_DAY_VIEW_11			34
#define BITMAP_SCH_DAY_VIEW_0			35
#define BITMAP_SCH_DAY_VIEW_1			36
#define BITMAP_SCH_DAY_VIEW_2			37
#define BITMAP_SCH_DAY_VIEW_3			38
#define BITMAP_SCH_DAY_VIEW_4			39
#define BITMAP_SCH_DAY_VIEW_5			40
#define BITMAP_SCH_DAY_VIEW_6			41
#define BITMAP_SCH_DAY_VIEW_7			42
#define BITMAP_SCH_DAY_VIEW_8			43
#define BITMAP_SCH_DAY_VIEW_9			44
#define BITMAP_SCH_DAY_VIEW_10			45
#define BITMAP_SCH_DAY_VIEW_11			46
#define SCROLLBAR_SCH_DAY_VIEW			47
#define	BUTTON_SCH_DAY_VIEW_ALARM		71
#define BUTTON_SCH_DAY_VIEW_DEL			99
#define BUTTON_SCH_DAY_VIEW_PURGE		100
#define MENU_SCH_DAY_VIEW			151
#define CHECKBOX_SCH_DAY_VIEW			165
#define CHECKBOX_SCH_WEEK_VIEW			166
#define CHECKBOX_SCH_MONTH_VIEW			167

//==================== FORM_SCH_ALARM ===================================
#define FORM_SCH_ALARM					48
#define BITMAP_SCH_ALARM_X				49
#define PUSHBUTTON_SCH_ALARM_ON			50
#define PUSHBUTTON_SCH_ALARM_OFF		51
#define POPUP_TRIGGER_SCH_ALARM_TIME	52
#define PUSHBUTTON_SCH_ALARM_VOICE		53
#define PUSHBUTTON_SCH_ALARM_TONE		54
#define STRING_SCH_ALARM_0				55
#define STRING_SCH_ALARM_1				56

//==================== FORM_SCH_TIME_SETTINGS ===================================
#define FORM_SCH_TIME_SETTINGS			57
#define BITMAP_SCH_TIME_X				58
#define BUTTON_SCH_TIME_BACK_0			59
#define BUTTON_SCH_TIME_BACK_1			60
#define STRING_SCH_TIME_0				61
#define STRING_SCH_TIME_1				62
#define STRING_SCH_TIME_2				63
#define STRING_SCH_TIME_3				64
#define STRING_SCH_TIME_HOUR			65
#define STRING_SCH_TIME_MIN				66
#define LIST_SCH_TIME_HOUR_0			67
#define LIST_SCH_TIME_HOUR_1			68
#define LIST_SCH_TIME_MIN				69
#define BUTTON_SCH_TIME_NOTIME			70
//#define	BUTTON_SCH_TIME_ALARM			71


//==================== FORM_SCH_REC ===================================
#define FORM_SCH_REC					119
#define BITMAP_SCH_REC_RECORD			120
#define BITMAP_SCH_REC_X				121
#define BITMAP_SCH_REC_PAUSE			122
#define BITMAP_SCH_REC_STOP				123
#define STRING_SCH_REC_MIN				124
#define STRING_SCH_REC_MAX				125
#define STRING_SCH_REC_VOLUME			126
#define SCROLLBAR_SCH_REC				127

//==================== FORM_SCH_REPLAY ===================================
#define FORM_SCH_REPLAY					128
#define BITMAP_SCH_REPLAY_PLAY			129

//==================== FORM_SCH_ALARM_SAVE_CHANGES ===================================
#define FORM_SCH_ALARM_SAVE_CHANGES			130
#define BITMAP_SCH_ALARM_SAVE_CHANGES_X		131
#define STRING_SCH_ALARM_SAVE_CHANGES		132
#define BUTTON_SCH_ALARM_SAVE_CHANGES_YES	133
#define BUTTON_SCH_ALARM_SAVE_CHANGES_NO	134

//==================== FORM_SCH_ALARM_SELECT ===================================
#define FORM_SCH_ALARM_SELECT				135
#define BITMAP_SCH_ALARM_SELECT_X			136
#define STRING_SCH_ALARM_SELECT_0			137
#define STRING_SCH_ALARM_SELECT_1			138

//==================== FORM_SCH_ALARM_SELECT ===================================
#define FORM_SCH_VOICE_EXISTS				139
//#define BITMAP_SCH_ALARM_SELECT_X			136
#define STRING_SCH_VOICE_EXISTS				140
#define BUTTON_SCH_VOICE_EXISTS_PLAY		141
#define BUTTON_SCH_VOICE_EXISTS_REPLACE		142
#define BITMAP_SCH_VOICE_EXISTS_X			143

//==================== FORM_SCH_PURGE ==================================
#define FORM_SCH_PURGE						144
#define STRING_SCH_PURGE_0					145	
#define STRING_SCH_PURGE_1					146	
#define POPUP_TRIGGER_SCH_PURGE				147
#define CHECKBOX_SCH_PURGE					148
#define BUTTON_SCH_PURGE_PROCEED			149
#define BUTTON_SCH_PURGE_CANCEL				150

/*============== FORM_SCH_DATE ==================================*/
#define		FORM_SCH_DATE						72
#define		STRING_SCH_DATE_DATE				73
#define		PUSH_SCH_DATE_JAN					74
#define		PUSH_SCH_DATE_FEB					75
#define		PUSH_SCH_DATE_MAR					76
#define		PUSH_SCH_DATE_APR					77
#define		PUSH_SCH_DATE_MAY					78
#define		PUSH_SCH_DATE_JUN					79
#define		PUSH_SCH_DATE_JUL					80
#define		PUSH_SCH_DATE_AUG					81
#define		PUSH_SCH_DATE_SEP					82
#define		PUSH_SCH_DATE_OCT					83
#define		PUSH_SCH_DATE_NOV					84
#define		PUSH_SCH_DATE_DEC					85
#define		STRING_SCH_DATE_WDAY				86
#define		TABLE_SCH_DATE						87
#define		BITMAP_SCH_DATE_LEFT				88
#define		BITMAP_SCH_DATE_RIGHT				89
#define		BUTTON_SCH_DATE_DISABLE_BKGRD		90
#define		PUSH_SCH_DATE_Y1					91
#define		PUSH_SCH_DATE_Y2					92
#define		PUSH_SCH_DATE_Y3					93
#define		PUSH_SCH_DATE_Y4					94
#define		PUSH_SCH_DATE_Y5					95
#define		LINE_SCH_DATE_YUP					96
#define		LINE_SCH_DATE_YDWN					97
#define		BUTTON_SCH_DATE_TODAY				98

#define		GROUP_ID_SCH_DATE_MONTH				2
#define		GROUP_ID_SCH_DATE_YEAR				4

/*===============================================================*/
/*============== FORM_SCH_MONTH =================================*/
#define		FORM_SCH_MONTH						101
#define		BITMAP_SCH_MONTH_LEFT				102
#define		BITMAP_SCH_MONTH_RIGHT				103
#define		STRING_SCH_MONTH_MONTH				104
#define		STRING_SCH_MONTH_WDAY1				105
#define		STRING_SCH_MONTH_WDAY2				106
#define		STRING_SCH_MONTH_WDAY3				107
#define		STRING_SCH_MONTH_WDAY4				108
#define		STRING_SCH_MONTH_WDAY5				109
#define		STRING_SCH_MONTH_WDAY6				110
#define		STRING_SCH_MONTH_WDAY7				111
#define		TABLE_SCH_MONTH						112
#define		BUTTON_SCH_MONTH_GOTO				113

#define		SCH_MONTH_LEFT						0
#define		SCH_MONTH_RIGHT						1


/*============== FORM_SCH_WEEK ==================================*/
#define		FORM_SCH_WEEK						114
#define		STRING_SCH_WEEK_WEEK				115
#define		BITMAP_SCH_WEEK_LEFT				116
#define		BITMAP_SCH_WEEK_RIGHT				117
#define     SCHLINE_SCH_WEEK					118

//==================== FORM_SCH_NO_MEM ===================================
#define FORM_SCH_NO_MEM                     154
#define STRING_SCH_NO_MEM                   155
#define BITMAP_SCH_NO_MEM                   156

//==================== FORM_SCH_VOICE_LOW_BATTERY ===================================
#define FORM_SCH_VOICE_LOW_BATTERY                157
#define BITMAP_SCH_VOICE_LOW_BATTERY_X            158
#define STRING_SCH_VOICE_LOW_BATTERY_0            159
#define STRING_SCH_VOICE_LOW_BATTERY_1            160

//==================== FORM_SCH_LOW_MEMORY =============================
#define FORM_SCH_LOW_MEMORY			161
#define BITMAP_SCH_LOW_MEMORY		162
#define STRING_SCH_LOW_MEMORY_0		163
#define STRING_SCH_LOW_MEMORY_1		164


#endif
