/*
================================ COPYRIGHT ================================
The contents of this file are subject to the VTech Informations Ltd. License
of VT-OS Ver. 1.1 operating system (the "License"); you may not use this 
file except in compliance with the License.  
 
Software distributed under the License is distributed on an "AS IS" basis, 
WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
for the specific language governing rights and limitations under the License.
 
The Original Code is VT-OS Ver. 1.1 Operating System, released 
on October 1st, 1999

The Initial Developer of the Original Code is VTech Informations Ltd.  All 
codes are Copyright (C) VTech Informations Ltd. 1999.  All Rights Reserved.
===========================================================================
*/

/*
===========================================================================
File        :   dev_pen.c
Author(s)   :   Andrew Hui
Company     :   VTech Informations Ltd.
Project     :   Helio 
Date:	    :   October 1st, 1999
Purpose:	:   Pen low level device driver
Revision    :   1.1
Note        :   23/09/98   Prepared by Andrew Hui (AH)                     
               23/09/98   V1.0 release (AH)                               
               23/10/98   V1.1 release (AH)                               
               09/12/98   V1.2 release (AH) <Optimize to O3>              
               09/12/98   V1.3 release (AH) <Optimize to O3>              
               10/12/98   V1.4 release (AH) <Optimize to O3>              
               15/12/98   V1.5 release (AH) <Optimize to O3>              
               05/01/99   V1.6 release (AH) <Optimize to O3>              
               01/03/99   V1.7 release (AH) <Optimize to O3>              
               19/03/99   V1.8 release (AH) <Optimize to O3>              
               12/04/99   V1.9 release (AH) <Optimize to O3>              
               17/05/99   V1.10 release (AH) <Optimize to O3>             
               24/05/99   V1.11 release (AH) <Optimize to O3>             
               29/06/99   V1.12 release (AH) <Optimize to O3>             
===========================================================================
*/              


#include "platform.h"
#ifdef PR31700

#define NEW_PEN_SCHEME    1

#include "include\dev_pen.h"
#include "include\penmgr.h"
#include "include\dev_io.h"
#include "include\dev_bety.h"
#include "include\datatype.h"

#include "include\dev_sib.h"

volatile WORD    PenState ;      /* state of Pen-Input handler state-machine */
volatile WORD    PenTmrAction;
volatile WORD    PenTmrHandle;
volatile WORD    PenTmrHdlrState;
volatile WORD    PenInitState;
volatile WORD    PenMsgSerial;
volatile WORD    PenLastSerial;
volatile WORD    PenDint;
volatile WORD    pen_xy_exchange;
volatile WORD    PenUpCount;

extern int PenUpSerial;
extern int PenUpCount;

void PenDownIntEnable()
{
    PenInitState=0;
    PenDownIntEnableSM();
}

void PenDownIntEnableSM()
{
    switch(PenInitState)
    {

        case 0:     /* request for access */
                    PenInitState ++;
                    SibEngaged ++;
                    SibAccess((BETTY_TS_CTRL |BTY_TSMX_GND |BTY_TSPX_POW |BTY_TSC_MODE(2) |BTY_TSC_BIAS_ENA  |BTY_WRT), 0);
                    SibAccess(( BETTY_INT_CLEAR |BTY_WRT),0);
                    SibAccess(( BETTY_INT_CLEAR |BTY_TSPX_INT_STAT|BTY_WRT),0);

                    /* recheck the pendown */
                    SibAccess( BETTY_TS_CTRL, PenDownIntEnableSM);

                    break;
        case 1:
                    PenInitState ++;

                    if (( ((SibDataInBuf)>>12) && 3)==0)    /* still pen down */
                    {
                        PenMsgSerial =0;
                        MsgAppendMsgInt(PenMgr, PEN_DOWN, 0, 0, (void*)(PenMsgSerial));
                        PenMsgSerial ++;

                        if (PenTmrHandle ==0)   /* enable timer interrupt */
                        {
                            PenTmrHandle = TmrIntEnableIntPen(PEN_SAMPLE_PERIOD, PenTmrIntHandler);
                            PenTmrAction=PEN_TIMER_START;
                            PenDint=1;  // disable

                        }
                        else
                        {
                            PenDint=1;  // disable
                            PenTmrAction=PEN_TIMER_START;
                        }

                        if (PenState==0)    PenGetxy();     /* read x,y */

                        SibAccess( BETTY_FAL_INT_ENABLE ,PenDownIntEnableSM);
                    }
                    else
                    {
                        if (PenTmrHandle!=0)
                        {
                            TmrIntDisableInt(PenTmrHandle);  /* disable timer interrupt */
                            PenTmrHandle=0;
                            PenLastSerial=PenMsgSerial;
                            PenMsgSerial=0;
                        }
                        SibAccess((BETTY_TS_CTRL |BTY_TSMX_POW |BTY_TSPX_POW
                                  |BTY_TSMY_GND |BTY_TSPY_GND
                                  |BTY_TSC_MODE(0) |BTY_WRT ), 0);
                        SibAccess(( BETTY_FAL_INT_ENABLE |BTY_WRT),0);
                        SibAccess(( BETTY_FAL_INT_ENABLE |BTY_TSPX_FAL_INT
                                   |BTY_WRT), PenDownIntEnableSM);
                    }
                    break;
        case 2 :    PenInitState =0;
                    SibEngaged --;
                    break;
    }
}


void PenTmrIntHandler()     /* call by timer */
{
    if (PenTmrAction==PEN_TIMER_STOP)
    {
        PenDownIntEnable();
        PenTmrAction=PEN_TIMER_STOPED ;
    }
    else
    {
        if (PenTmrAction==PEN_TIMER_START)
        {
            if (PenTmrHandle!=0)
            {
                if (PenState==0)
                {
                    PenGetxy();     /* read x,y */
                }
            }
        }
    }
}


static WORD x_ADC_value;    /* X-Coordinate from TTP (RAW data) */
static WORD y_ADC_value;    /* Y-Coordinate from TTP (RAW data) */
static WORD old_x, old_y;


/* trigger by SF0 int and ADC complete */
/* assume no one will access the SIB when TTP is using */

void PenRotateInit()
{
    pen_xy_rotate   = 0;
}

void PenInit()
{

    CPU->REG_INT_ENABLE_1 |= SIBIRQPOSINT;
    PenUpSerial     = 0;
    PenLastSerial   = 0;
    PenMsgSerial    = 0;
    PenDint         = 0;
    PenInitState    = 0;
    PenEvntPtr      = 0;
    PenState        = 0;
    PenMgrMode      = PEN_NORMAL;
    PenTmrHdlrState = 0;
    PenTmrHandle    = 0;
    PenUpCount      = 0;


    pen_xy_exchange = 1;
    pu_x=0;
    pu_y=0;

    PenTmrAction = PEN_TIMER_STOPED ;

    DIS_GBL_INT ;   /* AH */
    PenInitSM();
    ENA_GBL_INT ;   /* AH */

    Delay(0x1ff);

    while (PenInitState!=0)  ;  /* wait until Pen init complete */

}



void PenRestoreDefault()
{
  /* restore pen infomation stored in Flash */
        PenMinX = 78;
        PenMinY = 77;

        PenMaxX = 900;
        PenMaxY = 700;
}


void PenInitSM()
{

    switch(PenInitState)
    {

        case 0  :   /* setup Pre-Clear Pen Int on Betty Chip */
                    PenInitState ++;
                    SibEngaged ++;
                    SibAccess((BETTY_INT_CLEAR |BTY_WRT ), 0);
                    SibAccess((BETTY_INT_CLEAR |0xffff |BTY_WRT ), 0);
                    SibAccess((BETTY_RIS_INT_ENABLE |BTY_WRT ), 0);
                    SibAccess((BETTY_FAL_INT_ENABLE |BTY_TSPX_FAL_INT |BTY_WRT ), 0 );
                    SibAccess((BETTY_TS_CTRL |BTY_TSMX_POW |BTY_TSPX_POW
                              |BTY_TSMY_GND |BTY_TSPY_GND
                              |BTY_TSC_MODE(0) |BTY_WRT ), PenInitSM);
                    break;


        case 1 :    PenInitState =0;
                    SibEngaged --;
                    break;
    }
 }


WORD PenSetCalValue (WORD x1, WORD y1, WORD x2, WORD y2)  /* for Jason */
{
    PenMinX =   x1;
    PenMinY =   y1;

    PenMaxX =   x2;
    PenMaxY =   y2;
}


int pen1;

void PenGetxy()
{
    switch(PenState)
    {
        /* X-Coordinate from TTP (Switch ttp terminal) */
        case 0:     /* request for access */
                    PenState ++;
                    SibEngaged ++;
                    BtyAdcAccess(PenGetxy);

                    break;


        case 1:     /* x-axis */
                    if (PenDint==1)  /* if first pendown,disabled PD int */
                    {
                        PenDint=0;
                        PenState ++;
                    }
                    else PenState +=2;  /* skip case 1 */
                    SibAccess(( BETTY_TS_CTRL   |BTY_TSMY_POW     |BTY_TSPY_GND
                               |BTY_TSC_MODE(2) |BTY_TSC_BIAS_ENA |BTY_WRT),
                                PenGetxy);
                    break;


        case 2:     /* Disable Pendown interrupt */
                    PenState ++;
                    SibAccess(( BETTY_FAL_INT_ENABLE |BTY_WRT), PenGetxy);
                    break;

        case 3:     /* Disable Pendown interrupt */
                    PenState ++;
                    SibAccess(( BETTY_FAL_INT_ENABLE |BTY_WRT), 0);
                    SibAccess(( BETTY_ADC_CTRL
                                |TTP_ADC_PX
                                |BTY_ADC_SYNC_ENA |BTY_VREFBYP_CON
                                |BTY_ADC_ENA                  |BTY_WRT), 0);
                    SibAccess(( BETTY_ADC_CTRL
                                |TTP_ADC_PX
                                |BTY_ADC_SYNC_ENA |BTY_VREFBYP_CON
                                |BTY_ADC_ENA  |BTY_ADC_START  |BTY_WRT), 0);
                    SibAccess(( BETTY_ADC_DATA), PenGetxy);
                    break;


        case 4:     /* request a read from ADC */
                    if( ((SibDataInBuf>>15) & 1) != 0 )
                    {
                        PenState ++;

                        x_ADC_value = ((SibDataInBuf>>5) & 0x3FF);

                    /* Y-Coordinate from TTP (Switch ttp terminal) */
                        SibAccess((BETTY_TS_CTRL |BTY_TSMX_GND |BTY_TSPX_POW
                                  |BTY_TSC_MODE(2) |BTY_TSC_BIAS_ENA  |BTY_WRT),
                                   PenGetxy);
                    }
                    else
                        SibAccess((BETTY_ADC_DATA ), PenGetxy); /* one more */

                    break;


        case 5:    /* Pre-Clear Pen down interrupt */

                    PenState ++;
                    SibAccess((BETTY_ADC_CTRL
                              |TTP_ADC_PY
                              |BTY_ADC_SYNC_ENA |BTY_VREFBYP_CON
                              |BTY_ADC_ENA                  |BTY_WRT), 0);

                    SibAccess((BETTY_ADC_CTRL
                              |TTP_ADC_PY
                              |BTY_ADC_SYNC_ENA |BTY_VREFBYP_CON
                              |BTY_ADC_ENA  |BTY_ADC_START  |BTY_WRT), 0);
                    SibAccess((BETTY_ADC_DATA ), PenGetxy);

                    break;

        case 6:     /* request a read from ADC */

                    if( ((SibDataInBuf>>15) & 1)==1 )
                    {
                        PenState ++;

                        y_ADC_value = (SibDataInBuf>>5) &0x3FF;


                        /* disable ADC */
                        SibAccess(( BETTY_ADC_CTRL
                                    |TTP_ADC_PX
                                    |BTY_ADC_SYNC_ENA |BTY_VREFBYP_CON
                                    |BTY_WRT), 0);

                        SibAccess(( BETTY_TS_CTRL |BTY_TSMX_POW |BTY_TSPX_POW
                                                  |BTY_TSMY_GND |BTY_TSPY_GND
                                                  |BTY_TSC_MODE(0)
                                                  |BTY_WRT), PenGetxy);
                    }
                    else
                        SibAccess(( BETTY_ADC_DATA ), PenGetxy);
                    break;



        case 7:    /* Check if pen is still down */
                    PenState ++;
                    SibAccess(( BETTY_TS_CTRL), 0);
                    SibAccess(( BETTY_INT_CLEAR |BTY_WRT), 0);
                    SibAccess(( BETTY_INT_CLEAR |BTY_TSPX_INT_STAT|BTY_WRT), 0);
                    SibAccess(( BETTY_TS_CTRL), PenGetxy);
                    break;

        case 8:    /* send message */
                    if (((SibDataInBuf>>12) & 0x3) ==0)
                    {
                        if( (y_ADC_value >1020) | (x_ADC_value >1020)  )
                        {
                            if (PenUpCount==0)
                                PenUpCount =1;
                            else
                                  MsgAppendMsgReplaceInt(PenMgr, PEN_UP, 0, 0,(void*)(PenMsgSerial));

                        }
                        else
                        {
                            if((y_ADC_value <8) | (x_ADC_value <8) )
                            {
                                y_ADC_value = old_y;
                                x_ADC_value = old_x;
                            }
                            else
                            {
                                old_y= y_ADC_value;
                                old_x= x_ADC_value;
                            }
                            if ( pen_xy_exchange == 0 )
                            {

#ifdef NEW_PEN_SCHEME
                                if (PenMgrMode==PEN_NORMAL)
                                    MsgAppendMsgReplaceInt(PenMgr, PEN_MOVE, x_ADC_value, y_ADC_value, (void*)(PenMsgSerial));
                                else
#endif
                                    MsgAppendMsgInt(PenMgr, PEN_MOVE, x_ADC_value, y_ADC_value, (void*)(PenMsgSerial));




                            }
                            else
                            {

#ifdef NEW_PEN_SCHEME
                                if (PenMgrMode==PEN_NORMAL)
                                    MsgAppendMsgReplaceInt(PenMgr, PEN_MOVE, y_ADC_value, x_ADC_value, (void*)(PenMsgSerial));
                                else
#endif
                                    MsgAppendMsgInt(PenMgr, PEN_MOVE, y_ADC_value, x_ADC_value, (void*)(PenMsgSerial));

                            }
                        }
                        PenMsgSerial ++;
                    }
                    else   /* x,y may not be valid */
                    {
                        if (PenUpCount==0)
                            PenUpCount = 1;
                        else
                        {
                              MsgAppendMsgReplaceInt(PenMgr, PEN_UP, 0, 0,(void*)(PenMsgSerial));
                            PenMsgSerial ++;
                        }
                    }

                    SibEngaged --;
                    PenState=0;
                    break;
    }

    if (PenState==0)
    {
        AdcQueue[AdcQPtr.front].status=ADC_Q_COMPLETE;
        BtyAdcAccessHandler();
    }
}



#endif

