/*
================================ COPYRIGHT ================================
The contents of this file are subject to the VTech Informations Ltd. License
of VT-OS Ver. 1.1 operating system (the "License"); you may not use this
file except in compliance with the License.

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
for the specific language governing rights and limitations under the License.

The Original Code is VT-OS Ver. 1.1 Operating System, released
on October 1st, 1999

The Initial Developer of the Original Code is VTech Informations Ltd.  All
codes are Copyright (C) VTech Informations Ltd. 1999.  All Rights Reserved.
===========================================================================
*/

/*
===========================================================================
File		:	dev_pwr.c
Author(s)	:	Andrew Hui
Company 	:	VTech Informations Ltd.
Project 	:	Helio
Date:		:	October 1st, 1999
Purpose:	:	power management device driver
Revision	:	1.1
Note		:	None
===========================================================================
*/

#include "platform.h"
#ifdef PR31700

#include	"include\uart.h"
#include	"include\pr31700c.h"
#include	"include\pr3910.h"
#include	"include\pr31700s.h"
#include	"include\dev_pwr.h"
#include	"include\dev_pen.h"
#include	"include\dev_snd.h"
#include	"include\dev_sib.h"
#include	"include\intc.h"
#include	"include\sio.h"
#include	"include\DataType.h"
#include	"include\CpuIoReg.h"
#include	"include\lcddrv.h"

#ifdef DEBUG_BAT_LOW_TO_LCD
extern UWORD lcd_mem[];
#endif

#define CPU_IDLE_SPEED			CPU_CLK_9M
#define CPU_SEMI_IDLE_SPEED 	CPU_CLK_37M


#define UART_A_CTS	(1<<2)	/* input */
#define UART_A_RTS	(1<<4)	/* output */


#define ESD_DELAY   0x1fff

volatile WORD BatAdcValue;

volatile WORD BatteryLowestLevel;
volatile WORD SemiPowerOff;
volatile WORD BatteryEmpty;

void AHTurnLcdOff(void) ;
void PowerMgrInit2();
int LcdTurnedOff(void) ;
int TriggerByBatLow;


PwrType PowerOnStatus;
void BettyInit();

/*---------------------------------------------------------------------------
  Prototype :	void  CpuChangeSpeed(int speed);
  Purpose	:	Set CPU to run at different speed
  Scope 	:	OS
  input 	:	None
  return	:	N/A
---------------------------------------------------------------------------*/

void	CpuChangeSpeed(int speed)
{
	register int spd;

	DIS_GBL_INT;
	spd = ( (CP0ReadConfigReg() & ~(CPU_IDLE_SPEED_MASK)) | (speed) ) ;
    __asm("nop");
    __asm("nop");
    __asm("nop");
    CP0WriteConfigReg(spd) ;
	ENA_GBL_INT;
}

void	CpuChangeSpeedInt(int speed)
{
	register int spd;

	spd = ( (CP0ReadConfigReg() & ~(CPU_IDLE_SPEED_MASK)) | (speed) ) ;
    __asm("nop");
    __asm("nop");
    __asm("nop");
	CP0WriteConfigReg(spd) ;
}

int    CpuChkSpeed()
{
	 return (( CP0ReadConfigReg() & (CPU_IDLE_SPEED_MASK)) >>10);
}


/*---------------------------------------------------------------------------
  Prototype :	void  CpuIdle();
  Purpose	:	Set CPU to idle mode wait for enabled interrupt
  Scope 	:	OS
  input 	:	None
  return	:	N/A
---------------------------------------------------------------------------*/

void CpuDoze()
{
	register WORD tmp;
	register WORD Cp0status;
	register WORD CpuClkSpeed;
	register WORD ChangeSpeed;

	/* Disable Interrupt */
	tmp = CP0ReadStatusReg();	 Cp0status= tmp & CP0_IDLE_MASK;
	CP0WriteStatusReg(tmp&~(CP0_IDLE_MASK));

	/* Set CPU_CLK to slow speed */
	tmp = CP0ReadConfigReg() ;	  CpuClkSpeed = tmp & CPU_IDLE_SPEED_MASK ;
#ifdef PSD
    if ((!SibEngaged)&(!SndAction))
#else
    if ((!SibEngaged))  /* change to 9MHz */
#endif
    {

        CPU->REG_CLK_CTRL &=  ~(ENSIBMCLK);
		CPU->REG_SIB_CTRL &= ~(ENSIB |ENSF0 |ENSF1);
        CP0WriteConfigReg( ( (tmp&~(CPU_IDLE_SPEED_MASK)) | (CPU_IDLE_SPEED)) );
	}
    else        CP0WriteConfigReg( ( (tmp&~(CPU_IDLE_SPEED_MASK)) | (CPU_SEMI_IDLE_SPEED) ) );

	ENA_GBL_INT;
		CPU->REG_POWER_CTRL |= STOPCPU;   CPU->REG_POWER_CTRL &= ~(STOPCPU);
	DIS_GBL_INT;

	tmp = CP0ReadConfigReg() ;

	if (!SibEngaged)
	{
	   CPU->REG_CLK_CTRL |= ENSIBMCLK; CPU->REG_SIB_CTRL |= ENSIB |ENSF0 |ENSF1;
	}

	/* Restore CPU_CLK */
	CP0WriteConfigReg( (tmp & ~(CPU_IDLE_SPEED_MASK)) | CpuClkSpeed);
	/* Restore Enabled Interrupt */
	tmp = CP0ReadStatusReg();
	CP0WriteStatusReg((tmp&~(CP0_IDLE_MASK))|Cp0status);
}


void CpuDozeOnSemiOff()
{
	volatile WORD tmp;
	volatile WORD Cp0status;
	volatile WORD CpuClkSpeed;
	register WORD ChangeSpeed;

	/* Disable Interrupt */
	tmp = CP0ReadStatusReg();
	Cp0status= tmp & CP0_IDLE_MASK;
	CPU->REG_CLK_CTRL &=  ~(ENSIBMCLK);
	CPU->REG_SIB_CTRL &= ~(ENSIB |ENSF0);
	CP0WriteStatusReg(tmp&~(CP0_IDLE_MASK));

	/* Set CPU_CLK to slow speed */
	tmp = CP0ReadConfigReg() ;
	CpuClkSpeed = tmp & CPU_IDLE_SPEED_MASK ;
	CP0WriteConfigReg( ( (tmp&~(CPU_IDLE_SPEED_MASK)) | (CPU_IDLE_SPEED) ) );

	ENA_GBL_INT;
		CPU->REG_POWER_CTRL |= STOPCPU;
		CPU->REG_POWER_CTRL &= ~(STOPCPU);
	DIS_GBL_INT;

	/* Restore CPU_CLK */
	tmp = CP0ReadConfigReg() ;
	CPU->REG_CLK_CTRL |= ENSIBMCLK	 ;
	CPU->REG_SIB_CTRL |= ENSIB |ENSF0;
	CP0WriteConfigReg( (tmp & ~(CPU_IDLE_SPEED_MASK)) | CpuClkSpeed);

	/* Restore Enabled Interrupt */
	tmp = CP0ReadStatusReg();
	CP0WriteStatusReg((tmp&~(CP0_IDLE_MASK))|Cp0status);
}




/*---------------------------------------------------------------------------
  Prototype :	void  pSaveIoReg();
  Purpose	:	Save CPU register content
  Scope 	:	OS
  input 	:	None
  return	:	N/A
---------------------------------------------------------------------------*/

								   /* Storage for saving reg. when Power Off */
WORD SaveIoRegLoc[MAX_SAVE_LOC];	/* This locations should be permanetly	  */
								   /* allocated in a protected region		 */

pSaveIoReg()
{
	register WORD *SaveTo;
	register WORD *Cpu;

	SaveTo= SaveIoRegLoc;
	Cpu = (WORD*)0xB0C00000;
	SaveTo [pCLK_CTRL		 ] = Cpu [IOREG_CLK_CTRL		];

}


pRestoreIoReg()
{
	register WORD *SaveTo;
	register WORD *Cpu;

	SaveTo= SaveIoRegLoc;
	Cpu = (WORD*)0xB0C00000;
	Cpu [IOREG_CLK_CTRL 	   ] = SaveTo [pCLK_CTRL		] ;
}


/*===========================================================*/



volatile WORD	 NormalOperation ;	  /* check by Interrupt handler   */
volatile WORD	 WakeCpuUp		 ;	  /* condition to power-up system */

#define  TRING_PIN	(1<<11)
#define  FLASH_RDY	(1<<7 )
HwTringOff()
{
	register WORD *CpuReg;

	CpuReg = CPU_IOREG_BASE;	/* defined in CpuIoReg.h */

	 /* Uart Tranciever */
	CpuReg[IOREG_MFIO_SEL		 ] |=  TRING_PIN|FLASH_RDY;
	CpuReg[IOREG_MFIO_DIRECTN	 ] |= (TRING_PIN);
	CpuReg[IOREG_MFIO_DATA_OUT	 ] |= (TRING_PIN);
	CpuReg[IOREG_MFIO_DIRECTN	 ] &= ~(FLASH_RDY);
	CpuReg[IOREG_MFIO_POWER_DOWN ] &= ~(TRING_PIN);   /* don't power down */
	CpuReg[IOREG_MFIO_POWER_DOWN ] |= FLASH_RDY;   /* don't power down */
}

HwUartAInit()	/* called by UartInit() in uart.c */
{
	register WORD *CpuReg;

	CpuReg = CPU_IOREG_BASE;	/* defined in CpuIoReg.h */

	 /* Uart Tranciever */
	CpuReg[IOREG_MFIO_SEL		 ] |= UART_TX_ENABLE |UART_RX_DISABLE
									 |UART_A_CTS |UART_A_RTS ;
	CpuReg[IOREG_MFIO_DIRECTN	 ] |= UART_TX_ENABLE |UART_RX_DISABLE |UART_A_RTS;
	CpuReg[IOREG_MFIO_DIRECTN	 ] &= ~ UART_A_CTS;
	CpuReg[IOREG_MFIO_DATA_OUT	 ] &= ~UART_TX_ENABLE  ;  /* disable uart tx */
	CpuReg[IOREG_MFIO_DATA_OUT	 ] |=  UART_RX_DISABLE ;  /* disable uart rx */
	CpuReg[IOREG_MFIO_DATA_OUT	 ] &= ~(UART_A_RTS) ;
	CpuReg[IOREG_MFIO_POWER_DOWN ] |= (UART_A_RTS |UART_A_CTS);
	CpuReg[IOREG_MFIO_POWER_DOWN ] &= ~(UART_TX_ENABLE |UART_RX_DISABLE);

	/* Uart Module */
	CpuReg[IOREG_UARTA_CTRL_1]	&=~ENUART;	 /* disable UART A */
}

UartAOn()	/* called by Uart Manager when she want to use it */
{
	register WORD *CpuReg;
	register WORD tmp;

	CpuReg = CPU_IOREG_BASE;	/* defined in CpuIoReg.h */

	/* Uart Tranciever */
	CpuReg[IOREG_MFIO_SEL		 ] |= UART_TX_ENABLE |UART_RX_DISABLE
									 |UART_A_CTS |UART_A_RTS ;
	CpuReg[IOREG_MFIO_DIRECTN	 ] |= UART_TX_ENABLE |UART_RX_DISABLE |UART_A_RTS;
	CpuReg[IOREG_MFIO_DIRECTN	 ] &= ~ UART_A_CTS;
	CpuReg[IOREG_MFIO_DATA_OUT	 ] |=  UART_TX_ENABLE  ;  /* disable uart tx */
	CpuReg[IOREG_MFIO_DATA_OUT	 ] &= ~(UART_RX_DISABLE | UART_A_RTS) ;
	CpuReg[IOREG_MFIO_POWER_DOWN ] |= (UART_A_RTS |UART_A_CTS);
	CpuReg[IOREG_MFIO_POWER_DOWN ] &= ~(UART_TX_ENABLE |UART_RX_DISABLE);

	/* Uart Module */
	tmp = CpuReg[IOREG_UARTA_HLDG]; 			 /* dummy read			 */
	tmp = CpuReg[IOREG_UARTA_HLDG]; 			 /* dummy read			 */
	CpuReg[IOREG_UARTA_CTRL_1]	|= ENUART;		 /* enable	UART A		 */
	CpuReg[IOREG_UARTA_CTRL_1] &= ~DISTXD;		 /* enable Tx			 */
	CPU->REG_INT_ENABLE_2 |= UARTARXINT;
}


UartAOff()	/* called by Uart Manager when she don't use it */
{

	register WORD *CpuReg;
	register WORD tmp;

	CpuReg = CPU_IOREG_BASE;	/* defined in CpuIoReg.h */

	/* Uart Tranciever */
	CpuReg[IOREG_MFIO_SEL		 ] |= UART_TX_ENABLE |UART_RX_DISABLE
									 |UART_A_CTS |UART_A_RTS ;
	CpuReg[IOREG_MFIO_DIRECTN	 ] |= UART_TX_ENABLE |UART_RX_DISABLE |UART_A_RTS;
	CpuReg[IOREG_MFIO_DIRECTN	 ] &= ~ UART_A_CTS;
	CpuReg[IOREG_MFIO_DATA_OUT	 ] &= ~(UART_TX_ENABLE |UART_A_RTS) ;  /* disable uart tx */
	CpuReg[IOREG_MFIO_DATA_OUT	 ] |=  UART_RX_DISABLE ;  /* disable uart rx */
	CpuReg[IOREG_MFIO_POWER_DOWN ] |= (UART_A_RTS |UART_A_CTS);
	CpuReg[IOREG_MFIO_POWER_DOWN ] &= ~(UART_TX_ENABLE |UART_RX_DISABLE);

	/* Uart Module */
	CpuReg[IOREG_UARTA_CTRL_1]	&=~ENUART;	 /* disable UART A */
	CpuReg[IOREG_UARTA_CTRL_1] |= DISTXD;		 /* disable Tx			 */
	tmp = CpuReg[IOREG_UARTA_HLDG]; 			 /* dummy read			 */
	tmp = CpuReg[IOREG_UARTA_HLDG]; 			 /* dummy read			 */
	CPU->REG_INT_ENABLE_2 &= ~UARTARXINT;
}



HwAudioInit() /* called by SndInit() in dev_snd.c */
{
	register WORD *CpuReg;

	CpuReg = CPU_IOREG_BASE;	/* defined in CpuIoReg.h */

	/* audio amplifier */
	CpuReg[IOREG_MFIO_SEL		 ] |= AUDIO_DISABLE ;
	CpuReg[IOREG_MFIO_DIRECTN	 ] |= AUDIO_DISABLE ;
	CpuReg[IOREG_MFIO_DATA_OUT	 ] |= AUDIO_DISABLE ;  /* disable audio amp */
	CpuReg[IOREG_MFIO_POWER_DOWN ] &= ~AUDIO_DISABLE ;
}


HwAudioOn()   /* called by sound manager when she want to playback audio */
{
	register WORD *CpuReg;

	CpuReg = CPU_IOREG_BASE;	/* defined in CpuIoReg.h */

	CpuReg[IOREG_MFIO_SEL		 ] |= AUDIO_DISABLE ;
	CpuReg[IOREG_MFIO_DIRECTN	 ] |= AUDIO_DISABLE ;
	CpuReg[IOREG_MFIO_DATA_OUT	 ] &= ~AUDIO_DISABLE ;	/* enable audio amp */
	CpuReg[IOREG_MFIO_POWER_DOWN ] &= ~AUDIO_DISABLE ;

}


HwAudioOff()  /* called by sound manager when she don't use audio output */
{
	register WORD *CpuReg;

	CpuReg = CPU_IOREG_BASE;	/* defined in CpuIoReg.h */

	CpuReg[IOREG_MFIO_SEL		 ] |= AUDIO_DISABLE ;
	CpuReg[IOREG_MFIO_DIRECTN	 ] |= AUDIO_DISABLE ;
	CpuReg[IOREG_MFIO_DATA_OUT	 ] |= AUDIO_DISABLE ;  /* disable audio amp */
	CpuReg[IOREG_MFIO_POWER_DOWN ] &= ~AUDIO_DISABLE ;
}


HwHotSyncInit() /* called by SndInit() in dev_snd.c */
{
	register WORD *CpuReg;

	CpuReg = CPU_IOREG_BASE;	/* defined in CpuIoReg.h */

	/* Hot Sync Key */
	CpuReg[IOREG_MFIO_SEL		 ] |= (1<<25) ;
	CpuReg[IOREG_MFIO_DIRECTN	 ] &= ~(1<<25) ;
	CpuReg[IOREG_MFIO_POWER_DOWN ] &= ~(1<<25) ;
}


HwHotSyncOn()	/* called by sound manager when she want to playback audio */
{
	register WORD *CpuReg;

	CpuReg = CPU_IOREG_BASE;	/* defined in CpuIoReg.h */

	/* Hot Sync Key */
	CpuReg[IOREG_MFIO_SEL		 ] |= (1<<25) ;
	CpuReg[IOREG_MFIO_DIRECTN	 ] &= ~(1<<25) ;
	CpuReg[IOREG_MFIO_POWER_DOWN ] &= ~(1<<25) ;

}

HwHotSyncOff()	/* called by sound manager when she don't use audio output */
{
	register WORD *CpuReg;

	CpuReg = CPU_IOREG_BASE;	/* defined in CpuIoReg.h */

	/* Hot Sync Key */
	CpuReg[IOREG_MFIO_SEL		 ] |= (1<<25) ;
	CpuReg[IOREG_MFIO_DIRECTN	 ] &= ~(1<<25) ;
	CpuReg[IOREG_MFIO_POWER_DOWN ] &= ~(1<<25) ;
}


#define DIAGNOSTIC_PIN	(1<<19)

HwDiagnosticPinOff()  /* called by sound manager when she don't use audio output */
{
	register WORD *CpuReg;

	CpuReg = CPU_IOREG_BASE;	/* defined in CpuIoReg.h */

	CpuReg[IOREG_MFIO_SEL		 ] |=  DIAGNOSTIC_PIN ;
	CpuReg[IOREG_MFIO_DIRECTN	 ] &= ~DIAGNOSTIC_PIN ;
	CpuReg[IOREG_MFIO_POWER_DOWN ] |=  DIAGNOSTIC_PIN ;
}



CpuUartBOff()  /* called by Uart Manager when she don't use it */
{
	register WORD *CpuReg;
	register WORD tmp;

	CpuReg = CPU_IOREG_BASE;	/* defined in CpuIoReg.h */

	/* Uart Module */
	CpuReg[IOREG_UARTB_CTRL_1]	&=~ENUART;	 /* disable UART A */
	CpuReg[IOREG_UARTB_CTRL_1]	|= DISTXD;		  /* disable Tx 		  */
	tmp = CpuReg[IOREG_UARTB_HLDG]; 			 /* dummy read			 */
	tmp = CpuReg[IOREG_UARTB_HLDG]; 			 /* dummy read			 */
}



CpuSpiOff()
{
	WORD tmp;
	register WORD *CpuReg;

	CpuReg = CPU_IOREG_BASE;	/* defined in CpuIoReg.h */

	tmp = CpuReg[IOREG_SPI_HLDG];				 /* dummy read			 */
	tmp = CpuReg[IOREG_SPI_HLDG];
	CpuReg[IOREG_SPI_CTRL] &= ~1;			 /* disable SPI 		 */
}

CpuChiOff()
{
	register WORD *CpuReg;

	CpuReg = CPU_IOREG_BASE;	/* defined in CpuIoReg.h */

	CpuReg[IOREG_CHI_CTRL]	   &= ~1;			 /* disable CHI bus 	 */
}

CpuMBusOff()
{
	register WORD *CpuReg;

	CpuReg = CPU_IOREG_BASE;	/* defined in CpuIoReg.h */

	CpuReg[IOREG_MBUS_CTRL_1]  &= ~1;			 /* Disable MagicBus	 */
}


CpuIROff()
{

	register WORD *CpuReg;

	CpuReg = CPU_IOREG_BASE;	/* defined in CpuIoReg.h */

	CpuReg[IOREG_IR_CTRL_1] 	 &= ~7;
}


CpuSibOff()
{
	register WORD tmp;
	register WORD *CpuReg;

	CpuReg = CPU_IOREG_BASE;	/* defined in CpuIoReg.h */

	/* empty all buffer */
	tmp=	CpuReg[IOREG_SIB_SND_HLDG];
	tmp=	CpuReg[IOREG_SIB_TEL_HLDG];
	tmp=	CpuReg[IOREG_SIB_SF0_CTRL];
	tmp=	CpuReg[IOREG_SIB_SF1_CTRL];

	tmp=	CpuReg[IOREG_SIB_SF0_STATUS];
	tmp=	CpuReg[IOREG_SIB_SF1_STATUS];

	CpuReg[IOREG_SIB_CTRL] &=~(ENSF0|ENSF1|ENSIB);
	Delay(0x1000);

}

///////////////////////////////////////////////////////////////////////

#define HwPwrOk()  (CPU->REG_POWER_CTRL & PWROK)

int hwoffsw;


HwSystemPowerOff()
{                          /* power down devices */ hwoffsw =0;
    HwSystemPowerOffGo();
}

HwSystemEmgPowerOff()
{                          /* power down devices */ hwoffsw =1;
	HwSystemPowerOffGo();
}


HwSystemPowerOffGo()
{
	register WORD PowerStable;
	register WORD i;
	register WORD bc;

	if (!hwoffsw)	 DIS_GBL_INT;
	CpuChangeSpeedInt(CPU_CLK_37M);

	if (!LcdTurnedOff() )		 AHTurnLcdOff();

	OffEL();
	HwSioOff();
	HwHotSyncOff();
	HwAudioOff();
	HwDiagnosticPinOff();
	HwTringOff();

    if (!hwoffsw)   pSaveIoReg();
	CpuSpiOff();
	CpuChiOff();
	CpuMBusOff();
	CpuIROff();
	CpuUartBOff();
	CpuSibOff();
	UartAOff();
	BettyQueueFlush();
	SndDriverOff();
	if (hwoffsw)	TmrInit();

	NormalOperation = FALSE;
	WakeCpuUp		= FALSE;
	SemiPowerOff	= FALSE;

	IntDisableBatEmptyInt();  IntDisableBatDoorInt();  IntDisableBatLowInt();

	if (!hwoffsw)
	{
		if((WakeCpuUp==FALSE) || (BatDoorOpen()) || (HwPwrOk()==0) )
			bc=1;
		else bc=0;
	}
	else
	{
		if (((WakeCpuUp==FALSE) && (SemiPowerOff==FALSE)) ||(BatDoorOpen())||(HwPwrOk()==0))
			bc=1;
		else bc=0;
	}

	while ( bc )
	{
		while (((WakeCpuUp==FALSE) && (SemiPowerOff==FALSE)) ||(BatDoorOpen())||(HwPwrOk()==0))
		{

			DIS_GBL_INT;
			if (hwoffsw)   CpuChangeSpeedInt(CPU_CLK_37M);
			if (BatDoorOpen())
			{
				WakeCpuUp =FALSE; SemiPowerOff =FALSE;
			}
			IntDisableBatEmptyInt();  IntDisableBatDoorInt();
			IntDisableBatLowInt();

			if (!hwoffsw)
			{
				if (((WakeCpuUp==FALSE) && (SemiPowerOff==FALSE)) ||(BatDoorOpen())||(HwPwrOk()==0))
					 CpuPowerOff();

				if (BatDoorOpen())
				{
					WakeCpuUp	 = FALSE;
					SemiPowerOff = FALSE;
				}

				__asm("nop");   __asm("nop");   __asm("nop");  __asm("nop"); __asm("nop");   __asm("nop");
			}
			else
			{

				while ( (BatDoorOpen())||(HwPwrOk()==0))
				{

					if ( (BatDoorOpen())||(HwPwrOk()==0) )
					{
						WakeCpuUp	=FALSE;
						SemiPowerOff=FALSE;
					}


					if (  ((WakeCpuUp==FALSE) && (SemiPowerOff==FALSE))
						  ||(BatDoorOpen())||(HwPwrOk()==0)  )
					{

						CpuPowerOff();
					}
				}


				if (BatDoorOpen())
				{
					WakeCpuUp	 = FALSE;
					SemiPowerOff = FALSE;
				}

			}/////////
			ENA_GBL_INT;
		 }

			__asm("nop");   __asm("nop");   __asm("nop");  __asm("nop");   __asm("nop");   __asm("nop");

		if (!hwoffsw)	CpuChangeSpeedInt(CPU_CLK_37M);

		while ((SemiPowerOff) && (!WakeCpuUp) && (!BatDoorOpen()) && (HwPwrOk()))
		{
			DIS_GBL_INT;
			IntClearBatEmptyInt();	IntClearBatDoorInt();
			IntEnableBatEmptyInt();  IntEnableBatDoorInt();

			if((SemiPowerOff) && (!WakeCpuUp) && (!BatDoorOpen()) && (HwPwrOk()))
				 CpuDozeOnSemiOff();

			if (hwoffsw)   CpuChangeSpeedInt(CPU_CLK_37M);
            if (!hwoffsw)
            {
                if (BatDoorOpen()|| (!HwPwrOk()))
                {
                    WakeCpuUp    = FALSE;
                    SemiPowerOff = FALSE;
                }
            }
            else
            {
                if (BatDoorOpen())
                {
                    WakeCpuUp    = FALSE;
                    SemiPowerOff = FALSE;
                }
            }

			ENA_GBL_INT;
		}

		IntDisableBatEmptyInt();	IntDisableBatDoorInt();
		IntClearBatDoorInt();		IntClearBatEmptyInt();

		if (!hwoffsw)
		{
			if((WakeCpuUp==FALSE) || (BatDoorOpen()) || (HwPwrOk()==0) )
				bc=1;
			else bc=0;
		}
		else
		{
			if (((WakeCpuUp==FALSE) && (SemiPowerOff==FALSE)) ||(BatDoorOpen())||(HwPwrOk()==0))
				bc=1;
			else bc=0;
		}

	}

	DIS_GBL_INT;

    SemiPowerOff=FALSE;
	CpuChangeSpeedInt(CPU_CLK_37M);

	IntDisableBatEmptyInt();	IntDisableBatDoorInt();
	IntClearBatDoorInt();		IntClearBatEmptyInt();

	NormalOperation=TRUE;

    if (!hwoffsw)  pRestoreIoReg();   /* save register contents */

	/* initialize all necessary device */
	BettyReset();
	SibInit();
    BettyQueueFlush();

    if (!hwoffsw)
    {
        SibEnable();        OffEL();
        ENA_GBL_INT;
        HwSioInit();        HwTringOff();        BettyInit();
        PenInit();          Delay(0x1ff);        SndDDInit();
        ELInit();        LcdTurnOn();
        DIS_GBL_INT;
        PowerMgrInit2();
        ENA_GBL_INT;
    }
    else
    {
        HwTringOff();
    }

}


SndDriverOff()
{
  CPU->REG_SIB_DMA_CTRL &= ~(ENDMATXSND | ENDMARXSND) ;  /* disable DMA */
}



CpuPowerOff()
{
	register WORD tmp;
	register WORD *CpuReg;
	register WORD *SaveTo;
	register WORD Cp0status;

	SaveTo = SaveIoRegLoc;
	CpuReg = CPU_IOREG_BASE;	/* defined in CpuIoReg.h */
	tmp = 12345678;

	while ( (BatDoorOpen())||(HwPwrOk()==0)|| (tmp==12345678) )
	{
		/* set to kernel mode , disable interrupt */
		tmp = CP0ReadStatusReg() ;
		#ifdef NORMAL_POWER_OFF
		tmp &= ~(SR_IEC | RUNTIME_INTERRUPT_MASK) ;
		tmp |= SR_PE ;
		#endif
		#ifndef NORMAL_POWER_OFF
		tmp &= ~( RUNTIME_INTERRUPT_MASK) ;
		tmp |= SR_PE |SR_IEC ;
		#endif
		CP0WriteStatusReg(tmp);

		/* save all interrupt enable register */
		SaveTo [pINT_ENABLE_1	 ] = CpuReg [IOREG_INT_ENABLE_1    ];
		SaveTo [pINT_ENABLE_2	 ] = CpuReg [IOREG_INT_ENABLE_2    ];
		SaveTo [pINT_ENABLE_3	 ] = CpuReg [IOREG_INT_ENABLE_3    ];
		SaveTo [pINT_ENABLE_4	 ] = CpuReg [IOREG_INT_ENABLE_4    ];
		SaveTo [pINT_ENABLE_5	 ] = CpuReg [IOREG_INT_ENABLE_5    ];

		/* clear all interrupt and disable all interrupt */
		CpuReg[IOREG_INT_STATUS_1]=0xffffffff;
		CpuReg[IOREG_INT_STATUS_2]=0xffffffff;
		CpuReg[IOREG_INT_STATUS_3]=0xffffffff;
		CpuReg[IOREG_INT_STATUS_4]=0xffffffff;
		CpuReg[IOREG_INT_STATUS_5]=0xffffffff;

		/* enable cpu wake-up interrupt */
		CpuReg[IOREG_INT_ENABLE_1]	= WAKE_INT_ENABLE_1 ;
		CpuReg[IOREG_INT_ENABLE_2]	= WAKE_INT_ENABLE_2 ;
		CpuReg[IOREG_INT_ENABLE_3]	= WAKE_INT_ENABLE_3 ;
		CpuReg[IOREG_INT_ENABLE_4] &= (SIO_WAKE_INT_MASK); /* special handle*/
		CpuReg[IOREG_INT_ENABLE_4] |= WAKE_INT_ENABLE_4 ;  /* for SIO		*/
		CpuReg[IOREG_INT_ENABLE_5]	= WAKE_INT_ENABLE_5 ;
		ENA_GBL_INT;	CpuGotoSleep(); /* power down CPU */DIS_GBL_INT;
	   /* Restore Enabled Interrupt */

		#ifdef NORMAL_POWER_OFF
		tmp = CP0ReadStatusReg();
		tmp |= SR_PE | SR_IEC | RUNTIME_INTERRUPT_MASK ;
		#endif
		#ifndef NORMAL_POWER_OFF
		tmp = CP0ReadStatusReg();
		tmp |= SR_PE | SR_IEC | RUNTIME_INTERRUPT_MASK ;
		#endif
		CP0WriteStatusReg(tmp);
	}
}


CpuGotoSleep()
{
	register WORD i;
	register WORD j;
	register WORD *cpu_reg;

	register WORD tmp2;
	register WORD tmp3;


	register WORD *CpuReg;

	CpuReg = CPU_IOREG_BASE;	/* defined in CpuIoReg.h */
	i = 99;

	tmp2 = CpuReg[IOREG_MEM_CONFIG_4];
	tmp3 = CpuReg[IOREG_POWER_CTRL];


	CP0WriteCauseReg(0);

	for (i=0; i<2; i++)
	{
		 cpu_reg=(WORD*)0xb0e00000; cpu_reg[0]=0x000c0080;	cpu_reg=(WORD*)0xb0f00000;	 cpu_reg[0]=0x000c0080;
		 CpuReg[IOREG_MEM_CONFIG_4] = tmp2; CpuReg[IOREG_POWER_CTRL]=tmp3;

		 __asm("nop"); __asm("nop"); __asm("nop"); __asm("nop");    __asm("nop"); __asm("nop"); __asm("nop"); __asm("nop");         __asm("nop"); __asm("nop"); __asm("nop"); __asm("nop");    __asm("nop"); __asm("nop"); __asm("nop"); __asm("nop");
		 __asm("nop"); __asm("nop"); __asm("nop"); __asm("nop");    __asm("nop"); __asm("nop"); __asm("nop"); __asm("nop");         __asm("nop"); __asm("nop"); __asm("nop"); __asm("nop");    __asm("nop"); __asm("nop"); __asm("nop"); __asm("nop");

		 #ifdef NORMAL_POWER_OFF
		 CpuReg[IOREG_POWER_CTRL] &=~(FORCESHUTDOWN|ENFORCESHUTDOWN);
		 #endif
		 CpuReg[IOREG_MEM_CONFIG_4] &= ~MEMPOWERDOWN;
		 __asm("nop");
		 __asm("nop");
		 __asm("nop");
		 __asm("nop");
		 __asm("nop");
		 __asm("nop");

		 tmp2 |= MEMPOWERDOWN;
		 #ifdef NORMAL_POWER_OFF
		 tmp3  &= (~(VCCON|PWRCS|DIVMOD));
		 #endif
		 tmp3  &= (~(VCCON|PWRCS));   tmp3	|= (COLDSTART);
		 CpuReg[IOREG_MEM_CONFIG_3] &=~1;
		 j=0;
		 do
		 {
			 j++  ;
		 } while (j<0x1fff);
	}

	j=0;
	do { j++  ; } while (j<0x1fff);  CpuPowerOn();
}

CpuPowerOn()
{
	WORD i;

	register WORD *CpuReg;
	register WORD *SaveTo;

	SaveTo = SaveIoRegLoc;
	CpuReg = CPU_IOREG_BASE;	/* defined in CpuIoReg.h */

	/* Restore all interrupt enable register */
	CpuReg [IOREG_INT_ENABLE_1] = SaveTo [pINT_ENABLE_1];
	CpuReg [IOREG_INT_ENABLE_2] = SaveTo [pINT_ENABLE_2];
	CpuReg [IOREG_INT_ENABLE_3] = SaveTo [pINT_ENABLE_3];
	CpuReg [IOREG_INT_ENABLE_4] |= (SaveTo[pINT_ENABLE_4]
								  &(~(SIO_WAKE_INT_MASK)));
	CpuReg [IOREG_INT_ENABLE_5]  =	SaveTo [pINT_ENABLE_5];
}

volatile WORD ChkBatWaitingTmrHandle;
volatile WORD PwrLowWarningTmrHandle;
volatile WORD PwrLowFirstWarningTmrHandle;
volatile WORD ChkBatState;
volatile WORD BatCheckLevel;
volatile WORD BatLowHandlerState;
volatile WORD BatChkMainLevelReq;
volatile WORD BatteryLevelStatus;

/*---------------------------------------------------------------------------
  Prototype :	void  PowerMgrInit();
  Purpose	:	Initialize the Power Manager
  Scope 	:	OS
  input 	:	None
  return	:	N/A
---------------------------------------------------------------------------*/

void PowerMgrInit()
{
	PwrLowFirstWarningTmrHandle =0;
	PwrLowWarningTmrHandle	=  0;
	ChkBatWaitingTmrHandle	=  0;
	ChkBatState 			=  0;
	BatLowHandlerState		=  0;
	BatChkMainLevelReq		=  0;
	NormalOperation 		= TRUE; 	//AH 990630
	TriggerByBatLow 		= FALSE;	//AH 990712

	BatteryLowestLevel		= 10240;	//AH990709
	SemiPowerOff			= FALSE;

	CPU->REG_MFIO_SEL		  |=  BAT_DOOR_INT;
	CPU->REG_MFIO_DIRECTN	  &= ~BAT_DOOR_INT; /* input */
	CPU->REG_MFIO_POWER_DOWN  &= ~BAT_DOOR_INT; /* can be power down */

	BatteryLevelStatus = BAT_NORMAL;

	BatteryEmpty	   = FALSE;

	IntClearBatDoorInt();
	IntClearBatEmptyInt();
	IntClearBatLowInt();

	IntEnableBatDoorInt();
	IntEnableBatEmptyInt();
	IntEnableBatLowInt();

}

void PowerMgrInit2()
{
    PwrLowFirstWarningTmrHandle =0;
	PwrLowWarningTmrHandle	=  0;
	ChkBatWaitingTmrHandle	=  0;
	ChkBatState 			=  0;
	BatLowHandlerState		=  0;
	BatChkMainLevelReq		=  0;
	NormalOperation 		= TRUE; 	//AH 990630
	TriggerByBatLow 		= FALSE;	//AH 990712

	BatteryLowestLevel		= 10240;	//AH990709
	SemiPowerOff			= FALSE;

	CPU->REG_MFIO_SEL		  |=  BAT_DOOR_INT;
	CPU->REG_MFIO_DIRECTN	  &= ~BAT_DOOR_INT; /* input */
	CPU->REG_MFIO_POWER_DOWN  &= ~BAT_DOOR_INT; /* can be power down */

	BatteryLevelStatus = BAT_NORMAL;

	BatteryEmpty	   = FALSE;

	IntEnableBatDoorInt();
	IntEnableBatEmptyInt();
	IntEnableBatLowInt();

}



/*---------------------------------------------------------------------------
  Prototype :	void  BatDoorOpen();
  Purpose	:	Check Battery door closed or not
  Scope 	:	OS
  input 	:	None
  return	:	1 : Open,  0: Close
---------------------------------------------------------------------------*/

WORD BatDoorOpen()
{

#ifdef PSD
    CPU->REG_MFIO_SEL         |=  BAT_DOOR_INT;
	CPU->REG_MFIO_DIRECTN	  &= ~BAT_DOOR_INT; /* input */
	CPU->REG_MFIO_POWER_DOWN  &= ~BAT_DOOR_INT; /* can be power down */
#endif
	if ((CPU->REG_MFIO_DATA_IN & BAT_DOOR_INT)==0)
		return (0); /* Battery Door Closed */
	else
		return (1); /* Battery Door Open   */

}


/*---------------------------------------------------------------------------
  Prototype :	void  BatChkMainLevel()
  Purpose	:	Do a ADC on Battery to obtain a voltage level
  Scope 	:	OS
  input 	:	None
  return	:	1: not permitted,  0: premitted
---------------------------------------------------------------------------*/

WORD BatChkMainLevel()		 /* assumed called from forground */
{
	if (ChkBatState==0)
	{
		DIS_GBL_INT ;
		ChkBatterySM();
		ENA_GBL_INT ;
#ifdef PSD
		while ((ChkBatState) && (BatteryEmpty != FALSE))
            CpuDoze();
#endif
		return (0); 	/* operation permitted */
	}
	else
	{

		return (1);    /* operation NOT permitted */
	}

}



/*---------------------------------------------------------------------------
  Prototype :	void  BatChkMainLevelInt()
  Purpose	:	Do a ADC on Battery to obtain a voltage level
  Scope 	:	OS (within interrupt)
  input 	:	None
  return	:	1: not permitted,  0: premitted
---------------------------------------------------------------------------*/


WORD BatChkMainLevelInt()		/* assumed called from interrupt */
{
	if (ChkBatState==0) /* ok to check battery level */
	{
		if (ChkBatWaitingTmrHandle != 0)   /* disable timer if needed */
			TmrIntDisableInt(ChkBatWaitingTmrHandle);

		ChkBatWaitingTmrHandle =0;
		ChkBatterySM(); 		/* operation permitted */
	}
	else  /* operation NOT permitted, wait some time */
	{
		if (ChkBatWaitingTmrHandle==0)
            ChkBatWaitingTmrHandle = TmrIntEnableIntPen(BATTERY_WAIT_SOME_TIME, BatChkMainLevelInt );
	}

}


/*---------------------------------------------------------------------------
  Prototype :	void  PowerMgr(MsgType)
  Purpose	:	Generate different events with power management
  Scope 	:	OS
  input 	:	None
  return	:	N/A
---------------------------------------------------------------------------*/

PowerMgr(MsgType *PwrMsg)
{
	register WORD tmp;
	register WORD level;

	switch((WORD)(PwrMsg->msgID))
	{
	  case BAT_MAIN_LEVEL :

	  level = (100*(PwrMsg->para1 - BAT_LEVEL_EMPTY) + 10*(BAT_LEVEL_MAX -BAT_LEVEL_EMPTY)/2 )
			   / ((BAT_LEVEL_MAX -BAT_LEVEL_EMPTY)*10) ;

							   if ((level <0)||(PwrMsg->para1<BAT_LEVEL_EMPTY))
									  level = 0;
								if (level >10) level = 10;

								if ( BatteryLowestLevel < PwrMsg->para1 )
									 tmp = BatteryLowestLevel ;
								else	/* update minimum level */
								{
									 tmp = PwrMsg->para1  ;
									 BatteryLowestLevel = PwrMsg->para1;
								}

								if (BatteryLowestLevel < BAT_LEVEL_LOW_P)
								{
									BatLowPowerManagement();   /* turn off all the current hunger device */
									BatteryLevelStatus = BAT_LOW;
								}

								if (TriggerByBatLow)
								{
									if (PwrMsg->para1<BAT_LEVEL_LOW_P)
									{
										BatLowPowerManagement();   /* turn off all the current hunger device */
										BatteryLevelStatus = BAT_LOW;

                                        EvtAppendEvtReplace(POWER_EVENT, BAT_MAIN_LOW, BatMapLifeTime( tmp ), level, 0 );
									}
									else	/* disable timer interrupt				 */
									{		/* re-enable battery low(2.2V) interrupt */

										if (PwrLowWarningTmrHandle !=0)   /* disable timer interrupt  */
											TmrIntDisableInt(PwrLowWarningTmrHandle );

										PwrLowWarningTmrHandle = 0;

										IntClearBatLowInt();
										IntEnableBatLowInt();
									}

									TriggerByBatLow = FALSE;

								}
								else
								{
                                    EvtAppendEvtReplace(POWER_EVENT, BAT_MAIN_LEVEL, BatMapLifeTime(tmp), level, 0);
								}

								break;

      case BAT_MAIN_LOW   :     EvtAppendEvtReplace(POWER_EVENT, BAT_MAIN_LOW, 0, 0, 0 );                                                   break;
      case BAT_MAIN_EMPTY :     EvtAppendEvtReplace(POWER_EVENT, BAT_MAIN_EMPTY, 0, 0, 0 );                                                 break;
      case BAT_DOOR_OPEN  :     EvtAppendEvtReplace(POWER_EVENT, BAT_DOOR_OPEN, 0, 0, 0 );                                                    break;
      defaults            :     break;

	}
}


/*---------------------------------------------------------------------------
  Prototype :   void  BatLowHandler()
  Purpose	:	Generate battery low warning events
  Scope 	:	OS
  input 	:	None
  return	:	N/A
---------------------------------------------------------------------------*/

BatLowHandler() 	/* Timer handler -	within interrupt */
{

	if (PwrLowFirstWarningTmrHandle !=0)
		TmrIntDisableInt(PwrLowFirstWarningTmrHandle );

	TriggerByBatLow = TRUE ;
	BatChkMainLevelInt();	/* how to get this information ? */

	if (PwrLowWarningTmrHandle ==0)
		PwrLowWarningTmrHandle = TmrIntEnableIntPen(BATTERY_LOW_WARNING_PERIOD,
													BatLowHandler);
}



/*---------------------------------------------------------------------------
  Prototype :	void  NegPwrIntISR()
  Purpose	:	ISR to handle Power Int   (2.2v)
  Scope 	:	OS
  input 	:	None
  return	:	N/A
---------------------------------------------------------------------------*/

NegPwrIntISR()
{
	if ((BatteryEmpty==FALSE) && ((CPU->REG_INT_STATUS_5 & NEGPWROKINT)==0))
	{
		if (PwrLowWarningTmrHandle ==0)   /* enable timer interrupt */
		{
		  PwrLowFirstWarningTmrHandle = TmrIntEnableIntPen(100, BatLowHandler);
		}
		IntDisableBatLowInt();
	}
}


/*---------------------------------------------------------------------------
  Prototype :	void  NegPwrOkIntISR()
  Purpose	:	ISR to handle Power OK Int	(2V)
  Scope 	:	OS
  input 	:	None
  return	:	N/A
---------------------------------------------------------------------------*/

NegPwrOkIntISR()	/* Battery nearly EMPTY */
{
	register WORD *CpuReg;
	register WORD  counter;

#ifdef ESDSAVE
	counter = ESD_DELAY;
	while ((counter --)&&( HwPwrOk()==0 ));
#endif

	if ( !(CPU->REG_POWER_CTRL & PWROK) )	/* attack by ESD case */
	{

		CpuReg = CPU_IOREG_BASE;	/* defined in CpuIoReg.h */
		BatteryEmpty	   = TRUE;
		if (SemiPowerOff==TRUE)
		{
			SemiPowerOff = FALSE ;
			WakeCpuUp	 = FALSE ;
			CpuReg[IOREG_INT_STATUS_1]=0xffffffff;
			CpuReg[IOREG_INT_STATUS_2]=0xffffffff;
			CpuReg[IOREG_INT_STATUS_3]=0xffffffff;
			CpuReg[IOREG_INT_STATUS_4]=0xffffffff;
			CpuReg[IOREG_INT_STATUS_5]=0xffffffff;
		}
		else
		{
		   CpuChangeSpeedInt(CPU_CLK_9M);

		   CpuSibOff();
		   BettyQueueFlush();

		   AHTurnLcdOff();
		   HwAudioOff();
		   OffEL();

			if (PwrLowFirstWarningTmrHandle !=0)
				TmrIntDisableInt(PwrLowFirstWarningTmrHandle );

			if (PwrLowWarningTmrHandle !=0) 	  /* enable timer interrupt  */
				TmrIntDisableInt(PwrLowWarningTmrHandle );
			PwrLowWarningTmrHandle = 0;

			if (ChkBatWaitingTmrHandle != 0)   /* disable the waiting timer if needed */
				TmrIntDisableInt(ChkBatWaitingTmrHandle);

			ChkBatWaitingTmrHandle=0;

			IntDisableBatEmptyInt();
			IntDisableBatLowInt();
			IntClearBatLowInt();
		}
	}
}




NegBatDoorIntISR()	  /* MFIO:8 ;	1: Close,  0: Open */
{					/* Battery Door Open (1->0)    */
	register WORD *CpuReg;
	register WORD counter;

#ifdef ESDSAVE
    counter = ESD_DELAY;
	while ((counter --)&&( (CPU->REG_MFIO_DATA_IN & BAT_DOOR_INT)!=0  ));
#endif
	if ((CPU->REG_MFIO_DATA_IN & BAT_DOOR_INT))   /* attack by ESD case */
	{
		CpuReg = CPU_IOREG_BASE;	/* defined in CpuIoReg.h */

		if (SemiPowerOff==TRUE)
		{
			SemiPowerOff = FALSE ;
			WakeCpuUp	 = FALSE ;
			CpuReg[IOREG_INT_STATUS_1]=0xffffffff;
			CpuReg[IOREG_INT_STATUS_2]=0xffffffff;
			CpuReg[IOREG_INT_STATUS_3]=0xffffffff;
			CpuReg[IOREG_INT_STATUS_4]=0xffffffff;
			CpuReg[IOREG_INT_STATUS_5]=0xffffffff;
		}
		else
		{
			MsgAppendMsgInt((void*)PowerMgr, BAT_DOOR_OPEN, 0, 0, (void*)0);
			IntDisableBatDoorInt();
		}
	}
}


/* Production is using postive edge 0 -> 1	*/
PosBatDoorIntISR()	  /* MFIO:8 ;	1: Close,  0: Open */
{					/* Battery Door Open (1->0)    */
	register WORD *CpuReg;
	register WORD counter;

#ifdef ESDSAVE
    counter = ESD_DELAY;
	while ((counter --)&&( (CPU->REG_MFIO_DATA_IN & BAT_DOOR_INT)!=0  ));
#endif

	if ((CPU->REG_MFIO_DATA_IN & BAT_DOOR_INT))   /* attack by ESD case */
	{

		CpuReg = CPU_IOREG_BASE;	/* defined in CpuIoReg.h */

		if (SemiPowerOff==TRUE)
		{
			SemiPowerOff = FALSE ;
			WakeCpuUp	 = FALSE ;
			CpuReg[IOREG_INT_STATUS_1]=0xffffffff;
			CpuReg[IOREG_INT_STATUS_2]=0xffffffff;
			CpuReg[IOREG_INT_STATUS_3]=0xffffffff;
			CpuReg[IOREG_INT_STATUS_4]=0xffffffff;
			CpuReg[IOREG_INT_STATUS_5]=0xffffffff;
		}
		else
		{
			MsgAppendMsgInt((void*)PowerMgr, BAT_DOOR_OPEN, 0, 0, (void*)0);
			IntDisableBatDoorInt();
		}
	}
}


//////////////////////////////////////////////////////////////////

void ChkBatterySM()
{
	if (BatteryEmpty != FALSE)
	{
		if (ChkBatState !=0)	/* kill the previous job */
		{
			AdcQueue[AdcQPtr.front].status=ADC_Q_COMPLETE;
			BtyAdcAccessHandler();
			ChkBatState = 0;
		}

	}
	else
	{
		switch(ChkBatState)
		{
			case 0: 	/* set io pin to output */

						SibEngaged ++;
						ChkBatState++;
						BtyAdcAccess(ChkBatterySM);
						break;

			case 1:
						ChkBatState++;
						SibAccess( BETTY_ADC_CTRL
								   |TTP_ADC_AD0
								   |BTY_ADC_SYNC_ENA |BTY_VREFBYP_CON
								   |BTY_ADC_ENA 				|BTY_WRT, 0);

						SibAccess( BETTY_ADC_CTRL
								   |TTP_ADC_AD0
								   |BTY_ADC_SYNC_ENA |BTY_VREFBYP_CON
								   |BTY_ADC_ENA |BTY_ADC_START	|BTY_WRT, 0);

						SibAccess(BETTY_ADC_DATA, ChkBatterySM);
						break;

			case 2: 	/* wait for ADC complete */
						if(((SibDataInBuf>>15)&0x1)!=0)
						{
							ChkBatState++;
							BatAdcValue = ((SibDataInBuf>>5) &0x3FF) ;

						SibAccess( BETTY_ADC_CTRL
								   |TTP_ADC_AD0
								   |BTY_ADC_SYNC_ENA |BTY_VREFBYP_CON
								   |BTY_ADC_ENA 				|BTY_WRT, 0);

						SibAccess( BETTY_ADC_CTRL
								   |TTP_ADC_AD0
								   |BTY_ADC_SYNC_ENA |BTY_VREFBYP_CON
								   |BTY_ADC_ENA |BTY_ADC_START	|BTY_WRT, 0);

							SibAccess(BETTY_ADC_DATA, ChkBatterySM);

						}
						else	/* do one more time */
						{
							SibAccess(BETTY_ADC_DATA, ChkBatterySM);
						}
						break;




			case 3: 	/* wait for ADC complete */
						if(((SibDataInBuf>>15)&0x1)!=0)
						{
							ChkBatState++;
							if (BatAdcValue < ((SibDataInBuf>>5) &0x3FF))
									BatAdcValue = ((SibDataInBuf>>5) &0x3FF) ;

						SibAccess( BETTY_ADC_CTRL
								   |TTP_ADC_AD0
								   |BTY_ADC_SYNC_ENA |BTY_VREFBYP_CON
								   |BTY_ADC_ENA 				|BTY_WRT, 0);

						SibAccess( BETTY_ADC_CTRL
								   |TTP_ADC_AD0
								   |BTY_ADC_SYNC_ENA |BTY_VREFBYP_CON
								   |BTY_ADC_ENA |BTY_ADC_START	|BTY_WRT, 0);

							SibAccess(BETTY_ADC_DATA, ChkBatterySM);

						}
						else	/* do one more time */
						{
							SibAccess(BETTY_ADC_DATA, ChkBatterySM);
						}
						break;

			case 4: 	/* wait for ADC complete */
						if(((SibDataInBuf>>15)&0x1)!=0)
						{
							ChkBatState=0;
							SibEngaged --;
							if (BatAdcValue < ((SibDataInBuf>>5) &0x3FF))
									BatAdcValue = ((SibDataInBuf>>5) &0x3FF) ;

							MsgAppendMsgInt((void*)PowerMgr,
											BAT_MAIN_LEVEL,
											(int)(BatAdcValue ),
											0, (void*)0 );

							SibAccess( BETTY_ADC_CTRL |BTY_ADC_SYNC_ENA
									  |TTP_ADC_AD0 |BTY_VREFBYP_CON
									  |BTY_WRT, 0);
						}
						else	/* do one more time */
						{
							SibAccess(BETTY_ADC_DATA, ChkBatterySM);
						}
						break;

			default:	break;
		}

		if (ChkBatState==0 )
		{
			AdcQueue[AdcQPtr.front].status=ADC_Q_COMPLETE;
			BtyAdcAccessHandler();
		}
	}
}




WORD CheckBatteryLow()
{

    if(BatteryLevelStatus== BAT_NORMAL)         return (0);
    if(BatteryLevelStatus== BAT_LOW   )         return (1);

}


WORD  CheckBatteryEmpty()
{
    return (BatteryEmpty);
}


WORD BatChkMainLevelBakup()
{
    return(BatMapLifeTime(BatteryLowestLevel));
}


BatLowPowerManagement()
{
	OffEL();
	HwAudioOff();
}

int BatteryLifeMeter[11] = {  18, 30, 38, 42, 48, 50, 55, 63, 70, 80, 82 };

int BatMapLifeTime( int v )
{
	WORD i;
	WORD level;

    level = (100*(v - BAT_LEVEL_EMPTY) + 10*(BAT_LEVEL_MAX -BAT_LEVEL_EMPTY)/2 )
			/((BAT_LEVEL_MAX -BAT_LEVEL_EMPTY)) ;

     for (i=0; i<11; i++)  if ( BatteryLifeMeter[i] > level )   return (i-(i%2));
     return (10);

}

#define A_LCD_FR		  70
#define A_LCD_LINERATE	  (A_LCD_FR * (LCD_WIDTH +1 ))
#define A_LCD_VIDRATE	  ( (A_LCD_CP/A_LCD_LINERATE) -1 )

void AHTurnLcdOff(void)
{
	CPU->REG_VIDEO_CTRL_2 = VIDRATE(A_LCD_VIDRATE) | HORZVAL(LCD_WIDTH / 4 - 1)
							| LINEVAL(LCD_HEIGHT+30);

	while ( ((CPU->REG_VIDEO_CTRL_1 >>22)&0x3ff) <161  ) ;
	CPU->REG_VIDEO_CTRL_1 |= ENFREEZEFRAME;
	while ( ((CPU->REG_VIDEO_CTRL_1 >>22)&0x3ff)!=0  ) ;

	CPU->REG_VIDEO_CTRL_1 &= ~ENVID;
	CPU->REG_CLK_CTRL &= ~ENVIDCLK;
	CPU->REG_VIDEO_CTRL_1 &= ~DISPON;
	CPU->REG_MFIO_SEL |= (1<<3);
	CPU->REG_MFIO_DIRECTN |= (1<<3);
	CPU->REG_MFIO_DATA_OUT &= ~(1<<3);

	CPU->REG_VIDEO_CTRL_2 = VIDRATE(A_LCD_VIDRATE) | HORZVAL(LCD_WIDTH / 4 - 1)
							| LINEVAL(LCD_HEIGHT-1);
	CPU->REG_VIDEO_CTRL_1 &= ~ENFREEZEFRAME;
}

WORD  LcdTurnedOff(void)
{
	CPU->REG_MFIO_SEL |= (1<<3);
	CPU->REG_MFIO_DIRECTN |= (1<<3);

	if( ((CPU->REG_MFIO_DATA_OUT >>3)&0x1) ==1)    return (0);
	else return (1);
}


/* should not enable this interrupt */
PosPwrIntISR()
{
//    printf("\n\t+ PwrInt   ");

}



/* should not enable this interrupt */
PosPwrOkIntISR()
{
//    printf("\n\t\t\t\t+ PwrOkInt ");

}

NegOnButnISR()
{
//    printf("\n\t\t\t\t+ PwrOkInt ");

}

/////////////////////////////////////////

#endif


