/*
================================ COPYRIGHT ================================
The contents of this file are subject to the VTech Informations Ltd. License
of VT-OS Ver. 1.1 operating system (the "License"); you may not use this
file except in compliance with the License.

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
for the specific language governing rights and limitations under the License.

The Original Code is VT-OS Ver. 1.1 Operating System, released
on October 1st, 1999

The Initial Developer of the Original Code is VTech Informations Ltd.  All
codes are Copyright (C) VTech Informations Ltd. 1999.  All Rights Reserved.
===========================================================================
*/

/*
===========================================================================
File        :   dev_sib.c
Author(s)   :   Andrew Hui
Company     :   VTech Informations Ltd.
Project     :   Helio
Date:       :   October 1st, 1999
Purpose:    :   SIB low level driver
Revision    :   1.1
Note        :   23/09/98   Prepared by Andrew Hui (AH)
               23/09/98   V1.0 release (AH)
               23/10/98   V1.1 release (AH)
               09/12/98   V1.2 release (AH) <Optimize to O3>
               15/12/98   V1.3 release (AH) <Optimize to O3>
               05/01/99   V1.4 release (AH) <Optimize to O3>
               09/04/99   V1.5 release (AH) <Optimize to O3>
               12/04/99   V1.6 release (AH) <Optimize to O3>
               13/04/99   V1.7 release (AH) <Optimize to O3>
               17/05/99   V1.8 release (AH) <Optimize to O3>
               29/06/99   V1.9 release (AH) <Optimize to O3>
               08/09/99   V2.0 SibCapture and SibRelease added
===========================================================================
*/



#include "platform.h"
#ifdef PR31700

//#define DEBUG_SIB_DEV_DRIVER 1


#include    "include\datatype.h"

#include    "include\pr31700s.h"
#include    "include\pr31700c.h"

#include    "include\dev_bety.h"

#include    "include\dev_sib.h"
#include    "include\dev_pen.h"
#include    "include\penmgr.h"

volatile WORD SibEngaged   ;
extern int PenUpCount;

/* call back the caller routine */
void (*SibAccessReqAck)();  /* SIB access grant acknowledge; init to zero*/
void (*SibReadReqAck)();    /* Return from SibRead : Internal use only */
void (*SibWriteReqAck)();   /* Return from SibWrite: Internal use only */

void (*BtyAdcRdyReqAck)();  /* ADC ready acknowledge */
void (*BtyPenDownAck)();    /* Pen Down acknowledge  */

/* state variable of sib routine */
volatile WORD SibIntState;      /* SIB write state should init to zero */
volatile WORD SibRdState;       /* SIB read  state should init to zero */
volatile WORD SibWrtState;      /* SIB write state should init to zero */


/* Sib access variable*/
volatile WORD SibDataInBuf;
volatile WORD SibReg;           /* set by read or write caller routine */

extern WORD    PenLastSerial;

/*  SibQueueItem.status */
#define SIB_Q_WAITING       0   /* set when queue in */
#define SIB_Q_PROCESSING    1   /* set in SIBSF0INT  */
#define SIB_Q_PROCESSED     2   /* set in SIBSF1INT and it is a Rd command */
#define SIB_Q_COMPLETE      3   /* set in SIBSF1INT, clear in Rd()/Wr()    */

#define ADC_Q_WAITING       0   /* set when queue in */
#define ADC_Q_PROCESSING    1   /* set in SIBSF0INT  */
#define ADC_Q_PROCESSED     2   /* set in SIBSF1INT and it is a Rd command */
#define ADC_Q_COMPLETE      3   /* set in SIBSF1INT, clear in Rd()/Wr()    */

#define SIB_Q_FULL         -1
#define SIB_Q_OK            1

#define ADC_Q_FULL         -1
#define ADC_Q_OK            1

WORD SibSndandTelSize;  //AH990908 store the Dma TEL_SIZE, SND_SIZE

SibQueuePointer     SibQPtr;
SibQueueItem        SibQueue[SIB_Q_SIZE];

AdcQueuePointer     AdcQPtr;
AdcQueueItem        AdcQueue[ADC_Q_SIZE];

#define SIB_CB_ST  0
#define SIB_CB_WR  1
#define SIB_CB_RD  2

void dSibWriteData( void (*caller)());

int wait1frame;

WORD SibAccess(WORD cmd, void (*caller)() );
void dSibAccess ();

WORD SibAccess(WORD cmd, void (*caller)() )
{

    if(SibQPtr.rear == -1 )  /* no item in queue */
    {
        SibQPtr.rear  = 0;
        SibQPtr.front = 0;

        SibQueue[SibQPtr.rear].cmd    = cmd;
        SibQueue[SibQPtr.rear].caller = caller;
        SibQueue[SibQPtr.rear].status = SIB_Q_WAITING;

        SibQPtr.rear = SibQPtr.front;

        if (SibAccessReqAck==0) SibAccessReqAck = dSibAccess;

        BtyClearSF1Int();  /* enable SF0  */ BtyEnableSF1Int();  /* enable SF0  */

    }
    else
    {
        if(((SibQPtr.rear+1)%SIB_Q_SIZE) == SibQPtr.front )  /* Q full */
            return (SIB_Q_FULL);
        else    /* Q not full and have more then one item */
        {

            SibQPtr.rear = (SibQPtr.rear+1) % SIB_Q_SIZE;
            SibQueue[SibQPtr.rear].cmd    = cmd;
            SibQueue[SibQPtr.rear].caller = caller;
            SibQueue[SibQPtr.rear].status = SIB_Q_WAITING;
        }
    }

    return(SIB_Q_OK);
}


void dSibAccess ()
{
    WORD tPtr;
    WORD ServiceDone;

    ServiceDone=SIB_CB_ST;

    if ( (SibQPtr.rear == -1) )       /* empty queue  */
    {
        return ;
    }

    tPtr=SibQPtr.front;
    switch (SibQueue[tPtr].status)
    {
        case SIB_Q_PROCESSING :

            /* kill prev job first */
            if ( (SibQueue[tPtr].cmd & BTY_WRT)==0 ) /* read */
            {
                ServiceDone =SIB_CB_RD;
            }
            else    /* write */
            {
                ServiceDone =SIB_CB_WR;
            }

            SibQueue[tPtr].status = SIB_Q_COMPLETE;

            if ( tPtr == SibQPtr.rear ) break;
                tPtr = (tPtr+1) % SIB_Q_SIZE ;

        case SIB_Q_WAITING :

            if (SibQueue[tPtr].status != SIB_Q_WAITING) break;
                SibQueue[tPtr].status = SIB_Q_PROCESSING;

            SibReg =  SibQueue[tPtr].cmd ;

            if (((SibQueue[tPtr].cmd)&BTY_WRT)==0 )
            {
                SibReadReqAck = dSibAccess ;
                SibRead();
            }
            else
            {
               SibWriteReqAck = dSibAccess ;
               SibWrite();
            }
            break;
        default :
            break;
    }
    tPtr=SibQPtr.front;

    if((ServiceDone ==SIB_CB_RD))     dSibReadData(SibQueue[tPtr].caller);
    else
        if((ServiceDone ==SIB_CB_WR)) dSibWriteData(SibQueue[tPtr].caller);

    if ( (SibQPtr.rear == -1) )       /* empty queue  */
    {
        BtyDisableSF0Int();  /* enable SF1  */
        BtyDisableSF1Int();  /* enable SF1  */
    }
}



/*===========================================================================
        Prototype         : void dSibReadData(int cmd, void (*callback)() )
        Purpose           : Provide a low level read request service for SIB
        Scope             : Internal
        Input parameters  : cmd : command to issue
                            callback() :call back who when done

        Output parameters : SibDataInBuf      : data read from betty
        Return value      : None
        Comments          : call back the caller
=============================================================================*/

WORD dSibReadData( void (*caller)())  /* call by dSibAccess() only */
{                                     /* terminate interrupt if no more job */
    static WORD data;

    if (SibQPtr.rear == SibQPtr.front)
    {
        SibQPtr.rear = -1 ;
        SibQPtr.front=  0 ;
        BtyDisableSF0Int(); /* disable SF0 */
        BtyDisableSF1Int(); /* disable SF1 */
    }
    else    SibQPtr.front = (SibQPtr.front+1) % SIB_Q_SIZE;

    if ((int)caller!=0)     caller();
}

/*===========================================================================
        Prototype         : void dSibWriteData(void (*caller)())
        Purpose           : Provide a low level write request service for SIB
        Scope             : Internal
        Input parameters  : caller() : tell the caller the request served
        Output parameters : None
        Return value      : None
        Comments          : call back the caller
=============================================================================*/

void dSibWriteData(void (*caller)())
{


    if (SibQPtr.rear == SibQPtr.front) /* last item in Q  */
    {
        SibQPtr.rear = -1 ;     /* queue become empty */
        SibQPtr.front=  0 ;
        BtyDisableSF0Int(); /* disable SF0 */
        BtyDisableSF1Int(); /* disable SF1 */
    }
    else SibQPtr.front = (SibQPtr.front+1) % SIB_Q_SIZE;

    if ((int)caller!=0)    caller();

}

BettyQueueFlush()
{
    SibEngaged =0;

    SibQPtr.front=  0;
    SibQPtr.rear = -1;

    AdcQPtr.front=  0;
    AdcQPtr.rear = -1;

    BtyDisableSF0Int(); /* disable SF0 */
    BtyDisableSF1Int(); /* disable SF1 */

    BtyClearSF0Int(); /* disable SF0 */
    BtyClearSF1Int(); /* disable SF1 */
}


/*===========================================================================
        Prototype         :  void SibInit()
        Purpose           :  initialize SIBus
        Scope             :
        Input parameters  :
        Output parameters :
        Return value      :
=============================================================================*/
void SibInit()
{
    SibRdState =0; SibWrtState =0; SibReadReqAck =0; SibWriteReqAck =0;
    SibAccessReqAck=0; SibIntState =0; SibEngaged =0; SibQPtr.front= 0;
    SibQPtr.rear = -1; AdcQPtr.front= 0; AdcQPtr.rear = -1;

    CPU->REG_POWER_CTRL |= DIVMOD;

    BtyDisableSF0Int();
    BtyDisableSF1Int();
    BtyClearSF0Int();
    BtyClearSF1Int();

    CPU->REG_SIB_CTRL &=  ~(SCLKDIV(7));
    CPU->REG_SIB_CTRL |=  SCLKDIV(0);

    /* config SIB clock */
    CPU->REG_CLK_CTRL &=  ~(SIBMCLKDIV(7));
    CPU->REG_CLK_CTRL |=  SIBMCLKDIR | ENSIBMCLK | SIBMCLKDIV(2)| CSERSEL;

    CPU->REG_SIB_CTRL |=  ENSIB |ENSF0 |ENSF1 ;      /* Enable subframe 0 */

}


/*===========================================================================
        Prototype         : void dSibWrite(int cmd)
        Purpose           : Provide a write request service for SIB
        Scope             : Internal
        Input parameters  : cmd : command to issue
        Output parameters : SibAccessReqAck() : call back if SIB SF0 interrupt
        Return value      : None
        Comments          : call back the caller
=============================================================================*/
void dSibWrite(int cmd )
{
    BtyWriteReg(cmd);
}


BtyAdcAccessHandler ()  /* execute job or delete job from Q, Within interrupt */
{
    /* there must be at least one job in the queue */
    if (AdcQueue[AdcQPtr.front].status==ADC_Q_COMPLETE)     /* delete */
    {
        if (AdcQPtr.rear == AdcQPtr.front) /* last item in Q  */
        {
            AdcQPtr.rear = -1 ;     /* queue become empty */
            AdcQPtr.front=  0 ;
        }
        else
            AdcQPtr.front = (AdcQPtr.front+1) % ADC_Q_SIZE;
    }

    if (AdcQPtr.rear != -1)     /* queue still have waiting job */
    {
        if (AdcQueue[AdcQPtr.front].status==ADC_Q_WAITING)
        {
            AdcQueue[AdcQPtr.front].status=ADC_Q_PROCESSING;

            AdcQueue[AdcQPtr.front].calling();  /* execution */

        }
    }
}




WORD BtyAdcAccess(void (*calling)())
{

    if(AdcQPtr.rear == -1 )  /* no item in queue */
    {
        AdcQPtr.rear = AdcQPtr.front;
        AdcQueue[AdcQPtr.rear].calling  = calling;
        AdcQueue[AdcQPtr.rear].status   = ADC_Q_WAITING;
    }
    else
    {
        if(((AdcQPtr.rear+1)%ADC_Q_SIZE) == AdcQPtr.front )  /* Q full */
        {
            return (ADC_Q_FULL);
        }
        else    /* Q not full and have more then one item */
        {
            AdcQPtr.rear = (AdcQPtr.rear+1) % ADC_Q_SIZE;
            AdcQueue[AdcQPtr.rear].calling   = calling;
            AdcQueue[AdcQPtr.rear].status    = ADC_Q_WAITING;
        }
    }

    if (AdcQPtr.rear == AdcQPtr.front) /* only one job in Q  */
        BtyAdcAccessHandler ();



    return(ADC_Q_OK);
}

/*===========================================================================
        Prototype         : void BtyAdcIsr()
        Purpose           : Interrupt service routine for ADC complete
        Scope             : Internal
        Input parameters  : None
        Output parameters : None
        Return value      : None
        Comments          : Not use in the version of driver
=============================================================================*/

/* ADC ready interrupt service routine */
void BtyAdcIsr()    /* cannot use because no interrupt to CPU,  */
                    /* but interrupt set internally             */
{
    // to be implement by caller
}


/*===========================================================================
        Prototype         : Void BtyIoIsr()
        Purpose           : Interrupt service routine for I/O port of Betty
        Scope             : Internal
        Input parameters  : None
        Output parameters : None
        Return value      : None
        Comments          : Not use in the version of driver
=============================================================================*/

volatile WORD BtyIoIsrState;

void BtyIoIsr()
{
    // to be implement by caller
    ;
}


/*===========================================================================
        Prototype         : void SibIntHandler()
        Purpose           :
        Scope             :
        Input parameters  :
        Output parameters :
        Return value      :
=============================================================================*/

/* Resolve Betty Interrupt */
void SibIntHandler()
{
    volatile register  WORD IntSrc;

    switch (SibIntState)
    {


        case 0:
                SibIntState ++;
                SibEngaged ++;
                BtyAdcAccess(SibIntHandler);
                break;

        case 1:
                SibIntState ++;
                SibAccess( BETTY_INT_STATUS, SibIntHandler);
                break;

        case 2:
                SibIntState =0;
                SibEngaged  --;

                IntSrc =  SibDataInBuf ;
                // Post message to pen manager => pen down
                MsgAppendMsgInt(PenMgr, PEN_DOWN, 0, 0, (void*)(PenMsgSerial));
                PenMsgSerial ++;
                PenUpCount = 0;

                if (PenTmrHandle ==0)   /* enable timer interrupt */
                {
                    PenTmrHandle = TmrIntEnableIntPen(PEN_SAMPLE_PERIOD, PenTmrIntHandler);
                    PenTmrAction=PEN_TIMER_START;
                    PenDint=1;  // disable
                }
                else
                {
                    PenDint=1;  // disable
                }
                if (PenState==0)    PenGetxy();     /* read x,y */

                break;

    }

    if (SibIntState==0)
    {
        AdcQueue[AdcQPtr.front].status=ADC_Q_COMPLETE;
        BtyAdcAccessHandler();
    }
}



void SibIrqPosISR()
{
#ifdef DEFUALT_SIB_SPEED
    if ((SibIntState!=0) || (PenState!=0) || (PenTmrAction!=PEN_TIMER_STOPED) )
    { /* if not yet finish previous service */
        ;
    }
    else
#endif
        SibIntHandler();          /* clear calling routine */


}

/*===========================================================================
        Prototype         : void SibRead()
        Purpose           : Provide a read request service for SIB
        Scope             : Internal
        Input parameters  : SibReadReqAck()
                            SibReg            : Bty reg to be read
                            SibRdState
        Output parameters : SibAccessReqAck() : call back if SIB SF0 interrupt
                            SibDataInBuf      : data read from betty
        Return value      : None
        Comments          : call back the caller
=============================================================================*/


void SibRead ()  /* trigger by SF1 */
{
    switch (SibRdState)
    {
        case 2  :
                    if (  SibDataInBuf==(CPU->REG_SIB_SF0_STATUS & 0xffff))
                    {
                        SibRdState=0;               /* read complete      */
                        SibDataInBuf=CPU->REG_SIB_SF0_STATUS;
                        SibDataInBuf=SibDataInBuf & 0x0ffff;
                        SibAccessReqAck=0;
                    }
                    else
                    {
                        SibAccessReqAck=SibRead;
                        SibDataInBuf=CPU->REG_SIB_SF0_STATUS;
                        SibDataInBuf=SibDataInBuf & 0x0ffff;
                    }
                    break;

        case 1  :   SibRdState++;               /* read complete      */
                    SibAccessReqAck=SibRead;
                    SibDataInBuf=CPU->REG_SIB_SF0_STATUS;
                    SibDataInBuf=SibDataInBuf & 0x0ffff;
                    break;

        case 0  :   SibAccessReqAck=SibRead;    /* read request       */
                    BtyReadReg(SibReg);
                    SibRdState ++;
                    break;
    }

    if (SibRdState==0)    SibReadReqAck();

}



/*===========================================================================
        Prototype         : void SibWrite()
        Purpose           : Provide a write request service for SIB
        Scope             : Internal
        Input parameters  : SibWriteReqAck()
                            SibReg            : Bty + data to be sent
                            SibRdState
        Output parameters : SibAccessReqAck() : call back if SIB SF0 interrupt
        Return value      : None
        Comments          : call back the caller
=============================================================================*/

void SibWrite()  /* trigger by SF0 */
{

    switch (SibWrtState)
    {
                    /* set terminals to waiting pendown mode */
        case 1  :
                    SibWrtState=0;              /* write complete           */
                    SibAccessReqAck=0;
                    break;

        case 0  :   SibAccessReqAck=SibWrite;   /* write data               */
                    BtyWriteReg(SibReg);        /* send out write command   */
                    SibWrtState ++;
                    break;
    }

    if (SibWrtState==0)    SibWriteReqAck();
}


/*===========================================================================
        Prototype         : void SF0Isr()
        Purpose           : Interrupt service routine for SIB's SF0 interrupt
        Scope             : OS
        Input parameters  : None
        Output parameters : None
        Return value      : None
        Comments          : Call back the consumer of SIB
=============================================================================*/

/* SF0 interrupt service routine */
void SF0Isr()   /* write */
{
    if( SibAccessReqAck != 0 ) /*  request */SibAccessReqAck();
    BtyClearSF0Int(); /* disable SF0 */
}


/*===========================================================================
        Prototype         : void SF1Isr()
        Purpose           : Interrupt service routine for SIB's SF1 interrupt
        Scope             : OS
        Input parameters  : None
        Output parameters : None
        Return value      : None
        Comments          : Call back the consumer of SIB
=============================================================================*/

void SF1Isr()   /* read /write  */
{

    if( SibAccessReqAck != 0 ) /*  request */SibAccessReqAck();
    BtyClearSF1Int(); /* disable SF0 */

}

volatile int SibReadDataComplete;
volatile int SibWriteDataComplete;

/* API for Application to access SIB */
int SibReadData( int BtyReg )
{
    DIS_GBL_INT;
    SibEngaged ++;
    SibReadDataComplete=0;
    SibAccess( BtyReg ,SibReadDataHandle );
    ENA_GBL_INT;

    while ( (volatile)SibReadDataComplete != 1 ) ;
    return (SibDataInBuf);

}


void SibWriteData(int BtyReg, int cmd)
{
    DIS_GBL_INT;
    SibEngaged ++;
    SibWriteDataComplete=0;
    SibAccess( BtyReg | cmd, SibWriteDataHandle );
    ENA_GBL_INT;

    while ( (volatile)SibWriteDataComplete !=1 ) ;
}


void SibReadDataHandle(void)     /* called by SibAccess in interrupt */
{
    SibEngaged --;
    SibReadDataComplete=1;
}


void SibWriteDataHandle(void)    /* called by SibAccess in interrupt */
{
    SibEngaged --;
    SibWriteDataComplete=1;
}


void SibCapture()
{
    DIS_GBL_INT;
    SibEngaged ++;
    ENA_GBL_INT;
}


void SibRelease()
{
    DIS_GBL_INT;
    SibEngaged --;
    ENA_GBL_INT;
}

#endif

