/*
================================ COPYRIGHT ================================
The contents of this file are subject to the VTech Informations Ltd. License
of VT-OS Ver. 1.1 operating system (the "License"); you may not use this 
file except in compliance with the License.  
 
Software distributed under the License is distributed on an "AS IS" basis, 
WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
for the specific language governing rights and limitations under the License.
 
The Original Code is VT-OS Ver. 1.1 Operating System, released 
on October 1st, 1999

The Initial Developer of the Original Code is VTech Informations Ltd.  All 
codes are Copyright (C) VTech Informations Ltd. 1999.  All Rights Reserved.
===========================================================================
*/

/*
===========================================================================
File        :   dev_snd.c
Author(s)   :   Andrew Hui
Company     :   VTech Informations Ltd.
Project     :   Helio 
Date:	    :   October 1st, 1999
Purpose:	:   Sound Low level Driver
Revision    :   1.1
Note        :   18/08/98   Prepared by Andrew Hui (AH)                     
                05/01/99   2.0 change sib access method                    
                05/08/99   2.1 Change play vol resp. to battery level      
                08/09/99   2.2 Fixed Physical address masking              
===========================================================================
*/              




#include    "include\std.h"
#include    "include\ttp.h"
#include    "include\pr31700s.h"
#include    "include\pr31700c.h"

#include    "include\datatype.h"
#include    "include\dev_snd.h"
#include    "include\intc.h"
#include    "include\dev_sib.h"
#include    "include\dev_bety.h"
#include    "include\dev_pwr.h"



/* define the physical address masking and offset for both bank0 and bank1 */
#define DRAM_PHY_ADDR_MASK    0x07ffffff
#define DRAM_PHY_ADDR_OFFSET  0x04000000

extern void SndMgr();
extern WORD BatteryLowestLevel;


/* sib.h */
extern WORD SibReg;
extern void (*SibAccessReqAck)();  /* SIB access grant acknowledge; init to zero*/
extern void (*SibReadReqAck)();    /* Return from SibRead : Internal use only */
extern void (*SibWriteReqAck)();   /* Return from SibWrite: Internal use only */
extern WORD   SibIntState;
extern WORD   SibRdState;
extern WORD   SibWrtState;
extern WORD   SibDataInBuf;

/* snd_dd.h */
void SndInit(WORD SamplingRate, WORD DataLength, WORD InputGain, WORD OutputAttn );
void SndInitSM();
void SndEnable(WORD Channel);
void SndEffEnable(WORD Channel);                                //JN
void SndEnable_Int(WORD Channel);                               //JN & AH

void SndEnableSM();
void SndVolumeSM();
void SndPlyVolumeSM();
void SndPlyVolume();


volatile SHORT sndibuf1[SND_BUF_SIZE];
volatile SHORT sndibuf2[SND_BUF_SIZE];
volatile SHORT sndobuf1[SND_BUF_SIZE];
volatile SHORT sndobuf2[SND_BUF_SIZE];

volatile WORD     SndAction;             /* init to zero */
volatile WORD     frame_no ;              /* init to zero */

volatile WORD   SndInBufPA;
volatile WORD   SndOutBufPA;
volatile WORD   SndWrkInBufPA;
volatile WORD   SndWrkOutBufPA;
volatile WORD   SndInBuf;
volatile WORD   SndOutBuf;
volatile WORD   SndWrkInBuf;
volatile WORD   SndWrkOutBuf;

WORD   SM_par1, SM_par2, SM_par3, SM_par4, SM_par5;

volatile WORD SndInitState;         /* must in to zero */
volatile WORD SndEnableState;

volatile WORD samplerate;

/*-----------------------------------------------------------------------------
  Prototype     :   SndStartDma(WORD Action)
  Purpose       :   Setup DMA for sound input / Output
  Scope         :   OS
  Input         :   Action : SND_RECORDING / SND_PLAYING
  Output        :   None
  Return        :   none
  Description   :   Three global pointer variable SndInBuf, SndWrkInBuf,
                    SndWrkOutBuf, SndOutBuf are used to implement buffer
                    swapping.
-----------------------------------------------------------------------------*/
extern WORD SibSndandTelSize;
void SndSetBufferSize(WORD Size)
{
    SibSndandTelSize  &= ~SNDSIZEMASK ;
    SibSndandTelSize  |=  SNDSIZE(Size) ;
    CPU->REG_SIB_SIZE = SibSndandTelSize ;
}


SndStartDma(WORD Action)
{
    if (Action & (SND_PLAYING |SND_RECORDING))
    {
        /* disable DMA */
        CPU->REG_SIB_DMA_CTRL &= ~(ENDMATXSND | ENDMARXSND) ;  /* disable DMA */

        SndSetBufferSize((SND_BUF_SIZE*2) -1);

        frame_no ++;
    }


    if (Action &SND_PLAYING)
    {
        SndTxDmaAddr(SndOutBufPA); /*must be physical address*/
        /* start DMA */
        if (!(Action &SND_RECORDING))
        {
            CPU->REG_SIB_DMA_CTRL |= ENDMATXSND |SNDBUFF1TIME;
        }
    }


    if (Action &SND_RECORDING)
    {
        SndRxDmaAddr(SndInBufPA); /*must be physical address*/
        /* start DMA */
        if (!(Action &SND_PLAYING))
        {
            CPU->REG_SIB_DMA_CTRL |= ENDMARXSND |SNDBUFF1TIME;
        }
        else   /* Duplex */
        {
            CPU->REG_SIB_DMA_CTRL |= ENDMATXSND |ENDMARXSND |SNDBUFF1TIME;
        }
    }
}



/*-----------------------------------------------------------------------------
  Prototype     :   SndIntHandler()
  Purpose       :   Swap DMA buffers sound
  Scope         :   OS
  Input         :   None
  Output        :   None
  Return        :   none
  Description   :   Only responsible to post message and handling buffer
                    swapping
-----------------------------------------------------------------------------*/

SndIntHandler()
{
    volatile WORD     tmp;

        if (SndAction & (SND_PLAYING |SND_RECORDING))
        {
            /* disable DMA */
            CPU->REG_SIB_DMA_CTRL &= ~(ENDMATXSND | ENDMARXSND) ;  /* disable DMA */
            SndSetBufferSize((SND_BUF_SIZE*2) -1);
            frame_no ++;
        }

        if (SndAction &SND_PLAYING)
        {
            tmp = SndOutBuf;            /* swap out buffer */
            SndOutBuf =  SndWrkOutBuf;
            SndWrkOutBuf=tmp;

            tmp = SndOutBufPA;         /* swap out buffer (Physical Address)*/
            SndOutBufPA =  SndWrkOutBufPA;
            SndWrkOutBufPA=tmp;

            SndTxDmaAddr(SndOutBufPA); /*must be physical address*/
            if (!(SndAction &SND_RECORDING))
                CPU->REG_SIB_DMA_CTRL |= ENDMATXSND |SNDBUFF1TIME;  /* start DMA */
        }


        if (SndAction & SND_RECORDING)
        {
            tmp = SndInBuf;            /* swap in  buffer */
            SndInBuf = SndWrkInBuf;
            SndWrkInBuf = tmp;

            tmp = SndInBufPA;          /* swap in buffer (Physical Address)*/
            SndInBufPA = SndWrkInBufPA;
            SndWrkInBufPA = tmp;

            SndRxDmaAddr(SndInBufPA); /*must be physical address*/
            if (!(SndAction &SND_PLAYING))
                CPU->REG_SIB_DMA_CTRL |= ENDMARXSND |SNDBUFF1TIME;  /* start DMA */
            else   /* Duplex */
                CPU->REG_SIB_DMA_CTRL |= ENDMATXSND |ENDMARXSND |SNDBUFF1TIME;  /* start DMA */
        }

      if (SndAction)
      {
          SndMgr();
      }

}

/* Assume the following devices had been initialized :
    - Betty
    - Sib
    - Clock to Sib
                                                  */


/*-----------------------------------------------------------------------------
  Prototype     :   void SndVolume(WORD InputGain, WORD OutputAttn)
  Purpose       :   Init Betty for sound I/O
  Scope         :   OS
  Input         :   SamplingRate : Sampling rate (Hz)
                    DataLength   : 8bit  or 16 bit data mode
                    InputGain    : Input Gain of MIC
                    OutputAttn   : Output Attenuation of SPEAKER
  Output        :   None
  Return        :   none
  Description   :
-----------------------------------------------------------------------------*/
void SndVolume(WORD InputGain, WORD OutputAttn )
{

    SM_par3 = InputGain;
    SM_par4 = OutputAttn;

    SndInitState=0;

    DIS_GBL_INT;
    SndVolumeSM();
    ENA_GBL_INT;

    while (SndInitState!=0)   ;     /* wait until Sound Channel init complete */


}

void SndPlyVolume(WORD InputGain, WORD OutputAttn )
{
    SM_par3 = InputGain;
    SM_par4 = OutputAttn;

    SndInitState=0;

    DIS_GBL_INT;
    SndPlyVolumeSM();
    ENA_GBL_INT;

    while (SndInitState!=0)   ;     /* wait until Sound Channel init complete */
}


WORD AdjustSndLevel;


/* for Voice Playback */
int SndLevelMeter[11]    = {  15,    /*  0 */
                              15,    /*  1 */
                              15,    /*  2 */
                              14,    /*  3 */
                              11,    /*  4 */
                              7,     /*  5 */
                              5,     /*  6 */
                              3,     /*  7 */
                              2,     /*  8 */
                              2,     /*  9 */
                              2 };   /* 10 */


/* for Click Sound */
int SndClickLevelMeter[11]  = { 15,  /*  0 */
                              15,    /*  1 */
                              12,    /*  2 */
                              9,     /*  3 */
                              8,     /*  4 */
                              7,     /*  5 */
                              4,     /*  6 */
                              4,     /*  7 */
                              4,     /*  8 */
                              4,     /*  9 */
                              4 };   /* 10 */




/*-----------------------------------------------------------------------------
  Prototype     :   void SndVolumeSM()
  Purpose       :   State machine of SndPlayVolumeSM()
  Scope         :   Internal
  Input         :   None
  Output        :   None
  Return        :   none
  Description   :
-----------------------------------------------------------------------------*/


void SndVolumeSM()
{
    register tmp;
    switch(SndInitState)
    {
        case 0  :   SndInitState ++;
                    SibEngaged ++;
                    /* setup Sound on Betty Chip */
                    SibAccess((BETTY_AUD_CTRL_A |BTY_AUD_DIV(samplerate)
                              | BTY_AUD_GAIN(SM_par3) |BTY_WRT), 0);
                    SibAccess(BETTY_AUD_CTRL_B , SndVolumeSM);
                    break;


        case 1 :    /* set sound config  on Betty */
                    SndInitState ++;

                    AdjustSndLevel = BatMapLifeTime(BatteryLowestLevel);
                    if (SndClickLevelMeter[AdjustSndLevel] >SM_par4)
                        tmp =SndClickLevelMeter[AdjustSndLevel];
                    else
                        tmp = SM_par4;


                    SibAccess(( BETTY_AUD_CTRL_B
                             |(SibDataInBuf & ~(BTY_AUD_ATT(BTY_AUD_ATT_MASK)))
                             |BTY_AUD_ATT(tmp)
                             |BTY_WRT),SndVolumeSM);
                    break;

        case 2 :    SndInitState =0;
                    SibEngaged --;
                    break;
    }
}


void SndPlyVolumeSM()
{
    register tmp;

    switch(SndInitState)
    {

        case 0 :    SibEngaged ++;
                    SndInitState++;
                    BtyAdcAccess(SndPlyVolumeSM);
                    break;

        case 1:     SndInitState++;
                    SibAccess( BETTY_ADC_CTRL |BTY_ADC_SYNC_ENA
                               |TTP_ADC_AD0 |BTY_VREFBYP_CON
                               |BTY_ADC_ENA
                               |BTY_WRT, 0);

                    /* set input gain and sampling rate */
                    SibAccess((BETTY_AUD_CTRL_A |BTY_AUD_DIV(samplerate)
                              | BTY_AUD_GAIN(SM_par3) |BTY_WRT), 0);

                    SibAccess( BETTY_ADC_CTRL |BTY_ADC_SYNC_ENA
                               |TTP_ADC_AD0 |BTY_VREFBYP_CON
                               |BTY_ADC_ENA |BTY_ADC_START
                               |BTY_WRT, 0);

                    SibAccess(BETTY_ADC_DATA, SndPlyVolumeSM);
                    break;


        case 2:     /* wait for ADC complete */
                    if(((SibDataInBuf>>15)&0x1)!=0)
                    {
                        SndInitState++;

                    AdjustSndLevel = BatMapLifeTime( (SibDataInBuf>>5) &0x3FF);

                        SibAccess( BETTY_ADC_CTRL |BTY_ADC_SYNC_ENA
                                  |TTP_ADC_AD0 |BTY_VREFBYP_CON
                                  |BTY_WRT, 0);

                        SibAccess(BETTY_AUD_CTRL_B , SndPlyVolumeSM);
                    }
                    else    /* do one more time */
                    {
                        SibAccess(BETTY_ADC_DATA, SndPlyVolumeSM);
                    }
                    break;



        case 3 :    /* set sound config  on Betty */
                    SndInitState ++;

                    if (SndLevelMeter[AdjustSndLevel] >SM_par4)
                        tmp =SndLevelMeter[AdjustSndLevel];
                    else
                        tmp = SM_par4;

                    SibAccess(( BETTY_AUD_CTRL_B
                             |(SibDataInBuf & ~(BTY_AUD_ATT(BTY_AUD_ATT_MASK)))
                             |BTY_AUD_ATT(tmp)
                             |BTY_WRT),SndPlyVolumeSM);
                    break;

        case 4 :    SndInitState =0;
                    SibEngaged --;
                    break;
    }

    if (SndInitState==0 )
    {
        AdcQueue[AdcQPtr.front].status=ADC_Q_COMPLETE;
        BtyAdcAccessHandler();
    }


}


/*-----------------------------------------------------------------------------
  Prototype     :   void SndInit(WORD SamplingRate, WORD DataLength, WORD InputGain, WORD OutputAttn )
  Purpose       :   Init Betty for sound I/O
  Scope         :   OS
  Input         :   SamplingRate : Sampling rate (Hz)
                    DataLength   : 8bit  or 16 bit data mode
                    InputGain    : Input Gain of MIC
                    OutputAttn   : Output Attenuation of SPEAKER
  Output        :   None
  Return        :   none
  Description   :
-----------------------------------------------------------------------------*/
void SndInit(WORD SamplingRate, WORD DataLength, WORD InputGain, WORD OutputAttn )
{
    SM_par1 = SamplingRate;
    samplerate = SamplingRate;
    SM_par2 = DataLength;
    SM_par3 = InputGain;
    SM_par4 = OutputAttn;

    SndInitState=0;
    HwAudioInit();
    DIS_GBL_INT;
    SndInitSM();
    ENA_GBL_INT;

    while (SndInitState!=0)
    {
        __asm(" nop");
        __asm(" nop");
        __asm(" nop");
    }
}

/*-----------------------------------------------------------------------------
  Prototype     :   void SndInitSM()
  Purpose       :   State machine of SndInit()
  Scope         :   Internal
  Input         :   None
  Output        :   None
  Return        :   none
  Description   :
-----------------------------------------------------------------------------*/

void SndInitSM()
{
    switch(SndInitState)
    {
        case 0  :   /* Setup Sound on CPU :  SIB register  */
                    SndInitState ++;
                    SibEngaged ++;

                    /* Physical Address is set to SDRAM = 0x04xx xxx */
                    SndInBufPA     = ((WORD)(&sndibuf1[0]) & DRAM_PHY_ADDR_MASK) + DRAM_PHY_ADDR_OFFSET;
                    SndWrkInBufPA  = ((WORD)(&sndibuf2[0]) & DRAM_PHY_ADDR_MASK) + DRAM_PHY_ADDR_OFFSET;
                    SndOutBufPA    = ((WORD)(&sndobuf1[0]) & DRAM_PHY_ADDR_MASK) + DRAM_PHY_ADDR_OFFSET;
                    SndWrkOutBufPA = ((WORD)(&sndobuf2[0]) & DRAM_PHY_ADDR_MASK) + DRAM_PHY_ADDR_OFFSET;
                    SndInBuf     = (WORD)(&sndibuf1[0]);
                    SndWrkInBuf  = (WORD)(&sndibuf2[0]);
                    SndOutBuf    = (WORD)(&sndobuf1[0]);
                    SndWrkOutBuf = (WORD)(&sndobuf2[0]);

                    CPU->REG_SIB_CTRL |= SNDMONO |SM_par2 |RMONOSNDIN
                                        |SNDFSDIV(SM_par1 -1) ;

                    SibAccess((BETTY_AUD_CTRL_A |BTY_AUD_DIV(SM_par1)
                               |BTY_AUD_GAIN(SM_par3)|BTY_WRT), 0);

                    /* set sound config  on Betty */
                    SibAccess(( BETTY_AUD_CTRL_B |BTY_AUD_ATT(SM_par4)|BTY_WRT),SndInitSM);
                    break;

        case 1 :    SndInitState =0;
                    SibEngaged --;
                    break;
    }
}



/*-----------------------------------------------------------------------------
  Prototype     :   void SndEnable(WORD Channel)
  Purpose       :   Enable Sound I/O
  Scope         :   Internal
  Input         :   None
  Output        :   None
  Return        :   none
  Description   :   SndDisable() can be done by : setting Channel to zero to
                    disable both channel
-----------------------------------------------------------------------------*/

void SndEffEnable2(WORD Channel)
{
     SndEnable( Channel);
}

void SndEnable(WORD Channel)
{
    SM_par1         =0;
    SndEnableState  =0;

    if (Channel &MIC)
    {
        HwAudioOff();
        SM_par1 |= (BTY_AUD_IN_ENA | BTY_AUD_MUTE);
        SM_par1 &= ~(BTY_AUD_OUT_ENA);
    }

    if (Channel ==0)
    {
        HwAudioOff();
        SM_par1 |=  BTY_AUD_MUTE;
    }

    if (Channel &SPEAKER)
    {
        HwAudioOn();
        SM_par1 &= ~(BTY_AUD_IN_ENA | BTY_AUD_MUTE);
        SM_par1 |= (BTY_AUD_OUT_ENA);

    }

    DIS_GBL_INT;
    SndEnableSM();
    ENA_GBL_INT;

    while (SndEnableState!=0)    /* wait until Sound Channel init complete */
    {
        ;
    }
    if (SndAction==0)      CPU->REG_INT_ENABLE_1 &= ~SND1_0INT;     /* Enable SND interrupt */

}

void SndEnable_Int(WORD Channel)
{
    SM_par1         =0;
    SndEnableState  =0;

    if (Channel &MIC)
    {
        HwAudioOff();
        SM_par1 |= (BTY_AUD_IN_ENA | BTY_AUD_MUTE);
        SM_par1 &= ~(BTY_AUD_OUT_ENA);
    }

    if (Channel ==0)
    {
            HwAudioOff();
        SM_par1 |=  BTY_AUD_MUTE;
    }

    if (Channel &SPEAKER)
    {
        HwAudioOn();
        SM_par1 &= ~(BTY_AUD_IN_ENA | BTY_AUD_MUTE);
        SM_par1 |= (BTY_AUD_OUT_ENA);

    }

    SndEnableSM();
    if (SndAction==0)      CPU->REG_INT_ENABLE_1 &= ~SND1_0INT;     /* Enable SND interrupt */

}




/*-----------------------------------------------------------------------------
  Prototype     :   void SndEnableSM()
  Purpose       :   State machine of SndEnable()
  Scope         :   Internal
  Input         :   None
  Output        :   None
  Return        :   none
  Description   :
-----------------------------------------------------------------------------*/


void SndEnableSM()
{
    switch (SndEnableState)
    {
        case 0:
                SibEngaged ++;
                SndEnableState ++;
                CPU->REG_SIB_CTRL |= ENSND;
                SibAccess((BETTY_AUD_CTRL_B ),SndEnableSM);
                break;

        case 1:
                SndEnableState ++;
                SibAccess((BETTY_AUD_CTRL_B
                           | (SibDataInBuf & (BTY_AUD_ATT(BTY_AUD_ATT_MASK)))
                           | SM_par1 | BTY_WRT), SndEnableSM ) ;
                break;

        case 2:
                CPU->REG_INT_ENABLE_1 |= SND1_0INT;     /* Enable SND interrupt */
                SndEnableState = 0;
                SibEngaged --;
                break;
     }


}




