/*
================================ COPYRIGHT ================================
The contents of this file are subject to the VTech Informations Ltd. License
of VT-OS Ver. 1.1 operating system (the "License"); you may not use this 
file except in compliance with the License.  
 
Software distributed under the License is distributed on an "AS IS" basis, 
WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
for the specific language governing rights and limitations under the License.
 
The Original Code is VT-OS Ver. 1.1 Operating System, released 
on October 1st, 1999

The Initial Developer of the Original Code is VTech Informations Ltd.  All 
codes are Copyright (C) VTech Informations Ltd. 1999.  All Rights Reserved.
===========================================================================
*/

/*
===========================================================================
File        :   dev_tel.c
Author(s)   :   Andrew Hui
Company     :   VTech Informations Ltd.
Project     :   Helio 
Date:	    :   October 1st, 1999
Purpose:	:   Telecom codec low level driver
Revision    :   1.1
Note        :   27/08/99   1.0 Totally rewritten from Telecom.c (by TC).   
                           This new version is adopt to use new sib        
                           accessing method and system call.               
                08/09/99   1.1 Fixed Buffer size problem                   
===========================================================================
*/              


#include "include\dev_bety.h"
#include "include\intc.h"
#include "include\pr31700s.h"
#include "include\pr31700c.h"
#include "include\dev_sib.h"
#include "include\dev_tel.h"

/* DMA Buffers */
SHORT tel_rx_buf[TEL_BLOCK_SIZE*4];/* Buffer for modem */
SHORT tel_tx_buf[TEL_BLOCK_SIZE*4];

SHORT *ext_tel_rx_buf;  // Added by KN
SHORT *ext_tel_tx_buf;  // Added by KN

/* define the physical address masking and offset for both bank0 and bank1 */
#define DRAM_PHY_ADDR_MASK    0x07ffffff
#define DRAM_PHY_ADDR_OFFSET  0x04000000


/* this routine should be supplied by Applications */

void TelInit()
{
        TelOnHook();/* release the line */

        TelSetSamplingRate(TEL_SAMPLING_RATE);

        TelSetDmaAddress(tel_rx_buf, tel_tx_buf); // get information from system

        TelSetBufferSize(TEL_BLOCK_SIZE*4-1);
        TelSetUpBty(TEL_SAMPLING_RATE);
}


void TelOffHook()    /* Engage the Tel Line */
{
    CPU->REG_MFIO_DIRECTN  |= (1 << 5);
//    CPU->REG_MFIO_DATA_OUT &= ~(1 << 5);
    CPU->REG_MFIO_DATA_OUT |=  (1 << 5);
}

void TelOnHook()   /* Release the Tel Line */
{
    CPU->REG_MFIO_DIRECTN  |=  (1 << 5);
//    CPU->REG_MFIO_DATA_OUT |=  (1 << 5);
    CPU->REG_MFIO_DATA_OUT &= ~(1 << 5);
}

BOOLEAN TelHookStatus()
{
	// return true if off hook
	return (UWORD) (CPU->REG_MFIO_DATA_OUT & (1 << 5)) ? TRUE:FALSE;
}


void TelSetDmaAddress(SHORT *inbuf, SHORT *outbuf)
{
        ext_tel_rx_buf = inbuf;
        ext_tel_tx_buf = outbuf;

    CPU->REG_SIB_TEL_RX_START = ((WORD)(inbuf)  & DRAM_PHY_ADDR_MASK) + DRAM_PHY_ADDR_OFFSET;
    CPU->REG_SIB_TEL_TX_START = ((WORD)(outbuf) & DRAM_PHY_ADDR_MASK) + DRAM_PHY_ADDR_OFFSET;
}


void TelSetSamplingRate(WORD SamplingRate)
{
    CPU->REG_SIB_CTRL &= ~TELFSDIVMASK;
    CPU->REG_SIB_CTRL |= TEL16 | TELFSDIV(TEL_SAMPLE_RATE(SamplingRate)-1);
}

extern WORD SibSndandTelSize;
void TelSetBufferSize(WORD Size)
{
    SibSndandTelSize  &= ~TELSIZEMASK ;
    SibSndandTelSize  |= TELSIZE(Size) ;
    CPU->REG_SIB_SIZE = SibSndandTelSize ;
}



void TelSetUpBty(WORD SamplingRate )
{

    SibWriteData( BETTY_TEL_CTRL_A,
                 (BTY_TEL_DIV(TEL_SAMPLE_RATE(SamplingRate))|BTY_WRT) );

    SibWriteData(BETTY_TEL_CTRL_B, ( BTY_TEL_SIDE_ENA |BTY_TEL_OUT_ENA
                                    |BTY_TEL_IN_ENA   |BTY_WRT));
}

void TelStartDma()
{
    TelEnable();
    CPU->REG_SIB_DMA_CTRL |= ENDMARXTEL | ENDMATXTEL;
    /* Enable Interrupts */
    CPU->REG_INT_ENABLE_1 |= TEL0_5INT | TEL1_0INT;
}


void TelStopDma()
{
    CPU->REG_SIB_DMA_CTRL &= ~(ENDMARXTEL | ENDMATXTEL);
    /* Enable Interrupts */
    CPU->REG_INT_ENABLE_1 &= ~(TEL0_5INT | TEL1_0INT);
    TelDisable();
}

void TelEnable()
{
        CPU->REG_SIB_CTRL |= ENTEL;
}


void TelDisable()
{
        CPU->REG_SIB_CTRL &= ~ENTEL;
    SibWriteData(BETTY_TEL_CTRL_B, (BTY_WRT));
}


// Added by KN
void TelGetBufferPtr(SHORT **rx_buf, SHORT **tx_buf)
{
        *rx_buf = ext_tel_rx_buf;
        *tx_buf = ext_tel_tx_buf;
}
