/*
================================ COPYRIGHT ================================
The contents of this file are subject to the VTech Informations Ltd. License
of VT-OS Ver. 1.1 operating system (the "License"); you may not use this 
file except in compliance with the License.  
 
Software distributed under the License is distributed on an "AS IS" basis, 
WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
for the specific language governing rights and limitations under the License.
 
The Original Code is VT-OS Ver. 1.1 Operating System, released 
on October 1st, 1999

The Initial Developer of the Original Code is VTech Informations Ltd.  All 
codes are Copyright (C) VTech Informations Ltd. 1999.  All Rights Reserved.
===========================================================================
*/

/*
===========================================================================
File        :   penmgr.c
Author(s)   :   Andrew Hui
Company     :   VTech Informations Ltd.
Project     :   Helio 
Date:	    :   October 1st, 1999
Purpose:	:   Pen Manager
Revision    :   1.1
Note        :   23/09/98   Prepared by Andrew Hui (AH)					   
 				23/09/98   V1.0 release (AH)							   
 				09/12/98   V1.1 release (AH)  (Optimize to O3)			   
 				01/03/99   V1.2 release (AH)  Noise fileter added		   
 				19/03/99   V1.3 release (AH)  Noise fileter fixed		   
 				09/04/99   V1.4 release (AH)  Noise fileter fixed		   
 				12/04/99   V1.5 release (AH)  Noise fileter fixed		   
 				15/04/99   V1.6 release (AH)  Noise fileter fixed		   
 				24/05/99   V1.7 release (AH)  Calibration fixed 		   
 				29/06/99   V1.8 release (AH)  Calibration fixed 		   
===========================================================================
*/              


#include "platform.h"
#ifdef PR31700

//#define DEBUG_PEN_MGR 1		  /* remark this line to remove printing */
//#define NO_PEN_FILTER_NOISE
#define PEN_FILTER_NOISE

#include "include\penmgr.h"
#include "include\dev_pen.h"
#include "include\datatype.h"

volatile WORD PenMsgSrvID;
volatile WORD PenMsgPeriod;

PenBufType	PenEvntQ[PEN_EVNT_Q_SIZE];	 /* X, Y, PenStatus = PEN_UP/PEN_MOVE */

WORD PenMgrMode;   /* PEN_RECOGNITION, PEN_CALIBRATION, PEN_NORMAL */
extern WORD    PenLastSerial;

WORD PenMinX;
WORD PenMinY;
WORD PenMaxX;
WORD PenMaxY;

WORD PenEvntPtr  ;

WORD PenFilterMask [PEN_FILTER_SAMPLE] = {1,2,2};	/* sum >2 = pen move */

#ifdef PEN_FILTER_NOISE
int Iabs(int a)
{
    if (a<0)    return (-a);
    else        return a;
}
#endif

extern	int PenMessageSent;

volatile PenUpSerial;
volatile pen_xy_rotate;

volatile WORD pu_x, pu_y;
volatile WORD last_x, last_y;

int PenMgrPenDownCount;
                             
void PenMgr(MsgType *Pmsg)
{
	WORD weight;
	WORD x,y;
	WORD PMcount;
	static	WORD PM_state=PM_IDLE;
	int PendingPenUp;

#ifdef PEN_FILTER_NOISE
	volatile WORD x_cor[3];
	volatile WORD y_cor[3];
#endif

	WORD i;

	DIS_GBL_INT;

	if ( ((WORD)(Pmsg->msgPBP)==0)	)  /* serial no */
		PenUpSerial=0;

	if ((PenUpSerial>0) && ((WORD)(Pmsg->msgPBP)>(PenUpSerial))&&((WORD)(Pmsg->msgPBP) != 0))  /* last pen up serial no */
	{
			/* ignore pending message */
		ENA_GBL_INT;

	}
	else
	{

		if ( ((WORD)(Pmsg->msgPBP)==0)	)  /* serial no */
			PenUpSerial=0;

		ENA_GBL_INT;


#ifdef PEN_FILTER_NOISE
		for ( i=0; i<3; i++)
		{
			x_cor[i] =0;
			y_cor[i] =0;
		}
#endif
		if ((PenMgrMode==PEN_NORMAL)||(PenMgrMode==PEN_CALIBRATION))
		{

			switch((WORD)(Pmsg->msgID))  /* only PEN_MOVE have x,y information */
			{
				case PEN_DOWN : //printf(" Dn ");
								PenMgrPenDownCount=3;
								break;

				case PEN_MOVE : //printf(" Mv ");

                                DIS_GBL_INT;
                                PenMsgSrvID--;
                                ENA_GBL_INT;

								PenEvntQ[PenEvntPtr%PEN_EVNT_Q_SIZE].x		= Pmsg->para1; /* x_ADC_value; */
								PenEvntQ[PenEvntPtr%PEN_EVNT_Q_SIZE].y		= Pmsg->para2; /* y_ADC_value; */
								PenEvntQ[PenEvntPtr%PEN_EVNT_Q_SIZE].status = PEN_MOVE;
								PenEvntPtr++;
								break;

				case PEN_UP   : //printf(" Up ");

								if (PenMgrPenDownCount>0)
									PenMgrPenDownCount--;

								if ( (PenEvntPtr< PEN_FILTER_SAMPLE-1)
									  &&(PenMgrPenDownCount==0) )  /* U, M U */
								{
									DIS_GBL_INT;
									PenEvntPtr=0; /* reset */
									PenTmrAction = PEN_TIMER_STOP;
									PenUpSerial = (WORD)(Pmsg->msgPBP); //AH990412
									ENA_GBL_INT;
								}
								else
								{
									//printf(" Mu ");
									PenEvntQ[PenEvntPtr%PEN_EVNT_Q_SIZE].status = PEN_UP;
									PenEvntPtr++;
								}
								break;
			}


			weight=0;

			/* classify Pen UP/DN/MV */
			if ((PenEvntPtr >= PEN_FILTER_SAMPLE))	/* enough samples to process */
			{
				for (i=0; i<PEN_FILTER_SAMPLE; i++)
					weight += PenFilterMask[i] * PenEvntQ[(PenEvntPtr - PEN_FILTER_SAMPLE+i)%PEN_EVNT_Q_SIZE].status;
			}


			switch (PM_state)
			{
				case PM_IDLE:  // printf("i");
								x		=0;
								y		=0;
								PMcount =0;


											 /* Pen_Moving */
								if ((weight > PEN_MOVE_THRESHOLD)
//									 ||  ( PenMgrPenDownCount!=0) )
									|| ( weight >0) )
								{

									if (weight > PEN_MOVE_THRESHOLD)
										PenMgrPenDownCount=3;

				#ifdef NO_PEN_FILTER_NOISE
									for (i=0; i<PEN_FILTER_SAMPLE; i++) /* find averaged x, y */
									{
										if (PenEvntQ[(PenEvntPtr-PEN_FILTER_SAMPLE+i)%PEN_EVNT_Q_SIZE].status ==PEN_MOVE)
										{
											x += PenEvntQ [(PenEvntPtr -PEN_FILTER_SAMPLE+i)%PEN_EVNT_Q_SIZE].x;
											y += PenEvntQ [(PenEvntPtr -PEN_FILTER_SAMPLE+i)%PEN_EVNT_Q_SIZE].y;
											PMcount++;
										}
									}
									x /= PMcount;
									y /= PMcount;

				#endif	//NO_PEN_FILTER_NOISE


				#ifdef PEN_FILTER_NOISE


									for (i=0; i<PEN_FILTER_SAMPLE; i++) /* find averaged x, y */
									{
										if (PenEvntQ[(PenEvntPtr-PEN_FILTER_SAMPLE+i)%PEN_EVNT_Q_SIZE].status ==PEN_MOVE)
										{
											x_cor[PMcount] = PenEvntQ [(PenEvntPtr -PEN_FILTER_SAMPLE+i)%PEN_EVNT_Q_SIZE].x;
											y_cor[PMcount] = PenEvntQ [(PenEvntPtr -PEN_FILTER_SAMPLE+i)%PEN_EVNT_Q_SIZE].y;
											PMcount++;
										}
									}
//									  printf("\n::: <<<PD %d >>> " , PMcount);
									if ( PMcount > 1 )
									{
										x = PenFindCoordinate(x_cor[0], x_cor[1],x_cor[2], last_x, PMcount );
										y = PenFindCoordinate(y_cor[0], y_cor[1],y_cor[2], last_y, PMcount );
										last_x = x;
										last_y = y;
									}
									else
									{
										if (PMcount==1)
										{
											x = x_cor[0];
											y = y_cor[0];
										}
										else
										{
											x = last_x ;
											y = last_y ;
										}
									}

				#endif	// PEN_FILTER_NOISE
									if (PenMgrMode==PEN_NORMAL)
									{
										/* convert ADC value to 0 - 159 */

										x= (x-PenMinX)*160/(PenMaxX-PenMinX);
										y= (y-PenMinY)*160/(PenMaxY-PenMinY);
														#ifdef DEBUG_PEN_MGR
														printf("\n::DN:::::::::PEN: %d, %d",x,y);
//														  printf("\n::DN::PEN: %d, %d",x,y);
														#endif

										if (pen_xy_rotate)
											EvtAppendEvt (PEN_EVENT, PEN_DOWN, y, 159-x, 0 );
										else
											EvtAppendEvt (PEN_EVENT, PEN_DOWN, x, y, 0 );
                                    }

									if (PenMgrMode==PEN_CALIBRATION)
									{
														#ifdef DEBUG_PEN_MGR
														printf("\n\t\t\t\t\t\t\tPEN: %d, %d",x,y);
														#endif
											EvtAppendEvt (PEN_EVENT, PEN_DOWN, x, y, 0 );
									}

														#ifdef DEBUG_PEN_MGR
														printf("\n\t\t\t\tPD= (%d, %d) , (%d, %d) ", x, y, pu_x, pu_y);
														#endif
										pu_x = x;
										pu_y = y;

										PM_state=PM_PM;
								}
								else
								{
									/* else stay in idle mode */
									if (PenEvntPtr > PEN_FILTER_SAMPLE-1)
									{
#ifdef DEBUG_PEN_MGR
										printf("\n\n=================\n\n");
#endif
										PenEvntPtr=0;
									}

								}
								break;

				case PM_PM	:  // printf("m");
								x		=0;
								y		=0;
								PMcount =0;

								if ((weight > PEN_MOVE_THRESHOLD) /* Pen Move */
									|| ( PenMgrPenDownCount!=0) )
								{

									if (weight > PEN_MOVE_THRESHOLD)
										PenMgrPenDownCount=3;
				#ifdef NO_PEN_FILTER_NOISE
									for (i=0; i<PEN_FILTER_SAMPLE; i++) /* find averaged x, y */
									{
										if (PenEvntQ[(PenEvntPtr-PEN_FILTER_SAMPLE+i)%PEN_EVNT_Q_SIZE].status ==PEN_MOVE)
										{
											x += PenEvntQ [(PenEvntPtr -PEN_FILTER_SAMPLE+i)%PEN_EVNT_Q_SIZE].x;
											y += PenEvntQ [(PenEvntPtr -PEN_FILTER_SAMPLE+i)%PEN_EVNT_Q_SIZE].y;
											PMcount++;
										}
									}
									x /= PMcount;
									y /= PMcount;


				#endif	//NO_PEN_FILTER_NOISE




				#ifdef PEN_FILTER_NOISE

									for (i=0; i<PEN_FILTER_SAMPLE; i++) /* find averaged x, y */
									{
										if (PenEvntQ[(PenEvntPtr-PEN_FILTER_SAMPLE+i)%PEN_EVNT_Q_SIZE].status ==PEN_MOVE)
										{
											x_cor[PMcount] =  PenEvntQ [(PenEvntPtr -PEN_FILTER_SAMPLE+i)%PEN_EVNT_Q_SIZE].x;
											y_cor[PMcount] =  PenEvntQ [(PenEvntPtr -PEN_FILTER_SAMPLE+i)%PEN_EVNT_Q_SIZE].y;
											PMcount++;
										}
									}

//									  printf(" <<< %d >>> " , PMcount);

									if (PMcount >1)
									{
										x = PenFindCoordinate(x_cor[0], x_cor[1],x_cor[2], last_x, PMcount );
										y = PenFindCoordinate(y_cor[0], y_cor[1],y_cor[2], last_y, PMcount );
										last_x = x;
										last_y = y;
									}
									else
									{
										if (PMcount==1)
										{
											x = x_cor[0];
											y = y_cor[0];
										}
										else
										{
											x = last_x ;
											y = last_y ;
										}
									}

													//			  if ( Iabs(x - last_x) >200) x = last_x;
													//			  if ( Iabs(y - last_y) >200) y = last_y;


				#endif	// PEN_FILTER_NOISE


									if (PenMgrMode==PEN_NORMAL)
									{
										/* convert ADC value to 0 - 159 */

										x= (x-PenMinX)*160/(PenMaxX-PenMinX);
										y= (y-PenMinY)*160/(PenMaxY-PenMinY);
                                                            #ifdef DEBUG_PEN_MGR
                                                            printf("\n::PM:::::::::PEN: %d, %d",x,y);
															#endif
										if (pen_xy_rotate)
											EvtAppendEvt (PEN_EVENT, PEN_MOVE, y, 159-x, 0 );
										else
											EvtAppendEvt (PEN_EVENT, PEN_MOVE, x, y, 0 );
									}


									if (PenMgrMode==PEN_CALIBRATION)
									{
															#ifdef DEBUG_PEN_MGR
															printf("\n\t\t\t\t\t\t\tPEN: %d, %d",x,y);
															#endif
											EvtAppendEvt (PEN_EVENT, PEN_MOVE, x, y, 0 );
									}
															#ifdef DEBUG_PEN_MGR
															printf("\n\t\t\t\tPM= (%d, %d) , (%d, %d) ", x, y, pu_x, pu_y);
															#endif
											pu_x = x;
											pu_y = y;
								}
								else /* Pen_Up, go back to idle mode */
								{

									DIS_GBL_INT;
									PenEvntPtr=0;
									PenTmrAction = PEN_TIMER_STOP;
									PenUpSerial = (WORD)(Pmsg->msgPBP); //AH990412
	//								  printf("Qq");
									ENA_GBL_INT;

									PM_state=PM_IDLE;

										if (PenMgrMode==PEN_NORMAL)
										{
											if (pen_xy_rotate)
												EvtAppendEvt (PEN_EVENT, PEN_UP, pu_y, 159-pu_x, 0 );
											else
												EvtAppendEvt (PEN_EVENT, PEN_UP, pu_x, pu_y, 0 );
															#ifdef DEBUG_PEN_MGR
															printf("\n::PU:::::::::PEN: %d, %d",x,y);
															#endif
										}

										if (PenMgrMode==PEN_CALIBRATION)
										{
														#ifdef DEBUG_PEN_MGR
														printf("\n\t\t\t\t\t\t\tPEN: %d, %d",x,y);
														#endif
												EvtAppendEvt (PEN_EVENT, PEN_UP, x, y, 0 );
										}

														#ifdef DEBUG_PEN_MGR
														printf("\n\t\t\t\tPU= (%d, %d) , (%d, %d) ", x, y, pu_x, pu_y);
														#endif
								}

								break;
			}
			/*	 else	ignore pending message */

		}




	}

}



void PenSetCalMode(void)
{
	PenMgrMode=PEN_CALIBRATION;
	return;
}

void PenSetNormalMode(void)
{
	PenMgrMode=PEN_NORMAL;
	return;
}



#ifdef PEN_FILTER_NOISE
PenFindCoordinate(int a1, int a2, int a3, int Last, int count )
{
	int dist[3];
	int index;
	int tmp;
	int i, j;

	dist[0] = a1;
	dist[1] = a2;
	dist[2] = a3;

	if (count==3)
	{
		for (i=0; i<count; i++)
			for (j=i; j<count; j++)
			{
				if (dist[i] >dist[j])
				{
					tmp = dist[i];
					dist[i] = dist[j];
					dist[j] = tmp;
				}
			}

		tmp = dist[1];
	}
	else
	{

		dist[2] = Last;

		for (i=0; i<count+1; i++)
			for (j=i; j<count+1; j++)
			{
				if (dist[i] >dist[j])
				{
					tmp = dist[i];
					dist[i] = dist[j];
					dist[j] = tmp;
				}
			}
		tmp = dist[1];
	}

	return (tmp);

}

#endif	 //PEN_FILTER_NOISE

#endif

