/*
================================ COPYRIGHT ================================
The contents of this file are subject to the VTech Informations Ltd. License
of VT-OS Ver. 1.1 operating system (the "License"); you may not use this 
file except in compliance with the License.  
 
Software distributed under the License is distributed on an "AS IS" basis, 
WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
for the specific language governing rights and limitations under the License.
 
The Original Code is VT-OS Ver. 1.1 Operating System, released 
on October 1st, 1999

The Initial Developer of the Original Code is VTech Informations Ltd.  All 
codes are Copyright (C) VTech Informations Ltd. 1999.  All Rights Reserved.
===========================================================================
*/

/*
===========================================================================
File        :   sndmgr.c
Author(s)   :   Jason Ngan
Company     :   VTech Informations Ltd.
Project     :   Helio 
Date:       :   October 1st, 1999
Purpose:    :   Sound Manager
Revision    :   1.1
Note        :   None
                      Modification History                            
 ==================================================================== 
 1/19/99    : On board initial version 1.0
 1/28/99    : SndCoFcounter/FFFrame was returned by SndPause() for app.
                reference.
 3/21/99    : 10 'zero' frame were added at the output buffer
                in order to eliminate the startup noise
 3/23/99    : REC_TIME & PLAY_TIME EVENTs were added for app. ref.
                This two events will be generated when it is recording
                or playing a record in every second.
 3/24/99    : 1.) SndPlaySndEffect()
            2.) SndDataPlaySndEffectStart()
            3.) SndDataPlaySndEffect()
            Three functions were created for playing system sound
            effect.
 6/11/99    : The polling property of playing 'long' sound effect
                ( max. up to 10) was added.
 6/30/99    : AGC feature was added.
 7/07/99    : Play sound effect (beep sound) from original source (non-
                compressed).
 7/13/99    : All the EvtAppendEvt() were replaced by MsgAppendMsgInt().
                No event appending was allowed in interrupt routine.
                                                                      
**********************************************************************
===========================================================================
*/              

#include    "include\std.h"
#include    "include\datatype.h"
#include    "include\intc.h"

#include    "include\mmu.h"
#include    "include\dm_os.h"

#include    "include\sndmgr.h"
#include    "include\dev_snd.h"
#include    "include\sndmevt.h"
#include    "include\system.h"

#define     EFF
//#define       DEBUG
//#define       DEBUG_MSG
//#define       DEBUG_SND_EFF

//*************************************
// ------- Recording with AGC
#define       AGC
//*************************************
// ------- Play beep sound form compressed samples
// ------- Change together with SndRes.c
//#define       PLAY_COM_BEEP
//*************************************

#ifdef      EFF
#include    "include\sndres.h"
#endif

static SHORT indexT[16] = {
  -1,  1, -1,  1,  0,  0,  0,  0,
   0,  0,  0,  0,  0,  0,  0,  0,
};

static SHORT s_sizeT[60] = {
    7, 8, 9, 10, 11, 12, 13, 14, 16, 17,
    19, 21, 23, 25, 28, 31, 34, 37, 41, 45,
    50, 55, 60, 66, 73, 80, 88, 97, 107, 118,
    130, 143, 157, 173, 190, 209, 230, 253, 279, 307,
    337, 371, 408, 449, 494, 544, 598, 658, 724, 796,
    876, 963, 1060, 1166, 1282, 1411, 1552, 1707, 1878, 2047
};

/* cosine table */
static SHORT COS[256]={
     2621,  2625,  2639,  2662,  2693,  2734,  2784,  2843,
     2910,  2987,  3073,  3167,  3270,  3382,  3502,  3631,
     3768,  3914,  4068,  4230,  4401,  4579,  4765,  4959,
     5161,  5370,  5587,  5811,  6042,  6280,  6525,  6777,
     7036,  7300,  7571,  7849,  8132,  8420,  8715,  9015,
     9320,  9630,  9945, 10264, 10588, 10917, 11249, 11586,
    11926, 12269, 12616, 12966, 13318, 13674, 14031, 14391,
    14753, 15117, 15482, 15849, 16216, 16585, 16954, 17324,
    17694, 18064, 18433, 18803, 19171, 19539, 19905, 20271,
    20634, 20996, 21356, 21714, 22069, 22422, 22772, 23118,
    23462, 23802, 24138, 24471, 24799, 25123, 25443, 25758,
    26068, 26373, 26673, 26967, 27256, 27539, 27816, 28087,
    28352, 28610, 28862, 29107, 29345, 29576, 29800, 30017,
    30226, 30428, 30622, 30808, 30987, 31157, 31319, 31473,
    31619, 31757, 31885, 32006, 32117, 32221, 32315, 32400,
    32477, 32545, 32603, 32653, 32694, 32726, 32748, 32762,
    32762, 32748, 32726, 32694, 32653, 32603, 32545, 32477,
    32400, 32315, 32221, 32117, 32006, 31885, 31757, 31619,
    31473, 31319, 31157, 30987, 30808, 30622, 30428, 30226,
    30017, 29800, 29576, 29345, 29107, 28862, 28610, 28352,
    28087, 27816, 27539, 27256, 26967, 26673, 26373, 26068,
    25758, 25443, 25123, 24799, 24471, 24138, 23802, 23462,
    23118, 22772, 22422, 22069, 21714, 21356, 20996, 20634,
    20271, 19905, 19539, 19171, 18803, 18433, 18064, 17694,
    17324, 16954, 16585, 16216, 15849, 15482, 15117, 14753,
    14391, 14031, 13674, 13318, 12966, 12616, 12269, 11926,
    11586, 11249, 10917, 10588, 10264,  9945,  9630,  9320,
     9015,  8715,  8420,  8132,  7849,  7571,  7300,  7036,
     6777,  6525,  6280,  6042,  5811,  5587,  5370,  5161,
     4959,  4765,  4579,  4401,  4230,  4068,  3914,  3768,
     3631,  3502,  3382,  3270,  3167,  3073,  2987,  2910,
     2843,  2784,  2734,  2693,  2662,  2639,  2625,  2621,
};

/*- Global variables for ADPCM coding storage ----------*/
SHORT   SndCoIbuf[WmN];                 /* Current step change buffer */
SHORT   SndCovP[WmN];               /* Current step change prediction buffer */
SHORT   SndCoIndex;                     /* Current step change index */

BYTE    *SndCoPtr;                      /* Code word pointer for decoding */
BYTE    SndCoBuf[1+TTCodeLen];          /* Code word buffer for encoding */

BYTE    SndCoABuf[50][1+TTCodeLen];     /* Code word buffer for encoding */


SHORT   SndIOChannel;                   /* Sound Input/Output Channel */

UWORD   EncodataIndex;
UWORD   TotalLen;

volatile UWORD  SndCoFcounter;          /* Sound coding frame counter */
volatile BYTE   SndPlayAction;          /* Should be volatiled as it is */
                                        /* an important reference in waiting state */
DatabaseID  SndEff_dbid;
RecordID    SndEff_rec_id;

DatabaseID  SndMgr_dbid;
RecordID    SndMgr_rec_id;
USHORT      SndMgr_field_num;
BYTE        *SndResAddr;
SHORT       *OSndResAddr;


UWORD       FreeMem;
USHORT      VolPara = 27;
USHORT      ClickPara = 23;

SHORT       Snd_Eff_After_Stop = -1;
BYTE        Alarm_Acc[10];

#ifdef  AGC
WORD        AGC_Co;
WORD        AGC_Co_Acc;
UWORD       AGC_Coefficient;
#endif

SHORT       GSndEffName;
SHORT       *OSndCoPtr;

// SndRes.h
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
extern BYTE SND_SHARP[];
extern BYTE SND_ZHARP[];
extern BYTE SND_BWOO[];
extern BYTE SND_DONEDONE[];
extern BYTE SND_VEEWOO[];
extern BYTE SND_KOOKOO[];
extern BYTE SND_ROOSTER[];
//*****************************
extern BYTE SND0_BEEP[];
extern BYTE SND1_BEEP[];
extern BYTE SND2_BEEP[];
extern BYTE SND3_BEEP[];
extern BYTE SND4_BEEP[];
extern BYTE SND5_BEEP[];
//*****************************
extern SHORT SND0_OBEEP[];
extern SHORT SND1_OBEEP[];
extern SHORT SND2_OBEEP[];
extern SHORT SND3_OBEEP[];
extern SHORT SND4_OBEEP[];
extern SHORT SND5_OBEEP[];
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++



/*****************************************************************
 * Function:    SndStop
 * Purpose:     Stop sound playing or recording
 * Scope:       Application/Internal
 * Input:       None
 * Output:      None
 * Return:      None
 * Comment:     User should consider whether they need to close the
                database or not after stopping the action
******************************************************************/
void SndStop(void)
{
    WORD    i;

    if ((SndPlayAction == SND_PLAY_EFF) || (SndAction == SND_STOP))
        return;

    SndPlayAction = SND_STOP;
    /*- Sound Device Driver Initialization -----------*/
    if (SndAction == SND_PAUSE)
        SndAction = SND_STOP;
    else
    {

        SndAction = SND_STOP;
        SndIOChannel = SND_STOP;
        if (SndEnableState==0)
            SndEnable(SndIOChannel);
    }

    /*- Global variables initialization ---------------*/
    SndCoIndex      = 0;
    SndCoFcounter   = 0;
    for(i=0; i<WmN; i++)
        SndCovP[i] = SndCoIbuf[i] = 0;


    SndVolume(SND_MAX_GAIN, (WORD)(SND_MAX_VOL - ClickPara));
    SndPlaySndEffect(99);

    //if (Snd_Eff_After_Stop > -1)
        //SndPlaySndEffect(Snd_Eff_After_Stop);

    //Snd_Eff_After_Stop = -1;
    AGC_Co = 64;

    return;
}

/*****************************************************************
 * Function:    SndPause
 * Purpose:     Pause sound playing or recording
 * Scope:       Application
 * Input:       None
 * Output:      None
 * Return:      SndCoFcounter/FFFrame       start time for applications(RErecord/REplay)
 * Comment:     None
******************************************************************/
UWORD SndPause(void)
{
    /*- Sound Device Driver Initialization -----------*/
    SndAction = SND_STOP;
    SndIOChannel = SND_STOP;
    if (SndEnableState==0)
        SndEnable(SndIOChannel);
    SndVolume(SND_MAX_GAIN, (WORD)(SND_MAX_VOL - ClickPara));

    SndAction = SND_PAUSE;

    return (SndCoFcounter/FFFrame);
}


/*****************************************************************
 * Function:    SndRec
 * Purpose:     Sound recording
 * Scope:       Application/Internal
 * Input:       dbid                database ID
                rec_id              record ID
                field_num           field number
 * Output:      None
 * Return:      TRUE                if success
                FALSE               if database or record not found
 * Comment:     None
******************************************************************/
BOOLEAN SndRec(DatabaseID dbid, RecordID rec_id, USHORT field_num)
{

    WORD    i;
    UWORD   el_status;

    while(SndPlayAction != SND_STOP)    ;                   /* Wait for the end of the snd effect */

//----------------------------------------------------------------------
    if (MemoryTotalFree() < FREE_MEM_BLOCK_LIMIT + 33)
    {
        MsgAppendMsgInt(SndMgrMsgDecode, SND_MGR_MSG, LOW_MEM, 0,NULL);
        return FALSE;
    }

    FreeMem = BLOCK_SIZE*(MemoryTotalFree() - FREE_MEM_BLOCK_LIMIT - 2);

//----------------------------------------------------------------------
    if (!SndDataHandler(dbid, rec_id, field_num))
        return FALSE;

    ELState(&el_status);
    if (el_status)
        OffEL();

    /*- Global variables initialization ---------------*/
    SndCoIndex          = 0;
    SndCoFcounter       = 0;
    for(i=0; i<WmN; i++)
        SndCovP[i] = SndCoIbuf[i] = 0;

    /*- Sound Device Driver Initialization -----------*/
    SndAction = SND_RECORDING;
    SndIOChannel = MIC;


    if (SndEnableState==0)
        SndEnable(SndIOChannel);

    for(i=0; i<SND_BUF_SIZE; i++)
    {
        sndibuf1[i]=0;
        sndibuf2[i]=0;
        sndobuf1[i]=0;
        sndobuf2[i]=0;
    }

    SndStartDma(SND_RECORDING);

    return TRUE;
}


/*****************************************************************
 * Function:    SndRERec
 * Purpose:     Resume sound recording
 * Scope:       Application/Internal
 * Input:       dbid                database ID
                rec_id              record ID
                field_num           field number
 * Output:      None
 * Return:      TRUE                if success
                FALSE               if database or record not found
 * Comment:     Resume recording should be called after SndPause API
******************************************************************/
BOOLEAN SndRERec(DatabaseID dbid, RecordID rec_id, USHORT field_num)
{

    WORD  i;
    UWORD   el_status;

//  while(SndPlayAction != SND_STOP); /* Wait for the end of the snd effect */

    if (!SndDataHandler(dbid, rec_id, field_num))
        return FALSE;

    ELState(&el_status);
    if (el_status)
        OffEL();
    /*- Resume Recording -----------------------------*/

    /*- Sound Device Driver Initialization -----------*/
    SndPlayAction = SND_PLAY_REC;
    SndAction = SND_RECORDING;
    SndIOChannel = MIC;
    if (SndEnableState==0)
        SndEnable(SndIOChannel);

    for(i=0; i<SND_BUF_SIZE; i++)
    {
        sndibuf1[i]=0;
        sndibuf2[i]=0;
        sndobuf1[i]=0;
        sndobuf2[i]=0;
    }
    SndStartDma(SND_RECORDING);

    return TRUE;

}

/*****************************************************************
 * Function:    SndPlay
 * Purpose:     Sound playing
 * Scope:       Application
 * Input:       1. start_time       The time (in second) to be started.
                2. dbid             Database ID of the input record.
                3. rec_id           Record ID of the input record.
                4. field_num        Field number of the input record.
 * Output:      None
 * Return:      TRUE                if success
                FALSE               if database or record not found
 * Comment:     None
******************************************************************/
BOOLEAN SndPlay(UWORD start_time, DatabaseID dbid, RecordID rec_id, USHORT field_num)
{

    WORD  i;
    UWORD   el_status;

    while(SndPlayAction != SND_STOP)    ;   /* Wait for the end of the snd effect */

    if (!SndDataHandler(dbid, rec_id, field_num))
        return FALSE;

    //SndVolume(SND_MAX_GAIN, (WORD)(SND_MAX_VOL - VolPara));
    SndPlyVolume(SND_MAX_GAIN, (WORD)(SND_MAX_VOL - VolPara));


    SndPlayAction = SND_PLAY_REC;
    /*- Start DEcoding -------------------------------*/
    /*- Global variables initialization ---------------*/
    SndCoIndex      = 0;
    SndCoFcounter   = start_time*FFFrame;
    for(i=0; i<WmN; i++)
        SndCovP[i] = SndCoIbuf[i] = 0;

    if (SndDataPlayStart(start_time) != TRUE)
    {
        SndAction = SND_PLAYING;
        return FALSE;
    }


    ELState(&el_status);
    if (el_status)
        OffEL();

    /*- Sound Device Driver Initialization -----------*/
    SndAction = SND_PLAYING;
    SndIOChannel = SPEAKER;
    if (SndEnableState==0)
        SndEnable(SndIOChannel);

    for(i=0; i<SND_BUF_SIZE; i++)
    {
        sndibuf1[i]=0;
        sndibuf2[i]=0;
        sndobuf1[i]=0;
        sndobuf2[i]=0;
    }
    SndStartDma(SND_PLAYING);

    return TRUE;
}

/*****************************************************************
 * Function:    SndREPlay
 * Purpose:     Resume sound playing
 * Scope:       Application/Internal
 * Input:       start_time          the time (position) to be started
                dbid                database ID
                rec_id              record ID
                field_num           field number
 * Output:      None
 * Return:      TRUE                if success
                FALSE               if database or record not found
 * Comment:     Resume playing should be called after SndPause API
******************************************************************/
BOOLEAN SndREPlay(UWORD start_time, DatabaseID dbid, RecordID rec_id, USHORT field_num)
{
    WORD  i;
    UWORD   el_status;

//  while(SndPlayAction != SND_STOP); /* Wait for the end of the snd effect */

    if (!SndDataHandler(dbid, rec_id, field_num))
        return FALSE;

    //SndVolume(SND_MAX_GAIN, (WORD)(SND_MAX_VOL - VolPara));
    SndPlyVolume(SND_MAX_GAIN, (WORD)(SND_MAX_VOL - VolPara));

    SndPlayAction = SND_PLAY_REC;
    /*- Resume Playing -------------------------------*/
    SndCoFcounter -= (SndCoFcounter%FFFrame);

    if (SndDataPlayStart(start_time) != TRUE)
    {
        SndAction = SND_PLAYING;
        return FALSE;
    }

    ELState(&el_status);
    if (el_status)
        OffEL();

    /*- Sound Device Driver Initialization -----------*/
    SndAction = SND_PLAYING;
    SndIOChannel = SPEAKER;
    if (SndEnableState==0)
        SndEnable(SndIOChannel);

    for(i=0; i<SND_BUF_SIZE; i++)
    {
        sndibuf1[i]=0;
        sndibuf2[i]=0;
        sndobuf1[i]=0;
        sndobuf2[i]=0;
    }
    SndStartDma(SND_PLAYING);


    return  SND_ACTION_SUCCESS;

}

/*****************************************************************
 * Function:    SndPlaySndEffect
 * Purpose:     Sound effect playing
 * Scope:       Application
 * Input:       snd_eff_name        sound effect name
 * Output:      None
 * Return:      TRUE                if success
                FALSE               if invalid input
 * Comment:     None
******************************************************************/
BOOLEAN SndPlaySndEffect(SHORT snd_eff_name)
{
    WORD  i;


    if (CheckBatteryLow())
    {
        return FALSE;
    }

    // ------------------------------------------------------------------------------
    // ------------------------------------------------------------------------------
    if (snd_eff_name == 99 || SndAction == SND_PAUSE)
        SndVolume(SND_MAX_GAIN, SND_MAX_VOL);
    else
    {
        if ((snd_eff_name < 0 ) || (snd_eff_name > NUM_SND_EFF-1) || ClickPara == 0)
        {
            return FALSE;
        }

        if ((SndPlayAction != SND_STOP) && (snd_eff_name>-1) && (snd_eff_name < NUM_SPC_SND_EFF))
        {
            //SndVolume(SND_MAX_GAIN, (WORD)(SND_MAX_VOL - VolPara));
            SndPlyVolume(SND_MAX_GAIN, (WORD)(SND_MAX_VOL - VolPara));

            i = 0;
            while((Alarm_Acc[i] > -1) && i<10)
                i++;
            if (i < 10)
                Alarm_Acc[i] = (BYTE)snd_eff_name;

            return FALSE;
        }

        if ((SndAction == SND_PLAY_REC)||(SndAction == SND_PLAY_EFF))
        {
            //Snd_Eff_After_Stop = snd_eff_name;
            return FALSE;
        }

        if (snd_eff_name < NUM_SPC_SND_EFF)
            //SndVolume(SND_MAX_GAIN, (WORD)(SND_MAX_VOL - VolPara));
            SndPlyVolume(SND_MAX_GAIN, (WORD)(SND_MAX_VOL - VolPara));
        else
            SndVolume(SND_MAX_GAIN, (WORD)(SND_MAX_VOL - ClickPara));
    }
    // ------------------------------------------------------------------------------
    // ------------------------------------------------------------------------------


    SndPlayAction = SND_PLAY_EFF;
    /*- Start DEcoding -------------------------------*/
    /*- Global variables initialization ---------------*/
    /*
    SndCoIndex      = 0;
    SndCoFcounter   = 0;
    for(i=0; i<WmN; i++)
        SndCovP[i] = SndCoIbuf[i] = 0;
    */

    if (snd_eff_name == 99)
        SndDataPlaySndEffectStart(SNDRES5_BEEP);
    else
        SndDataPlaySndEffectStart(snd_eff_name);


    /*- Sound Device Driver Initialization -----------*/
    SndAction = SND_PLAYING;
    SndIOChannel = SPEAKER;
    if (SndEnableState==0)
        SndEnable(SndIOChannel);

    for(i=0; i<SND_BUF_SIZE; i++)
    {
        sndibuf1[i]=0;
        sndibuf2[i]=0;
        sndobuf1[i]=0;
        sndobuf2[i]=0;
    }
    SndStartDma(SND_PLAYING);


    return TRUE;
}


/*****************************************************************
 * Function:    SndPlayAdjSndEffect
 * Purpose:     Sound effect playing with adjustable volume control
 * Scope:       Application
 * Input:       snd_eff_name        sound effect name
                click_vol           click sound volume
 * Output:      None
 * Return:      TRUE                if success
                FALSE               if invalid input
 * Comment:     None
******************************************************************/
BOOLEAN SndPlayAdjSndEffect(SHORT snd_eff_name, SHORT click_vol)
{
    WORD  i;

    if (CheckBatteryLow() ||(snd_eff_name < NUM_SPC_SND_EFF) || !click_vol) return FALSE;

// ---------------------------------------------------------------------------------------**


    if ((SndAction == SND_PLAY_REC)||(SndAction == SND_PLAY_EFF) || (SndAction == SND_PAUSE))
    {
        Snd_Eff_After_Stop = snd_eff_name;
        return FALSE;
    }

    SndVolume(SND_MAX_GAIN, (WORD)(SND_MAX_VOL - click_vol));

    SndPlayAction = SND_PLAY_EFF;
    /*- Start DEcoding -------------------------------*/
    /*- Global variables initialization ---------------*/
    SndCoIndex      = 0;
    SndCoFcounter   = 0;
    for(i=0; i<WmN; i++)        SndCovP[i] = SndCoIbuf[i] = 0;

    SndDataPlaySndEffectStart(snd_eff_name);                  

    /*- Sound Device Driver Initialization -----------*/
    SndAction = SND_PLAYING;
    SndIOChannel = SPEAKER;
    if (SndEnableState==0)
        SndEnable(SndIOChannel);

    for(i=0; i<SND_BUF_SIZE; i++)
    {
        sndibuf1[i]=0;
        sndibuf2[i]=0;
        sndobuf1[i]=0;
        sndobuf2[i]=0;
    }

    SndStartDma(SND_PLAYING);

    return TRUE;
}


/*****************************************************************
 * Function:    SndMgr
 * Purpose:     Called by sound device driver, handle encoding/decoding
                process
 * Scope:       Internal
 * Input:       None
 * Output:      None
 * Return:      None
 * Comment:     None
******************************************************************/
void SndMgr(void)
{

    switch(SndIOChannel)
    {

        case MIC:
            Ader();
            break;

        case SPEAKER:
            switch(SndPlayAction)
            {
                case SND_PLAY_EFF:

#ifdef  PLAY_COM_BEEP
                    Kder();
                    if (SndDataPlayComSndEffect() != TRUE)
                        SndIOChannel = SND_STOP;
#else
                    if (GSndEffName < NUM_SPC_SND_EFF)
                    {
                        Kder();
                        if (SndDataPlayComSndEffect() != TRUE)
                            SndIOChannel = SND_STOP;
                    }
                    else
                    {

                        if (SndDataPlaySndEffect() != TRUE)
                            SndIOChannel = SND_STOP;
                    }
#endif
                    break;
                default:
                    Kder();
                    if (SndDataPlay() != TRUE)
                        SndIOChannel = SND_STOP;
                    break;
            }

            break;

        case SND_STOP:
            break;
    }

    return;
}

/*****************************************************************
 * Function:    SndDataHandler
 * Purpose:     This handler is used for sound recording or playing
                by handling different database, record and field
 * Scope:       Internal
 * Input:       dbid                    DataBase ID
                rec_id                  Record ID
                field_num               Field number
 * Output:      None
 * Return:      TRUE                if success
                FALSE               if database, record or field not found
 * Comment:     None
*****************************************************************/
BOOLEAN SndDataHandler(DatabaseID dbid, RecordID rec_id, USHORT field_num)
{

    SndMgr_dbid     = dbid;
    SndMgr_rec_id   = rec_id;
    SndMgr_field_num= field_num;

    return TRUE;
}

/*****************************************************************
 * Function:    SndDataRec
 * Purpose:     This function is used to record the data to database
 * Scope:       Internal
 * Input:       None
 * Output:      None
 * Return:      TRUE                if success
                FALSE               if database or record not found
 * Comment:     None
*****************************************************************/
BOOLEAN SndDataRec(void)
{   
    WORD    i;

    /* - Output code word updated by Data Manager */
    if (SndCoFcounter%FFFrame == 0)
    {

        FreeMem -= TTCodeLen+1;

        if (FreeMem < BLOCK_SIZE)
        {
            MsgAppendMsgInt(SndMgrMsgDecode, SND_MGR_MSG, LOW_MEM, 0,NULL);
            return FALSE;
        }

        for(i=0; i<1+TTCodeLen; i++)    SndCoABuf[0][i] = SndCoBuf[i];

        MsgAppendMsgInt(SndMgrMsgDecode, SND_MGR_MSG, DATA_UPDATE_FIELD_HEADER, SndCoFcounter, NULL);

    }
    else
    {
        FreeMem -= TTCodeLen;

        if (FreeMem < BLOCK_SIZE)
        {
            MsgAppendMsgInt(SndMgrMsgDecode, SND_MGR_MSG, LOW_MEM, 0, NULL);
            return FALSE;
        }

        for(i=0; i<TTCodeLen; i++)
            SndCoABuf[SndCoFcounter%50][i] = SndCoBuf[i];

        MsgAppendMsgInt(SndMgrMsgDecode, SND_MGR_MSG, DATA_UPDATE_FIELD, SndCoFcounter, NULL);

    }

    return TRUE;
}

/*****************************************************************
 * Function:    SndDataPlayStart
 * Purpose:     This function is used to read the database for sound playing
                at the first time
 * Scope:       Internal
 * Input:       start_time              the time to be started
 * Output:      None
 * Return:      TRUE                if success
                FALSE               if database or record not found
 * Comment:     None
*****************************************************************/
BOOLEAN SndDataPlayStart(UWORD start_time)
{
    UWORD   byte_read;
    Err     err;

    MsgAppendMsgInt(SndMgrMsgDecode, SND_MGR_MSG, DATA_FIELD_SIZE, 0, NULL);

    if (TotalLen < TTCodeLen+1)
    {
        MsgAppendMsgInt(SndMgrMsgDecode, SND_MGR_MSG, REC_END, 0, NULL);
        return FALSE;
    }

    EncodataIndex = start_time * BYTEPS;

    /* - Code word read by Data Manager through message */
    err = DataReadFieldSysMem(SndMgr_dbid, SndMgr_rec_id, SndMgr_field_num, EncodataIndex, TTCodeLen+1, &SndCoPtr, &byte_read);
    if (err != 1)
    {
        MsgAppendMsgInt(SndMgrMsgDecode, SND_MGR_MSG, REC_END, TTCodeLen+1, NULL);
        return FALSE;
    }

    MsgAppendMsgInt(SndMgrMsgDecode, SND_MGR_MSG, PLAY_TIME, SndCoFcounter, NULL);
    EncodataIndex += TTCodeLen+1;


     return TRUE;
}

/*****************************************************************
 * Function:    SndDataPlay
 * Purpose:     This function is used to read the database for sound playing
 * Scope:       Internal
 * Input:       None
 * Output:      None
 * Return:      TRUE                if success
                FALSE               if database or record not found
 * Comment:     None
*****************************************************************/
BOOLEAN SndDataPlay(void)
{
    Err     err;
    UWORD   byte_read;

    /* - Code word read by Data Manager */
    qfree(SndCoPtr);

    if (SndCoFcounter%FFFrame == 0)
    {

        if (TotalLen < EncodataIndex+TTCodeLen+1)
        {
            //EvtAppendEvt(SND_MGR_EVT, REC_END, 0, 0, NULL);
            MsgAppendMsgInt(SndMgrMsgDecode, SND_MGR_MSG, REC_END, 0, NULL);
            return FALSE;
        }

        err = DataReadFieldSysMem(SndMgr_dbid, SndMgr_rec_id, SndMgr_field_num, EncodataIndex, TTCodeLen+1, &SndCoPtr, &byte_read);
        if (err != 1)
        {
            MsgAppendMsgInt(SndMgrMsgDecode, SND_MGR_MSG, REC_END, TTCodeLen+1, NULL);
            return FALSE;
        }

        MsgAppendMsgInt(SndMgrMsgDecode, SND_MGR_MSG, PLAY_TIME, SndCoFcounter, NULL);
        EncodataIndex += TTCodeLen+1;

    }
    else
    {

        if (TotalLen < EncodataIndex+TTCodeLen)
        {
            MsgAppendMsgInt(SndMgrMsgDecode, SND_MGR_MSG, REC_END, 0, NULL);
            return FALSE;
        }
        err = DataReadFieldSysMem(SndMgr_dbid, SndMgr_rec_id, SndMgr_field_num, EncodataIndex, TTCodeLen, &SndCoPtr, &byte_read);
        if (err != 1)
        {
            MsgAppendMsgInt(SndMgrMsgDecode, SND_MGR_MSG, REC_END, TTCodeLen, NULL);
            return FALSE;
        }

        EncodataIndex += TTCodeLen;

    }
 
    return  TRUE;
}


/*****************************************************************
 * Function:    SndDataPlaySndEffectStart
 * Purpose:     This function is used to read the database for sound effect playing
 * Scope:       Internal
 * Input:       snd_eff_name        sound effect name
 * Output:      None
 * Return:      TRUE                if success
                FALSE               if database or record not found
 * Comment:     None
*****************************************************************/
void SndDataPlaySndEffectStart(SHORT snd_eff_name)
{

    WORD    i;


    GSndEffName = snd_eff_name;

    switch(snd_eff_name)
    {

        case SNDRES_SHARP:
            SndResAddr = SND_SHARP;
            TotalLen = SND_SHARP_SIZE;
            break;
        case SNDRES_ZHARP:
            SndResAddr = SND_ZHARP;
            TotalLen = SND_ZHARP_SIZE;
            break;
        case SNDRES_BWOO:
            SndResAddr = SND_BWOO;
            TotalLen = SND_BWOO_SIZE;
            break;
        case SNDRES_DONEDONE:
            SndResAddr = SND_DONEDONE;
            TotalLen = SND_DONEDONE_SIZE;
            break;
        case SNDRES_VEEWOO:
            SndResAddr = SND_VEEWOO;
            TotalLen = SND_VEEWOO_SIZE;
            break;
        case SNDRES_KOOKOO:
            SndResAddr = SND_KOOKOO;
            TotalLen = SND_KOOKOO_SIZE;
            break;
        case SNDRES_ROOSTER:
            SndResAddr = SND_ROOSTER;
            TotalLen = SND_ROOSTER_SIZE;
            break;
        //***********************************
#ifdef  PLAY_COM_BEEP
        case SNDRES0_BEEP:
            SndResAddr = SND0_BEEP;
            TotalLen = SND0_BEEP_SIZE;
            break;
        case SNDRES1_BEEP:
            SndResAddr = SND1_BEEP;
            TotalLen = SND1_BEEP_SIZE;
            break;
        case SNDRES2_BEEP:
            SndResAddr = SND2_BEEP;
            TotalLen = SND2_BEEP_SIZE;
            break;
        case SNDRES3_BEEP:
            SndResAddr = SND3_BEEP;
            TotalLen = SND3_BEEP_SIZE;
            break;
        case SNDRES4_BEEP:
            SndResAddr = SND4_BEEP;
            TotalLen = SND4_BEEP_SIZE;
            break;
        case SNDRES5_BEEP:
            SndResAddr = SND5_BEEP;
            TotalLen = SND5_BEEP_SIZE;
            break;
        default:
            SndResAddr = SND5_BEEP;
            TotalLen = SND5_BEEP_SIZE;
            break;
#else
        case SNDRES0_BEEP:
            OSndResAddr = SND0_OBEEP;
            TotalLen = SND0_OBEEP_SIZE;
            break;
        case SNDRES1_BEEP:
            OSndResAddr = SND1_OBEEP;
            TotalLen = SND1_OBEEP_SIZE;
            break;
        case SNDRES2_BEEP:
            OSndResAddr = SND2_OBEEP;
            TotalLen = SND2_OBEEP_SIZE;
            break;
        case SNDRES3_BEEP:
            OSndResAddr = SND3_OBEEP;
            TotalLen = SND3_OBEEP_SIZE;
            break;
        case SNDRES4_BEEP:
            OSndResAddr = SND4_OBEEP;
            TotalLen = SND4_OBEEP_SIZE;
            break;
        case SNDRES5_BEEP:
            OSndResAddr = SND5_OBEEP;
            TotalLen = SND5_OBEEP_SIZE;
            break;
        default:
            OSndResAddr = SND5_OBEEP;
            TotalLen = SND5_OBEEP_SIZE;
            break;
#endif

    }

#ifdef  PLAY_COM_BEEP
        SndCoPtr = SndResAddr;
        EncodataIndex = TTCodeLen+1;
#else
    if (GSndEffName < NUM_SPC_SND_EFF)
    {
        SndCoPtr = SndResAddr;
        EncodataIndex = TTCodeLen+1;
    }
    else
    {
        OSndCoPtr = (SHORT*)SndWrkOutBuf;
        for(i=0; i<SND_BUF_SIZE; i++)
            OSndCoPtr[i] = (SHORT)((((OSndResAddr[i]&0xff00)>>8) | ((OSndResAddr[i]&0x00ff)<<8))&0xffff);
        EncodataIndex = SND_BUF_SIZE;
    }
#endif


     return;
}



/*****************************************************************
 * Function:    SndDataPlaySndEffect
 * Purpose:     This function is used to read the database for sound effect playing
 * Scope:       Internal
 * Input:       None
 * Output:      None
 * Return:      TRUE                if success
                FALSE               if database or record not found
 * Comment:     None
*****************************************************************/
BOOLEAN SndDataPlaySndEffect(void)
{
    WORD    i;
    BYTE    tmp;


        if(TotalLen < EncodataIndex+SND_BUF_SIZE)
        {

            /*- Global variables initialization ---------------*/
            /*
            SndCoIndex      = 0;
            SndCoFcounter   = 0;
            for(i=0; i<WmN; i++)
                SndCovP[i] = SndCoIbuf[i] = 0;
            */
            if (Alarm_Acc[0] > -1)
            {
                tmp = Alarm_Acc[0];
                for(i=0; i<9; i++)
                    Alarm_Acc[i] = Alarm_Acc[i+1];
                Alarm_Acc[9] = -1;
                SndDataPlaySndEffectStart((SHORT)tmp);
                return TRUE;
            }


            SndAction = SND_STOP;
            SndIOChannel = SND_STOP;
            SndPlayAction = SND_STOP;

            if (SndEnableState==0)
                SndEnable_Int(SndIOChannel);

            return FALSE;

        }
/*      else if (TotalLen < EncodataIndex+SND_BUF_SIZE)
        {
            OSndCoPtr = (SHORT*)SndWrkOutBuf;
            for(i=0; i<SND_BUF_SIZE; i++)
                OSndCoPtr[i] = 0;
            EncodataIndex += SND_BUF_SIZE;
        }*/
        else
        {
            OSndCoPtr = (SHORT*)SndWrkOutBuf;
            for(i=0; i<SND_BUF_SIZE; i++)
                OSndCoPtr[i] = (SHORT)((((OSndResAddr[i+EncodataIndex]&0xff00)>>8) | ((OSndResAddr[i+EncodataIndex]&0x00ff)<<8))&0xffff);
            EncodataIndex += SND_BUF_SIZE;
        }

 
    return  TRUE;
}


/*****************************************************************
 * Function:    SndDataPlayComSndEffect
 * Purpose:     This function is used to read the database for compressed sound effect playing
 * Scope:       Internal
 * Input:       None
 * Output:      None
 * Return:      TRUE                if success
                FALSE               if database or record not found
 * Comment:     None
*****************************************************************/
BOOLEAN SndDataPlayComSndEffect(void)
{
    WORD    i;
    BYTE    tmp;

    if (SndCoFcounter%FFFrame == 0)
    {

        if (TotalLen < EncodataIndex+TTCodeLen+1)
        {

            /*- Global variables initialization ---------------*/
            SndCoIndex      = 0;
            SndCoFcounter   = 0;
            for(i=0; i<WmN; i++)
                SndCovP[i] = SndCoIbuf[i] = 0;

            if (Alarm_Acc[0] > -1)
            {
                tmp = Alarm_Acc[0];
                for(i=0; i<9; i++)
                    Alarm_Acc[i] = Alarm_Acc[i+1];
                Alarm_Acc[9] = -1;
                SndDataPlaySndEffectStart((SHORT)tmp);
                return TRUE;
            }

            /*- Sound Device Driver Initialization -----------*/
            SndAction = SND_STOP;
            SndIOChannel = SND_STOP;

            SndPlayAction = SND_STOP;

            if (SndEnableState==0)
                SndEnable_Int(SndIOChannel);

            return FALSE;

        }
        SndCoPtr = SndResAddr + EncodataIndex;
        EncodataIndex += TTCodeLen+1;

    }
    else
    {

        if (TotalLen < EncodataIndex+TTCodeLen)
        {

            /*- Global variables initialization ---------------*/
            SndCoIndex      = 0;
            SndCoFcounter   = 0;
            for(i=0; i<WmN; i++)
                SndCovP[i] = SndCoIbuf[i] = 0;

            if (Alarm_Acc[0] > -1)
            {
                tmp = Alarm_Acc[0];
                for(i=0; i<9; i++)
                    Alarm_Acc[i] = Alarm_Acc[i+1];
                Alarm_Acc[9] = -1;
                SndDataPlaySndEffectStart((SHORT)tmp);
                return TRUE;
            }


            /*- Sound Device Driver Initialization -----------*/
            SndAction = SND_STOP;
            SndIOChannel = SND_STOP;

            SndPlayAction = SND_STOP;

            if (SndEnableState==0)
                SndEnable_Int(SndIOChannel);

            return FALSE;

        }

        SndCoPtr = SndResAddr + EncodataIndex;
        EncodataIndex += TTCodeLen;
    }
 
    return  TRUE;
}


/*****************************************************************
 * Function:    Ader
 * Purpose:     Perform encoding process
 * Scope:       Internal
 * Input:       None
 * Output:      None
 * Return:      None
 * Comment:     None
******************************************************************/
void Ader(void)
{
    /*------------------------------------------------*/
    BYTE    Ia[Pth];

    USHORT  i, j;
    USHORT  countn;
    USHORT  outbitcount = 0;
    USHORT  outbytecount= 0;

    SHORT   *dbuf;
    SHORT   sign;
    SHORT   diff;
    SHORT   step;
    SHORT   pDiff;
    SHORT   delta;
    SHORT   Iinbuf[W];
    SHORT   vP[W];
    SHORT   Iwbuf[W];

    WORD    data = 0;
    double  rb[Pth+1];
    /*------------------------------------------------*/

    for (i=0; i<Pth; i++)
    {
        Ia[i] = 0;
        rb[i] = 0.0;
    }
    rb[Pth] = 0.0;

    for (i=0; i<W; i++)
    {
       Iinbuf[i] = 0;
       vP[i]= 0;     
       Iwbuf[i]  = 0;
    }


    if (SndIOChannel == SND_STOP)
        return;

    dbuf = (SHORT*)SndWrkInBuf;


#ifdef  AGC

    if (AGC_Co_Acc < 6)
        AGC_Co_Acc = 0;
    else if(AGC_Co_Acc < 20)
        AGC_Co_Acc = 1;
    else if(AGC_Co_Acc < 70)
        AGC_Co_Acc = 2;
    else if(AGC_Co_Acc < 110)
        AGC_Co_Acc = 4;
    else if(AGC_Co_Acc < 145)
        AGC_Co_Acc = 7;
    else
        AGC_Co_Acc = 11;




    if (AGC_Coefficient < 12000000 && AGC_Co_Acc < 1 && AGC_Co < 300)
        AGC_Co++;
    else
        AGC_Co -= AGC_Co_Acc;


    AGC_Co_Acc = 0;
    for(i=0; i<N; i++)
    {
        Iinbuf[WmN+i] = (SHORT) ( ((dbuf[i] & 0x00ff)<<8) | ((dbuf[i] & 0xff00)>>8) )&0xFFFF ;
        Iinbuf[WmN+i] /= 16;

        Iinbuf[WmN+i] = (SHORT)( (((WORD)Iinbuf[WmN+i])*AGC_Co)/16);


        if (Iinbuf[WmN+i] > 2046 )
        {
            Iinbuf[WmN+i] = 2047;
            AGC_Co_Acc++;
        }
        else if(Iinbuf[WmN+i] < -2046)
        {
            Iinbuf[WmN+i] = -2047;
            AGC_Co_Acc++;
        }
    }
    AGC_Coefficient = CalAgcCo((UWORD*)Iinbuf, W);
#else
    for(i=0; i<N; i++)
    {
        Iinbuf[WmN+i] = (SHORT) ( ((dbuf[i] & 0x00ff)<<8) | ((dbuf[i] & 0xff00)>>8) )&0xFFFF ;
        Iinbuf[WmN+i] /= 16;
    }

#endif


    /* Initializations */
    step = s_sizeT[SndCoIndex];
    for(i=0; i<TTCodeLen+1; i++)    SndCoBuf[i] = 0;
    for(i=0; i<WmN; i++)
    {
        Iinbuf[i] = SndCoIbuf[i];
        vP[i] = SndCovP[i];
    }
    Iwindow(Iinbuf, Iwbuf, W);          /* Windowing the input buffer */
    Iautoco(Iwbuf, rb, W, Pth+1);       /* Autocorrelation */
    Icalcoef(rb, Ia, Pth);              /* Calculate the alpha coefficients*/

    /* Pack the index into the coded data stream */
    if (SndCoFcounter%FFFrame == 0)
        Ipack((SndCoIndex) & 0xff, 8, &outbitcount, &outbytecount);


    /* Pack the alpha into the coded data stream */
    for(i=0; i<Pth; i++)
        Ipack(Ia[i] & 0xff, 8, &outbitcount, &outbytecount);

    /* Process the data */
    for(countn=WmN; countn<W; countn++)
    {

        diff = Iinbuf[countn] - vP[countn-1];

        if (diff < 0)
        {
            sign = BITORDER;
            diff = -diff;
        }
        else    sign = 0;

        delta = 0;
        pDiff = (step >> 3);

        step >>= 2;

        if ( diff >= step )
        {
            delta |= 1;
            pDiff += step;
        }

        if (sign)
        {
            if (vP[countn-1]  <= -SCALE + pDiff)
                    vP[countn-1] = -SCALE;
            else    vP[countn-1] -= pDiff;
        }
        else
        {
            if(vP[countn-1] >= SCALE - pDiff)
                    vP[countn-1] = SCALE;
            else    vP[countn-1] += pDiff;
        }

        data = 0;

        for(i=1; i<=Pth; i++)
            data += (WORD)Ia[i-1] * (WORD)vP[countn-i];
        data /= ASCALE;

        if ( data > SCALE )        data =  SCALE;
        else if( data < -SCALE )   data = -SCALE;

        vP[countn] = (SHORT)data;

        delta |= sign;
        SndCoIndex += indexT[delta];

        if (SndCoIndex < 0)      SndCoIndex = 0;
        if (SndCoIndex > 59)     SndCoIndex = 59;

        step = s_sizeT[SndCoIndex];

        /* Step 6 - Pack the delta into bit stream */
        Ipack(delta & 0xff, BIT, &outbitcount, &outbytecount);

    } /* end of countn */

    /* - Output code word updated by Data Manager */
    /* - Not enough memory, assign a 'stop' action to voice manager */

    if (SndDataRec() != TRUE)
    {
        SndIOChannel = SND_STOP;
        return;
    }
    else
    {
        /* - Update values for next frame */
        for(i=0; i<WmN; i++)
        {
            SndCoIbuf[i] = Iinbuf[i+N];
            SndCovP[i] = vP[i+N];
        }
        SndCoFcounter++;
    }

    return;

} /* end of Ader */

/*****************************************************************
 * Function:    Kder
 * Purpose:     Perform decoding process
 * Scope:       Internal
 * Input:       None
 * Output:      None
 * Return:      None
 * Comment:     None
******************************************************************/
void Kder(void)
{
    /*------------------------------------------------*/
    BYTE    Ia[Pth];

    USHORT  i, j;
    USHORT  countn;
    USHORT  inbitcount = 0;
    USHORT  inbytecount= 0;

    SHORT   *dbuf;
    SHORT   Ioutbuf[W];
    SHORT   Ioutbuf_temp;
    SHORT   vP[W];

    SHORT   sign;
    SHORT   delta;
    SHORT   step;
    SHORT   pDiff;

    WORD    data = 0;
    /*------------------------------------------------*/


    for (i=0; i<Pth; i++)
        Ia[i] = 0;

    for (i=0; i<W; i++)
    {
       Ioutbuf[i] = 0;          /* Output buffer */
       vP[i]= 0;            /* Predicted output value */
    }

    /*------------------------------------------------*/


    dbuf = (SHORT*)SndWrkOutBuf;

    /* Initializations */
    for(i=0; i<WmN; i++)
    {
        Ioutbuf[i] = SndCoIbuf[i];
        vP[i] = SndCovP[i];
    }

    /* UNPack the index into the coded data stream */
    if (SndCoFcounter%FFFrame == 0)
        SndCoIndex = (SHORT)Iunpack(8, &inbitcount, &inbytecount);
    step = s_sizeT[SndCoIndex];

    /* UNPack the index into the coded data stream */
    for(i=0; i<Pth; i++)
        Ia[i] = (SHORT)Iunpack(8, &inbitcount, &inbytecount);

    for(countn=WmN; countn<W; countn++)
    {

        delta = (SHORT)Iunpack(BIT, &inbitcount, &inbytecount);
        SndCoIndex += indexT[delta];
        if ( SndCoIndex < 0 )   SndCoIndex = 0;
        if ( SndCoIndex > 59 )  SndCoIndex = 59;

        sign  = delta &  BITORDER;
        delta = delta & (BITORDER - 1);

        pDiff = step >> 3;

        if (delta & 1)   pDiff += step>>2;

        data = 0;
        for(i=1; i<=Pth; i++)
            data += (WORD)Ia[i-1] * (WORD)vP[countn-i];

        data /= ASCALE;

        if ( data > SCALE )        data =  SCALE;
        else if( data < -SCALE )   data = -SCALE;
        vP[countn] = (SHORT)data;

        if (sign)
        {
            if (vP[countn]  <= -SCALE + pDiff)
                    vP[countn] = -SCALE;
            else    vP[countn] -= pDiff;
        }
        else
        {
            if(vP[countn] >= SCALE - pDiff)
                    vP[countn] = SCALE;
            else    vP[countn] += pDiff;
        }

        step = s_sizeT[SndCoIndex];

        Ioutbuf[countn] = vP[countn];

    } /* end of countn */

    /* - Output value */
    if ((SndPlayAction == SND_PLAY_REC) && (SndCoFcounter < 25))
    {
        /* startup noise compensation */
        for(i=0; i<N; i++)
            dbuf[i] = 0;
    }
    else
    {
        for(i=0; i<N; i++)
        {
            Ioutbuf[WmN+i] *= 16;
            dbuf[i] = (SHORT)((((Ioutbuf[WmN+i]&0xff00)>>8) | ((Ioutbuf[WmN+i]&0x00ff)<<8))&0xffff) ;
        }
    }

    /* - Update step values */
    for(i=0; i<WmN; i++)
    {
        SndCoIbuf[i] = Ioutbuf[i+N];
        SndCovP[i] = vP[i+N];
    }
    SndCoFcounter++;        /* update frame counter */

    return;

} /* end of Kder */

/*****************************************************************
 * Function:    CalAgcCo
 * Purpose:     Calculating the AGC Coefficient
 * Scope:       Internal
 * Input:       Iinbuf          input buffer
                n               no. of sample per frame
 * Output:      None
 * Return:      acc_value       energy coefficient
 * Comment:     None
******************************************************************/
UWORD CalAgcCo(UWORD Iinbuf[], UWORD n)
{
    UWORD   i;
    UWORD   acc_value;

    acc_value = 0;
    for(i=0; i<n; i++)
        acc_value += Iinbuf[i]*Iinbuf[i];

    acc_value /= n;

    if (acc_value < 1)
        acc_value = 1;

    return acc_value;

} /* end of CalAgcCo() */

/*****************************************************************
 * Function:    Iwindow
 * Purpose:     Perform Hamming windowing
 * Scope:       Internal
 * Input:       Iinbuf          input buffer
                n               no. of sample per frame
 * Output:      Iwbuf           windowed output buffer
 * Return:      None
 * Comment:     None
******************************************************************/
void Iwindow(SHORT Iinbuf[], SHORT Iwbuf[], SHORT n)
{
    WORD i, t1, t2;

    for(i=0; i<n; i++)
    {

        t1 = (WORD)(Iinbuf[i]);
        t2 = (WORD)(COS[i]);

        Iwbuf[i] = (SHORT)((t1*t2)/32767);
    }

    if (Iwbuf[n>>1] == 0)
        Iwbuf[n>>1] = 1;

    return;

} /* end of Iwindow() */

/*****************************************************************
 * Function:    Iautoco
 * Purpose:     Calculate the autocorrelation of the input samples
 * Scope:       Internal
 * Input:       Iwbuf           input windowed buffer
                n               no. of sample per frame
                p               order of the LPC coefficients
 * Output:      rb              autocorrelated output buffer
 * Return:      None
 * Comment:     None
******************************************************************/
void Iautoco(SHORT Iwbuf[], double rb[], SHORT n, SHORT p)
{
    WORD t1, t2, i, j, data;

    /* Calculate the autocorrelation */
    for(i=0; i<p; i++)
    {
        data = 0;
        for(j=0; j<n-i; j++)
        {
            t1 = (WORD)(Iwbuf[j]  );
            t2 = (WORD)(Iwbuf[i+j]);
            data += t1*t2;
        }

        /* Save the autocorrelation */
        rb[i] = (double) ((double)data / 1072693504.0);
    }

    return;

} /* end of Iautoco() */


/*****************************************************************
 * Function:    Icalcoef
 * Purpose:     Calculate the coefficients for adaptive prediction
 * Scope:       Internal
 * Input:       rb              autocorrelation results
                p               predicition order
 * Output:      Ia              output coefficients
 * Return:      None
 * Comment:     None
******************************************************************/
void Icalcoef(double rb[], BYTE Ia[], SHORT p)
{
    WORD i, j;
    double  alpha, beta, k;
    double  tmp[Pth], a[Pth];

    a[0] = 0.0;
    a[1] = 0.0;
    a[2] = 0.0;
    a[3] = 0.0;

    alpha = rb[0];
    beta  = rb[1];

    k = beta / alpha;
    a[0] = k;

    for(i=1; i<p; i++)
    {

        alpha = alpha * (1.0 - k*k);
        beta = rb[i+1];

        for(j=0; j<i; j++)
            beta -= a[j] * rb[i-j];

        k = beta / alpha;
        a[i] = k;

        for(j=0; j<i; j++)
            tmp[j] = a[j] - k*a[i-1-j];

        for(j=0; j<i; j++)
            a[j] = tmp[j];
    }

    for(i=0; i<Pth; i++)
        Ia[i] = (BYTE)(a[i] * ASCALE);

    return;

} /* end of Icalcoef() */


/*****************************************************************
 * Function:    Ipack
 * Purpose:     Packing the data into different type of package for storage
                (e.g. 2, 3 or 4 bits)
 * Scope:       Internal
 * Input:       c                   data bit for packing
                nbit                no. of bit for packing into
                *outbitcount        output bit counter
                *outbitcount        output bit counter
 * Output:      *outbitcount        output bit counter
                *outbytecount       output byte counter
 * Return:      None
 * Comment:     None
******************************************************************/
void Ipack(UBYTE c, SHORT nbit, USHORT *outbitcount, USHORT *outbytecount)
{
    SHORT x;

    x = 7 - (*outbitcount - (((*outbitcount)>>3)<<3)) ;
    if(x < nbit-1)
    {
        SndCoBuf[*outbytecount] |= c>>(nbit-1-x) ;
        SndCoBuf[*outbytecount+1] |= (c<<(9-nbit+x)) & 0xff ;
    }
    else  SndCoBuf[*outbytecount] |= (c<<(x-nbit+1)) & 0xff ;

    *outbitcount += nbit ;
    *outbytecount = ((*outbitcount) >> 3) ;

    return;
} /* end of Ipack() */


/*****************************************************************
 * Function:    Iunpack
 * Purpose:     Unpacking the data into different type of package for storage
                (e.g. 2, 3 or 4 bits)
 * Scope:       Internal
 * Input:       nbit                no. of bit for packing into
                *inbitcount         input bit counter
                *inbytecount        input byte counter
 * Output:      *inbitcount         input bit counter
                *inbytecount        input byte counter
 * Return:      Decoded bit data
 * Comment:     None
******************************************************************/
UBYTE Iunpack(SHORT nbit, USHORT *inbitcount, USHORT *inbytecount)
{
    SHORT x;
    UBYTE c;

    x = 7 - ((*inbitcount) & 0x07);
    if(x < nbit - 1)
    {
        c = ((SndCoPtr[*inbytecount] & ~(0xfe<<x)) << (nbit-1-x)) & 0xff ;
        c |= ((SndCoPtr[*inbytecount+1]>>(9-nbit+x)) & ~(0xfe<<(nbit-2-x))) & 0xff ;
    }
    else  c = ((SndCoPtr[*inbytecount] >> (x-nbit+1)) & ~(0xfe<<(nbit-1))) & 0xff ;

    *inbitcount += nbit;
    *inbytecount = *inbitcount>>3;

    return(c);
} /* end of Iunpack() */


/*****************************************************************
 * Function:    SndDDInit
 * Purpose:     Sound Device Driver Initialization
 * Scope:       Internal
 * Input:       None
 * Output:      None
 * Return:      None
 * Comment:     None
******************************************************************/
void SndDDInit(void)
{
    WORD    i;



    for(i=0; i<10; i++)
        Alarm_Acc[i] = -1;
    
#ifdef  AGC
    AGC_Co = 64;
    AGC_Co_Acc = 0;
    AGC_Coefficient = 200;
#endif
    SndAction = 0;
    SndEnableState = 0;
    SndInit(SR_8KHZ, SND16, 0xf, 0 );
    SndVolume(SND_MAX_GAIN, (WORD)(SND_MAX_VOL - ClickPara));

    return;

} /* end of SndDDInit() */



/*****************************************************************
 * Function:    SndSetSndSettings
 * Purpose:     Set mic in gain and sound volumne of the system
 * Scope:       Internal
 * Input:       mic_gain            mic input gain should be 0 - 15
                vol                 speaker volumne should be 0 - 31
                click               click sound volumne control
                set_para            parameters settings indicator
 * Output:      None
 * Return:      TRUE                if success
                FALSE               if invalid parameters
 * Comment:     None
******************************************************************/
BOOLEAN SndSetSndSettings(SHORT mic_gain, SHORT vol, SHORT click, SHORT set_para)
{
    //InputGain :   0-15;
    //OutputAttn:   0-31;
    if (set_para>3)
        return FALSE;

    switch(set_para)
    {
        case 0:
            if (mic_gain > SND_MAX_GAIN || mic_gain < 0)
                return FALSE;
//----------------------------------------
            break;
        case 1:
            if (vol > SND_MAX_VOL || vol < 0)
                return FALSE;
            if (vol == 0)
                VolPara = 0;
            else
                VolPara = (vol-23)*8/10+23;

            if ((SndAction == SND_PLAYING) && (SndPlayAction == SND_PLAY_REC))
                //SndVolume(SND_MAX_GAIN, (WORD)(SND_MAX_VOL - VolPara));
                SndPlyVolume(SND_MAX_GAIN, (WORD)(SND_MAX_VOL - VolPara));
            break;
        case 2:
            if (click > SND_MAX_VOL || click < 0)
                return FALSE;
            ClickPara = click;
            SndVolume(SND_MAX_GAIN, (WORD)(SND_MAX_VOL - ClickPara));
            break;
        case 3:
            if (click == -1)
            {
                if (vol == 0)
                    VolPara = 0;
                else
                    VolPara = (vol-23)*8/10+23;
                return TRUE;
            }
            else if(vol == -1)
            {
                ClickPara = click;
                SndVolume(SND_MAX_GAIN, (WORD)(SND_MAX_VOL - ClickPara));
            }
            else
            {
                if (vol == 0)
                    VolPara = 0;
                else
                    VolPara = (vol-23)*8/10+23;
                ClickPara = click;
                SndVolume(SND_MAX_GAIN, (WORD)(SND_MAX_VOL - ClickPara));
            }

            //if ((vol > SND_MAX_VOL) || (vol < 0) || (click > SND_MAX_VOL) || (click < 0))
                //return FALSE;

            break;
        default:
            return FALSE;
    }

    return TRUE;
}

/*****************************************************************
 * Function:    SndMgrMsgDecode
 * Purpose:     Decode the message which id appened in the interrupt
 * Scope:       Internal
 * Input:       message             msg to be decoded
 * Output:      None
 * Return:      None
 * Comment:     None
******************************************************************/
void SndMgrMsgDecode(MsgType* message)
{
    Err     err;
    UWORD   byte_read;

#ifdef DEBUG    // ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    USHORT total_field;
#endif          // ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

    switch(message->msgID)
    {
        case SND_MGR_MSG:
        {
            switch(message->para1)
            {
                case LOW_MEM:
                    EvtAppendEvt(SND_MGR_EVT, LOW_MEM, 0, 0, NULL);
                    break;
                case REC_END:
                    EvtAppendEvt(SND_MGR_EVT, REC_END, 0, 0, NULL);
                    break;
                case DATA_UPDATE_FIELD_HEADER:
                    err = DataUpdateField(SndMgr_dbid, SndMgr_rec_id, SndMgr_field_num, TTCodeLen+1, &SndCoABuf[0][0]);


                    if (err != 1 && SndAction == SND_RECORDING)
                    {

                        EvtAppendEvt(SND_MGR_EVT, LOW_MEM, 0, 0, NULL);
                        break;
                    }
                    EvtAppendEvt(SND_MGR_EVT, REC_TIME, message->para2, 0, NULL);
                    break;
                case DATA_UPDATE_FIELD:
                    err = DataUpdateField(SndMgr_dbid, SndMgr_rec_id, SndMgr_field_num, TTCodeLen, &SndCoABuf[(message->para2)%50][0]);

                    if (err != 1 && SndAction == SND_RECORDING)
                    {
                        EvtAppendEvt(SND_MGR_EVT, LOW_MEM, 0, 0, NULL);
                    }
                    break;
                case PLAY_TIME:
                    EvtAppendEvt(SND_MGR_EVT, PLAY_TIME, message->para2, 0, NULL);
                    break;
                case DATA_FIELD_SIZE:
                    err = DataFieldSize(SndMgr_dbid, SndMgr_rec_id, SndMgr_field_num, &TotalLen);
                    if (err != 1)
                    {
                        EvtAppendEvt(SND_MGR_EVT, REC_END, 0, 0, NULL);
                    }
                    break;

                default:
                    return;
            }
        }
        break;

        default:
            return;
    }

    return;
}

