/*
================================ COPYRIGHT ================================
The contents of this file are subject to the VTech Informations Ltd. License
of VT-OS Ver. 1.1 operating system (the "License"); you may not use this 
file except in compliance with the License.  
 
Software distributed under the License is distributed on an "AS IS" basis, 
WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
for the specific language governing rights and limitations under the License.
 
The Original Code is VT-OS Ver. 1.1 Operating System, released 
on October 1st, 1999

The Initial Developer of the Original Code is VTech Informations Ltd.  All 
codes are Copyright (C) VTech Informations Ltd. 1999.  All Rights Reserved.
===========================================================================
*/

/*
===========================================================================
File        :   uart.c
Author(s)   :   Andrew Hui
Company     :   VTech Informations Ltd.
Project     :   Helio 
Date:	    :   October 1st, 1999
Purpose:	:   Uart low level driver
Revision    :   1.1
Note        :   07/07/98   Prepared by Andrew Hui (AH)                     
                07/07/98   V1.0 release (AH)                               
                23/07/98   V2.0 change buffer swap mechanism (AH)          
===========================================================================
*/              




#include    "include\pr31700s.h"
#include    "include\pr31700c.h"
#include    "include\msg.h"
#include    "include\qtype.h"
#include    "include\datatype.h"
#include    "include\uart.h"
#include    "include\intc.h"
#include "include\print.h"



/* GLOBAL Variable */
/* UART_A */
BYTE    UartATxBuf[UART_BUF_SIZE];       /* UART Tx buffer (DMA) */
BYTE    UartARxBuf1[UART_BUF_SIZE];       /* UART Rx buffer       */
BYTE    UartARxBuf2[UART_BUF_SIZE];       /* UART Rx buffer       */

WORD    UartARxReadPtr;
WORD    UartARxWritePtr;

BYTE    *UartARxBuf;
BYTE    *UartARxWrkBuf;

/* UART_B */

BYTE    UartBTxBuf[UART_BUF_SIZE];       /* UART Tx buffer (DMA) */
BYTE    UartBRxBuf1[UART_BUF_SIZE];       /* UART Rx buffer       */
BYTE    UartBRxBuf2[UART_BUF_SIZE];       /* UART Rx buffer       */

WORD    UartBRxReadPtr;
WORD    UartBRxWritePtr;

BYTE    *UartBRxBuf;
BYTE    *UartBRxWrkBuf;                                      

MsgType UartMsg;        /* define a UART Message */          

#define UartGetb(port)  ( (port==UART_A)  ?((BYTE)(CPU ->REG_UARTA_HLDG )) \
                                          :((BYTE)(CPU ->REG_UARTB_HLDG )) )

/* assume no another UART interrupt will occur */

void UartADmaFullIsr()
{
	UartADisableTxDma();
}

/*-----------------------------------------------------------------------------
Prototype        :  void UartIsrRx(WORD port)
Purpose          :  Interrupt handler for UART receive
Scope            :  interrupt
Input            :  port  : UART_A / UART_B
Output           :  None
Return           :  None
-----------------------------------------------------------------------------*/

//void    UartIsrRx(WORD port)
void    UartIsrRx()
{
	WORD port = UART_A;
    switch(port)
    {
        case UART_A:
            /* set to next position */

            if (UartARxWritePtr < UART_BUF_SIZE) /* not reach end */
            {
               UartARxBuf[UartARxWritePtr]= UartGetb(port);

                if ((UartARxWritePtr)==UART_BUF_SIZE-1)  /* last free byte */
                {
                    //UartErrorHandle(UART_RX_BUF_FULL, port, NULL);
                    //MsgAppend(UartMsg, &msgQueue);
                    MsgAppendMsgInt(UartMgr, UART_RX_BUF_FULL, port, 0, NULL);
                }

                if (UartARxWritePtr== 0)  /* Received first byte   */
                {
                    //UartErrorHandle(UART_RX_FIRST_BYTE,port,NULL);
                    //MsgAppend(UartMsg, &msgQueue);
                    MsgAppendMsgInt(UartMgr, UART_RX_FIRST_BYTE, port, 0, NULL);
                }
                UartARxWritePtr ++;
            }
            else   /* buffer overflow */
            {
              //UartErrorHandle(UART_RX_BUF_OVERFLOW, (WORD)UartGetb(port), port);
              //MsgAppend(UartMsg, &msgQueue);
              MsgAppendMsgInt(UartMgr, UART_RX_BUF_OVERFLOW, (WORD)UartGetb(port), port, NULL);
            }

            break;
        case UART_B:
            /* set to next position */
            if (UartBRxWritePtr < UART_BUF_SIZE) /* not reach end */
            {

                UartARxBuf[UartBRxWritePtr]= UartGetb(port);

                if ((UartBRxWritePtr)==UART_BUF_SIZE-1)  /* last free byte */
                {
                    //UartErrorHandle(UART_RX_BUF_FULL, port, NULL);
                    //MsgAppend(UartMsg, &msgQueue);
                    MsgAppendMsgInt(UartMgr, UART_RX_BUF_FULL, port, 0, NULL);
                    
                }
                else
                if (UartARxWritePtr== 0)  /* Received first byte   */
                {
                    //UartErrorHandle(UART_RX_FIRST_BYTE,port,NULL);
                    //MsgAppend(UartMsg, &msgQueue);
                    MsgAppendMsgInt(UartMgr, UART_RX_FIRST_BYTE, port, 0, NULL);
                }
                UartBRxWritePtr ++;
            }
            else   /* buffer overflow */
            {
              //UartErrorHandle(UART_RX_BUF_OVERFLOW, (WORD)UartGetb(port), port);
              //MsgAppend(UartMsg, &msgQueue);
              MsgAppendMsgInt(UartMgr, UART_RX_BUF_OVERFLOW, (WORD)UartGetb(port), port, NULL);
            }
            break;
    }
}



/*-----------------------------------------------------------------------------
Prototype       :   void UartErrorHandle(WORD msg, WORD para1, WORD para2)
Purpose         :   To prepare a message template
Scope           :   internal
Input           :   msg   : Message ID
                    para1 : parameter 1 in message
                    para2 : parameter 2 in message
Output          :   None
Return          :   None
-----------------------------------------------------------------------------*/
void    UartErrorHandle(WORD msg, WORD para1, WORD para2)
{
    UartMsg.ptr_fct=UartMgr;
    UartMsg.msgID=msg;
    UartMsg.para1=para1;
    UartMsg.para2=para2;
    UartMsg.msgPBP=NULL;
}



/*-----------------------------------------------------------------------------
  Prototype     :   WORD UartReadBuf(WORD port, BYTE **buf);
  Purpose       :   Retrieve a buffer of data from UART receive buffer
  Scope         :   OS
  Input         :   port    : UART_A / UART_B
  Output        :   buf     : max 16 bytes of data received
  Return        :   no of byte is valid in buf
  Description   :   This function will copy the uart receive buffer to the
                    specific buffer and then disable interrupt first, then
                    update read_ptr, write_ptr and the enable
                    interrupt.
-----------------------------------------------------------------------------*/
WORD    UartReadBuf(WORD port, BYTE *buf)
{
    register WORD i;
    register int *t1, *t2;

    if (port==UART_A)
    {
        t1 = (int*) UartARxBuf;
//        IntDisableGlobal();                 /* swap pointer */
		DIS_GBL_INT;
        UartARxReadPtr=UartARxWritePtr;
        UartARxWritePtr=0;
        UartARxBuf = UartARxWrkBuf;
//        IntEnableGlobal();
		ENA_GBL_INT;
        UartARxWrkBuf=(char*) t1;
        t1 = (int*) UartARxWrkBuf;
        t2 = (int*) buf;
        for (i=0; i<(UartARxReadPtr+3)/4; i++)  t2[i] = t1[i];
    }
    else if (port==UART_B)
    {
        t1 = (int*) UartBRxBuf;
//        IntDisableGlobal();                 /* swap pointer */
		DIS_GBL_INT;
        UartBRxReadPtr=UartBRxWritePtr;
        UartBRxWritePtr=0;
        UartBRxBuf = UartBRxWrkBuf;
//        IntEnableGlobal();
		ENA_GBL_INT;
        UartBRxWrkBuf=(char*) t1;
        t1 = (int*) UartBRxWrkBuf;
        t2 = (int*) buf;
        for (i=0; i<(UartBRxReadPtr+3)/4; i++)  t2[i] = t1[i];
    }             

    return(UartARxReadPtr);     /* no. of byte received */
}


/*-----------------------------------------------------------------------------
Prototype       :   void UartWriteBuf(WORD port, BYTE *buf, WORD length);
Purpose         :   Write a buffer of data to the UART transmit DMA buffer
Scope           :   OS
Input           :   port   : UART_A / UART_B
                    buf    : output buffer
                    length : no of byte in buffer
Output          :   None
Return          :   None
Description     :   UartMgr must check no DMA is carrying out. This function
                    will copy the buffer to uart transmit DMA buffer and
                    then start DMA and enable DMA complete interrupt.

-----------------------------------------------------------------------------*/
void    UartWriteBuf(WORD port, BYTE *buf, WORD length)
{
    register    WORD     i;

    if (port==UART_A)
    {
        for (i=0; i<length; i++)  UartATxBuf[i]=buf[i];

        UartADisableDmaInt();

        UartADmaAddr((WORD)(&UartATxBuf) &~(0x7<<29));   /*must be physical address*/
        UartASetDmaLen(length-1);
        UartAEnableDmaInt();
        UartAEnableTxDma();
    }
    else
    if (port==UART_B)
    {
        for (i=0; i<length; i++)  UartBTxBuf[i]=buf[i];

        UartBDisableDmaInt();

        UartBDmaAddr((WORD)(&UartBTxBuf) &~(0x7<<29));   /*must be physical address*/
        UartBSetDmaLen(length-1);

        UartBEnableDmaInt();
        UartBEnableTxDma();
    }
}



/*-----------------------------------------------------------------------------
Prototype       :   WORD UartAChkRxDmaBusy(WORD port);
Purpose         :   Check if UART  Rx DMA is using or not
Scope           :   OS
Input           :   port : UART_A / UART_B
Output          :
Return          :   UART_DMA_BUSY : Busy,  otherwise ready
Description     :   UartMgr must use this function to check DMA is ready or not
-----------------------------------------------------------------------------*/
WORD    UartChkRxDmaBusy(WORD port)
{
    if(port==UART_A)    return ((CPU ->REG_UARTA_CTRL_1 >>15)&1);
    else    if(port==UART_B)    return ((CPU ->REG_UARTB_CTRL_1 >>15)&1);
}


/*-----------------------------------------------------------------------------
Prototype       :   WORD UartChkTxDmaBusy(WORD port);
Purpose         :   Check if UART A Tx DMA is using or not
Scope           :   OS
Input           :   None
Output          :
Return          :   UART_DMA_BUSY : Busy,  otherwise ready
Description     :   UartMgr must use this function to check DMA is ready or not
-----------------------------------------------------------------------------*/
WORD    UartChkTxDmaBusy(WORD port)
{
    if(port==UART_A)   return ((CPU ->REG_UARTA_CTRL_1 >>14) &1);
    else    if(port==UART_B)   return ((CPU ->REG_UARTB_CTRL_1 >>14) &1);
}

/*---------------------------------------------------------------------------
  Prototype     :   void  UartInit(WORD port, WORD baud, WORD config );
  Purpose       :   initialize the UART port for serial communication
  Scope         :   OS
  input         :   port    : UART_A / UART_B
                    baud    : e.g.  38400
                    config  : data_bits, Parity, stop_bit : "8N1"
                              (defined in UART_DRV.H)
  output        :   None
  return        :   None
---------------------------------------------------------------------------*/
void    UartInit(WORD port, WORD baud, WORD config)
{
	/* config SIB clock, for all serial comm. in side CPU */

// clock init is done in SibInit
//    CPU->REG_CLK_CTRL |= SIBMCLKDIR | ENSIBMCLK | SIBMCLKDIV(2);
//    CPU->REG_CLK_CTRL &= ~CSERSEL;    /* set internal clock source */


	/* config UART port */
    if (port == UART_A)  /* UART A */
	{
        /* config UART clock */
        CPU->REG_CLK_CTRL &= ~CSERDIV(7) ;
        CPU->REG_CLK_CTRL |= CSERDIV(3) | ENCSERCLK | ENUARTACLK;

        CPU ->REG_UARTA_CTRL_1      = config;       /* set data, stop, parity*/
        CPU ->REG_UARTA_CTRL_2      = baud;         /* baud rate             */
		CPU ->REG_UARTA_DMA_CTRL_1	= 0;			/* DMA buf start Address */
        CPU ->REG_UARTA_DMA_CTRL_2  = 0;            /* DMA buf length        */
        HwUartAInit();
    }
    else
    if (port == UART_B)  /* UART B */
    {
		/* config UART clock */
        CPU->REG_CLK_CTRL &= ~CSERDIV(7);
        CPU->REG_CLK_CTRL |= CSERDIV(3) | ENCSERCLK | ENUARTBCLK;

        CPU ->REG_UARTB_CTRL_1      = config;       /* set data, stop, parity*/
        CPU ->REG_UARTB_CTRL_2      = baud;         /* baud rate             */
		CPU ->REG_UARTB_DMA_CTRL_1	= 0;			/* DMA buf start Address */
        CPU ->REG_UARTB_DMA_CTRL_2  = 0;            /* DMA buf length        */
//        HwUartBInit();
    }
} 



/*---------------------------------------------------------------------------
  Prototype     :   void UartEnable(WORD port)
  Purpose       :   To enable the UART
  Scope         :   OS
  Input         :   port : UART_A / UART_B
  Output        :   none
  Return        :   none
---------------------------------------------------------------------------*/
void    UartEnable(WORD port)
{
    if (port==UART_A)
    {
        CPU ->REG_UARTA_CTRL_1  |= ENUART;        /* enable Uart A         */
        while(~CPU ->REG_UARTA_CTRL_1 & UARTON) ; /* wait Uart A stable    */
    }
    else
    if (port==UART_B)
    {
        CPU ->REG_UARTB_CTRL_1  |= ENUART;        /* enable Uart B         */
        while(~CPU ->REG_UARTB_CTRL_1 & UARTON) ; /* wait Uart B stable    */
    }
}



/*---------------------------------------------------------------------------
  Prototype     :   void UartDisable(WORD port)
  Purpose       :   To disable the UART
  Scope         :   OS
  Input         :   port : UART_A / UART_B UART
  Output        :   none
  Return        :   none
  Descriptions  :   Make sure everything is sent out
---------------------------------------------------------------------------*/
void    UartDisable(WORD port)
{
    if(port==UART_A)         CPU ->REG_UARTA_CTRL_1 &= ~ENUART;
    else    if(port==UART_B)         CPU ->REG_UARTB_CTRL_1 &= ~ENUART;
}

/*---------------------------------------------------------------------------
  Prototype     :   WORD UartChkStatus(WORD port );
  Purpose       :   Check UART port status
  Scope         :   ALL
  input         :   port : UART_A / UART_B
  return        :   UART Port Status
---------------------------------------------------------------------------*/
WORD    UartChkStatus(WORD port)
{
    if(port==UART_A)        return (CPU ->REG_UARTA_CTRL_1);
    else    if(port==UART_B)        return (CPU ->REG_UARTB_CTRL_1);
}

/*---------------------------------------------------------------------------
  Prototype     :   WORD UartChkCfg(WORD port)
  Purpose       :   To get the UART configuration
  Scope         :   OS
  input         :   port : UART_A / UART_B
  Output        :
  Return        :   Same as the value of "config" that passed to UartInit
---------------------------------------------------------------------------*/
WORD    UartChkCfg(WORD port)
{
    if(port==UART_A)        return((CPU ->REG_UARTA_CTRL_1)&0x2e);
    else    if(port==UART_B)        return((CPU ->REG_UARTB_CTRL_1)&0x2e);
}




