/*
================================ COPYRIGHT ================================
The contents of this file are subject to the VTech Informations Ltd. License
of VT-OS Ver. 1.1 operating system (the "License"); you may not use this 
file except in compliance with the License.  
 
Software distributed under the License is distributed on an "AS IS" basis, 
WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License 
for the specific language governing rights and limitations under the License.
 
The Original Code is VT-OS Ver. 1.1 Operating System, released 
on October 1st, 1999

The Initial Developer of the Original Code is VTech Informations Ltd.  All 
codes are Copyright (C) VTech Informations Ltd. 1999.  All Rights Reserved.
===========================================================================
*/

/*
===========================================================================
File        :   uartmgr.c
Author(s)   :   Kenng Ng
Company     :   VTech Informations Ltd.
Project     :   Helio 
Date:	    :   October 1st, 1999
Purpose:	:   Uart Manager
Revision    :   1.1
Note        :   None
===========================================================================
*/              



#include "include\datatype.h"
#include "include\uartmgr.h"
#include "include\errcode.h"
#include "include\dm_os.h"
#include "include\qtype.h"
#include "include\msg.h"
#include "include\intc.h"
#include <stddef.h>
#include "include\print.h"
#include "include\eventmgr.h"
#include "include\LcdApi.h"

void UartAOn();
void UartAOff();

extern char *ovrbuf;
extern int ovrcount;
/*
AppID SysGetActiveAppID()
{
        return 1;
}
*/
//#define GetHandlePort(handle) (WORD) ((handle >> 8) & 0x000000ff)
#define GetHandlePort(handle) (WORD) (0)
//#define GetHandleApp(handle)  (AppID)((handle >> 16) & 0x0000ffff)
#define GetHandleApp(handle)  (WORD) (1)

extern WORD UartARxReadPtr;
extern WORD UartARxWritePtr;

int uartmgr_mode;
UBYTE need_hb;

char *strcpy(char *dest, const char *src);
void UartPrints(BYTE *c)
{
	while (*c)
	{
		UartPutc(UART_A,*c);
		c++;
    }
}
/*void UartPrints(char *buf) { UartPrintf(UART_A, buf); }*/
BYTE UartGetc(WORD port)
{   char data;
    while (UartRxData(port,  &data)!=UART_SUCCESS);
    return (char) data;
}
void UartPutc(WORD port, BYTE data)
{   while (UartTxData(port, data)!=UART_SUCCESS);
}
void UartPrintf(WORD port, BYTE *buffer)
{   WORD i=0;
    while (buffer[i])
    {
        UartPutc (port,  (BYTE) buffer[i] ) ;
        i++;
    }
}


/* send buffer */
struct com_lnk_struct
{
	UWORD size;			/* number of valid data in the buffer */
	UWORD byte_send;			/* no. of byte sent out */
	BYTE **data; 			/* pointer to the buffer */
	struct com_lnk_struct *next;	/* next buffer node */
};
typedef struct com_lnk_struct ComSendLnk;

typedef struct
{
	WORD    handle;         /* the handle ID */
	ComMode mode;           /* mode, buffer or char */
	BYTE *rec_buffer;
	UWORD rec_buffer_size;
	UWORD   rec_buffer_count;   /* number of byte recevied in buffer */
        BYTE status;
	struct com_lnk_struct *send_buffer;        /* if buffer mode, the buffer pointer */
} UartPortCtrl;

UartPortCtrl uart_port_ctrl[UART_PORT_MAX];

BYTE com_buffer[UART_BUF_SIZE];  /* buffer to get the uart buffer's data */

/******************************************************************************
		Prototype		  : 	void IsrInit(void)
		Purpose 		  : 	Initialize the interrupt
		Scope			  : 	OS
		Input parameters  : 	none
		Output parameters : 	none
		Return value	  : 	none
******************************************************************************/
/*
void IsrInit()
{
    register WORD status;

    status  = read_status_reg(0);
    status |= 0x5001;

    write_cause_reg(0);
    write_status_reg(status);
}
*/

WORD  UartRxData(WORD port, BYTE *data)
{
    if (port==UART_A)
    {
        if (~CPU ->REG_UARTA_CTRL_1 & RXHOLDFULL)   /* receive buffer empty */
            return (UART_BUF_EMPTY);
        *data = (char)(CPU ->REG_UARTA_HLDG );
    }
    if (port==UART_B)
    {
        if (~CPU ->REG_UARTB_CTRL_1 & RXHOLDFULL)   /* receive buffer empty */
            return (UART_BUF_EMPTY);
        *data = (char)(CPU ->REG_UARTB_HLDG );
    }
//    Delay(0xFFF);
    return (UART_SUCCESS);
}

/*---------------------------------------------------------------------------
  Prototype :   WORD UartTxData(WORD port, BYTE data)
  Purpose	:	Write data to UART port
  Scope 	:	ALL
  input     :   WORD  port     UART_A: port A,   UART_B: port B
  return	:	 UART_SUCCESS
				 UART_BUF_FULL
---------------------------------------------------------------------------*/
WORD  UartTxData(WORD port, BYTE data)
{
	if (port==UART_A)
	{
		if (~CPU ->REG_UARTA_CTRL_1 & EMPTY)		/* transmit buffer full */
			return (UART_BUF_FULL);

		CPU ->REG_UARTA_HLDG = data;
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
	}
	else /* port == UART_B */
	{
		if (~CPU ->REG_UARTB_CTRL_1 & EMPTY)		/* transmit buffer full */
			return (UART_BUF_FULL);
		CPU ->REG_UARTB_HLDG = data;
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");
        __asm(" nop ");

	}
	return (UART_SUCCESS);
}


/*********************************************************************
          Purpose:  Get the lower 2-byte of a 4-byte data
            Scope:  All
 Input parameters:  word - the 4-byte data
Output parameters:  None
           Return:  The lower 2-byte                                       
          Comment:
************************************************************************/
USHORT LParam(UWORD word)
{
	return (USHORT) (word & 0x0000ffff);
}

/*********************************************************************
          Purpose: Get the upper 2-byte of a 4-byte data 
            Scope: All
 Input parameters: word - the 4-byte data                     
Output parameters: None
           Return: The upper 2-byte                                        
          Comment:
************************************************************************/
USHORT HParam(UWORD word)
{
	return ((USHORT) (word >> 16));
}

/*********************************************************************
          Purpose: Set the value of lower 2-byte of a 4-byte data 
            Scope: All
 Input parameters: word - the 4-byte data
                   ldata - the 2-byte data
Output parameters: word - the new data
           Return: None
          Comment:
************************************************************************/
void SetLParam(UWORD *word, USHORT ldata)
{
	(USHORT) word[0] = ldata;
}

/*********************************************************************
          Purpose: Set the value of upper 2-byte of a 4-byte data
            Scope: All
 Input parameters: word - the 4-byte data
                   hdata - the 2-byte data
Output parameters: word - the new data
           Return: None                                        
          Comment:
************************************************************************/
void SetHParam(UWORD *word, USHORT hdata)
{
	*word &= 0x0000ffff;
	*word |= (((UWORD) hdata) << 16);
}


void ComSetDataMode(int mode)
{
        uartmgr_mode = mode;
}

int ComGetDataMode()
{
        return uartmgr_mode;
}


UBYTE DecodeData(UBYTE in, UBYTE *out)
{
    if(uartmgr_mode == COM_DATA_BIN)
    {
        *out = in;
        return 0;
    }

    if(need_hb == 2)
    {
       if( in == 0)
       {  /* rec: 0x7f -> 0x7f -> 0  => 0x7f*/
           *out = 0x7f;
       }
       else
       { /* rec: 0x7f -> 0x7f -> 1  => 0xff*/
           *out = 0xff;
       }
       need_hb = 0;
     }
     else
     {
       if( in == (UBYTE)0x7f)
       {
           need_hb++;
       }
       else
       {
          if(need_hb==1)
          {
             *out = in | (UBYTE)0x80;
             need_hb = 0;
          }
          else
          {
             *out = in;
          }
	}
      }
      return need_hb;
}

/*********************************************************************
          Purpose:  Open an uart port
            Scope:  Application
 Input parameters:  port  - port to open, UART_A or UART_B
Output parameters:  port_hdle - the handle to access this connection
           Return:  TRUE - Success
                    ERR_COM_PORT_BUSY - opened by other app
                    ERR_COM_INV_PORT - invalid port number
          Comment:
************************************************************************/
Err ComOpenPort(WORD port, WORD *port_hdle)
{
        AppID app;
        if (port > UART_PORT_MAX)
        {
        #ifdef DEUBG
         printf("\nInvalid port %ld ", port);
        #endif
         return ERR_COM_INV_PARAM;  /* no such port */
        }
	if (HParam(uart_port_ctrl[port].handle) != COM_FREE)  /* port busy */
        {
        #ifdef DEBUG
                printf("\nPort busy");
        #endif
                return ERR_COM_PORT_BUSY;
        }
#ifndef DEBUG     /* if port disable, cannot use printf thru' poart A */
        UartDisable(port);      /* disable the port */
#endif
        app = SysGetActiveAppID();

	/* build the handle			    */
	/* format of a handle :
		     16          8             8
		|  app id  |  port num  | random number |

		if app id = COM_FREE (0x0000), the port is considered to be free
	*/

        uartmgr_mode = COM_DATA_BIN;
	SetHParam(&uart_port_ctrl[port].handle, (USHORT) app);
	SetLParam(&uart_port_ctrl[port].handle, (uart_port_ctrl[port].handle & 0x0000ffff) +1);
	uart_port_ctrl[port].handle &= 0xffff00ff;
	uart_port_ctrl[port].handle |=  ( (port & 0x000000ff) << 8 );
	*port_hdle = uart_port_ctrl[port].handle;
	uart_port_ctrl[port].mode = COM_MODE_CHAR;	/* default char. mode */
	uart_port_ctrl[port].rec_buffer_size = 0;
	uart_port_ctrl[port].rec_buffer = NULL;	        /* set buffer to NULL */
        uart_port_ctrl[port].status = COM_RX_DISABLE;
	return TRUE;
}

/*********************************************************************
          Purpose: Check if the handle is valid or not
            Scope: Internal
 Input parameters: port_hdle - the handle to check
Output parameters: None
           Return: TRUE - the handle is valid
                   False - the handle is invalid
          Comment:
************************************************************************/
BOOLEAN ComIsHandleValid(WORD port_hdle)
{
	/* check port range */
	if (GetHandlePort(port_hdle) > UART_PORT_MAX)
		return FALSE;

	/* check handle signature */
	if ((uart_port_ctrl[GetHandlePort(port_hdle)].handle != port_hdle))
		return FALSE;

	return TRUE;
}

/*********************************************************************
          Purpose: Close an uart port
            Scope: Application
 Input parameters: port_hdle - handle of the port
Output parameters: None
           Return: TRUE - Success
                   ERR_COM_INV_HANDLE - invalid handle
          Comment:
************************************************************************/
Err ComClosePort(WORD port_hdle)
{
	if (!ComIsHandleValid(port_hdle))
                return ERR_COM_INV_HANDLE;
    UartAOff();
	/* mark the port free */
	SetHParam(&uart_port_ctrl[GetHandlePort(port_hdle)].handle, COM_FREE);
        uart_port_ctrl[GetHandlePort(port_hdle)].status = COM_FREE;
       UartDisable(GetHandlePort(port_hdle));  /* disable the port */

	return TRUE;
}

/*********************************************************************
          Purpose: Reset uart ports and initilize Uart Manager
            Scope: OS
 Input parameters: None                     
Output parameters: None 
           Return: None                                        
          Comment: All buffer & internal variables will reset
************************************************************************/
void ComReset()
{
	USHORT i;
	ComSendLnk *free_buf;
	ComSendLnk *current_buf;

#ifndef DEBUG           /* if disable the port, cannot use printf for debug */
//        for(i=0;i<=UART_PORT_MAX;i++)    /* disable all the port */
//              UartDisable(i); 
#endif
/*
                for (i=0;i<=UART_PORT_MAX;i++)  
		{
			current_buf = uart_port_ctrl[i].send_buffer;
			free_buf = NULL;
			while (current_buf != NULL)
			{
				free_buf = current_buf;
				current_buf = current_buf->next;
				pfree(free_buf);
			}

			if (uart_port_ctrl[i].rec_buffer != NULL)
			pfree(uart_port_ctrl[i].rec_buffer);
		}
*/
	for(i=0;i<=UART_PORT_MAX;i++)	/* mark each port free */
        {
                uart_port_ctrl[i].status = COM_FREE;
		SetHParam(&uart_port_ctrl[i].handle, COM_FREE);
        }
}

/*********************************************************************
          Purpose: Start to capture the uart port data
            Scope: Application
 Input parameters: port_hdle - handle of the port
                   mode - port mode, COM_MODE_CHAR or COM_MODE_BUFFER
                   time_out - time out if in buffer mode
                   buffer_size - buffer size, if in buffer mode
                   buffer - the buffer pointer, if in buffer mode
Output parameters: None
           Return: TRUE - Success
                   ERR_COM_INV_HANDLE - invalid handle
                   ERR_COM_INV_PARAM  - mode is invalid
                   ERR_COM_INV_BUFFER - buffer = NULL or buffer_size = 0, if in buffer mode
          Comment: uart data will not capture with ComOpenPort(), call
                   ComCapture to start capture uart data
************************************************************************/
Err ComCapture(WORD port_hdle, ComMode mode, UWORD time_out, UWORD buffer_size, BYTE *buffer)
{
        if (!ComIsHandleValid(port_hdle))
                return ERR_COM_INV_HANDLE;
    UartAOn();

    need_hb = 0;

	switch(mode)
	{
		case COM_MODE_BUFFER:
			if ((buffer == NULL) || (buffer_size == 0))
                                return ERR_COM_INV_BUFFER;
			/* todo: any to do if chg from char mode to buffer mode,
				or already in buffer mode, but reset the mode? */
			uart_port_ctrl[GetHandlePort(port_hdle)].rec_buffer = buffer;
			uart_port_ctrl[GetHandlePort(port_hdle)].rec_buffer_size = buffer_size;
			uart_port_ctrl[GetHandlePort(port_hdle)].rec_buffer_count = 0;
			uart_port_ctrl[GetHandlePort(port_hdle)].mode = COM_MODE_BUFFER;
                        uart_port_ctrl[GetHandlePort(port_hdle)].status = COM_RX_ENABLE;
			break;
		case COM_MODE_CHAR:
			/* todo: any to do if chg from mode ? */
			uart_port_ctrl[GetHandlePort(port_hdle)].rec_buffer = NULL;
			uart_port_ctrl[GetHandlePort(port_hdle)].rec_buffer_count = 0;
			uart_port_ctrl[GetHandlePort(port_hdle)].mode = COM_MODE_CHAR;
                        uart_port_ctrl[GetHandlePort(port_hdle)].status = COM_RX_ENABLE;
			break;
		default:
                        return ERR_COM_INV_PARAM;
	}
        uart_port_ctrl[GetHandlePort(port_hdle)].status = COM_RX_ENABLE;
	UartEnable(GetHandlePort(port_hdle));  /* Enable the uart port */
	return TRUE;
}

/*********************************************************************
          Purpose: Get the port mode
            Scope: Application
 Input parameters: port_hdle - handle of the port
Output parameters: mode - port mode
           Return: TRUE - Success
                   ERR_COM_INV_HANDLE - invalid handle
          Comment:
************************************************************************/
Err ComGetMode(WORD port_hdle, ComMode *mode)
{
	if(!ComIsHandleValid(port_hdle))	/* invalid handle */
                return ERR_COM_INV_HANDLE;

	*mode = uart_port_ctrl[GetHandlePort(port_hdle)].mode;
	return TRUE;
}

/*********************************************************************
          Purpose: set the baud, parity, stop bit of the port
            Scope: Application
 Input parameters: port_hdle - handle of the port
                   baud - the baud rate to set
                   e.g. UART_BAUD115200
                   config - Parity | Databit | Stop bit
                   e.g. UART_PARITY_NONE | UART_DATABIT8 | UART_STOPBIT1
Output parameters: None 
           Return: TRUE - Success
                   ERR_COM_INV_HANDLE - invalid handle
                   ERR_COM_INV_BUAD_RATE - invalid baud rate
          Comment: call this after ComOpenPort()
************************************************************************/
Err ComConfigPort(WORD port_hdle, WORD baud, WORD config)
{
	if(!ComIsHandleValid(port_hdle))
                return ERR_COM_INV_HANDLE;

	/* chk baud  */
	switch (baud)
	{
		case UART_BAUD115200:
		case UART_BAUD57600 :
                case UART_BAUD38400 :
		case UART_BAUD19200 :
		case UART_BAUD9600  :
		case UART_BAUD4800  :
		case UART_BAUD2400  :
		case UART_BAUD1200  :
		case UART_BAUD600   :
		case UART_BAUD300   :
			break;
		default:
                        return ERR_COM_INV_PARAM;
	}

	/* call DD to init. the port */
	UartInit(GetHandlePort(port_hdle), baud, config);

	return TRUE;
}

/*********************************************************************
          Purpose: get uart port configuration 
            Scope: Application 
 Input parameters: port_hdle - handle of the port
Output parameters: baud - baud rate, e.g. UART_BAUD38400
                   config - Parity | Databit | Stop bit
           Return: TRUE - Success
                   ERR_COM_INV_HANDLE - invalid handle
          Comment:
************************************************************************/
Err ComGetConfig(WORD port_hdle, WORD *baud, WORD *config)
{
	if(!ComIsHandleValid(port_hdle))
                return ERR_COM_INV_HANDLE;

	/* get config by DD */
	*config = UartChkCfg(GetHandlePort(port_hdle));
	return TRUE;
}

/*********************************************************************
          Purpose: Send data
            Scope: Application 
 Input parameters: port_hdle - handle of the port
                   buffer - data to send
                   buffer_size - size of buffer
Output parameters: None 
           Return: TRUE - Success
                   ERR_COM_INV_HANDLE - invalid handle
                   ERR_MEM_NO_MEM   - not enough memory
          Comment:
************************************************************************/
Err ComSendData(WORD port_hdle, BYTE **buffer, UWORD buffer_size)
{
	ComSendLnk *buf_lnk;
	ComSendLnk *chk_lnk;
	ComSendLnk *new_lnk;
	MsgType msg;

	if(!ComIsHandleValid(port_hdle))
                return ERR_COM_INV_HANDLE;

	new_lnk = (ComSendLnk *) pmalloc(sizeof(ComSendLnk));

	if (new_lnk != NULL)	/* enough mem */
	{
		new_lnk->next = NULL;
		new_lnk->byte_send = 0;
		new_lnk->size = buffer_size;
                new_lnk->data = buffer;

//                IntDisableGlobal();
  			DIS_GBL_INT;
		/* find buffer tail */
		buf_lnk = uart_port_ctrl[GetHandlePort(port_hdle)].send_buffer;
		while(buf_lnk != NULL)
		{
			chk_lnk = buf_lnk;
			buf_lnk = buf_lnk->next;
		}

		if (uart_port_ctrl[GetHandlePort(port_hdle)].send_buffer != NULL)
		{	/* already has some queue link */
			chk_lnk->next = new_lnk;
		}
		else    /* no send buffer yet */
		   uart_port_ctrl[GetHandlePort(port_hdle)].send_buffer = new_lnk;

//                IntEnableGlobal();
			ENA_GBL_INT;
	}
	else
                return ERR_MEM_NO_MEM;         /* no mem for new node */

        if (UartChkTxDmaBusy(GetHandlePort(port_hdle)) ) /* dma busy */
    {           
                msg.ptr_fct = UartMgr;
		msg.msgID = UART_START_TX;
                msg.para1 = GetHandlePort(port_hdle);
		msg.para2 = 0;
		msg.msgPBP = NULL;
		MsgAppend(msg, &msgQueue);  /* post a msg to start tx */
        }
        else
            ComWriteOutData(GetHandlePort(port_hdle));

	return TRUE;
}

/*********************************************************************
          Purpose: Send a 8-bit data
            Scope: Application 
 Input parameters: port_hdle - handle of the port
                   data - data to send
Output parameters: None 
           Return: TRUE - Success
                   ERR_COM_INV_HANDLE - invalid handle
                   ERR_MEM_NO_MEM   - not enough memory
          Comment:
************************************************************************/
Err ComSendChar(WORD port_hdle, BYTE data)
{
	BYTE *buf;
	Err result;

	/* port_hdle is checked in ComSendData, so, no need to chk here */

	if(!ComIsHandleValid(port_hdle))
                return ERR_COM_INV_HANDLE;

          while(UartChkTxDmaBusy(GetHandlePort(port_hdle)));
                UartWriteBuf(GetHandlePort(port), (BYTE*)&data, 1);

        #ifdef xxxxxxxxxxxxxxxxxxxx

        if ((buf = (BYTE *) pmalloc(sizeof(BYTE))) == NULL)
        {
        #ifdef DEBUG
                printf("\nComSendChar: no mem ");
        #endif 
                return ERR_MEM_NO_MEM;
        }
	*buf = data;
         if(uart_port_ctrl[UART_A].send_buffer != NULL)
                printf(" error ");
//         printf(" sc:%c", data);
	/* use ComSendData to send it */
	if ((result = (ComSendData(port_hdle, &buf, 1))) != TRUE)
		pfree(buf);  /* fail, should free the buffer */
        #endif
	return result;
}

/*********************************************************************
          Purpose: Get uart data from DD 
            Scope: Internal 
 Input parameters: port - uart port, UART_A or UART_B                     
Output parameters: None
           Return: None                                        
          Comment: copy data from uart buffer to uart manager buffer
************************************************************************/
void ComGetUartBuffer(WORD port)
{
	WORD buf_size;
	EvtType event;
	WORD i=0;
        BYTE data;

	/* get the buffer first to avoid uart buffer overflow */
	if (!((buf_size = UartReadBuf(port, com_buffer))))  /* buffer empty */
		return;


        if(uart_port_ctrl[port].status != COM_RX_ENABLE)      
	{
        #ifndef DEBUG
//todo                UartDisable(port);      /* disable the port if not opened */
        #endif
                return;
	}

	switch(uart_port_ctrl[port].mode)
	{
		case COM_MODE_CHAR:		/* post the char to event queue */
                   for(i=0;i<buf_size;i++)  /* send char as event */
                   {
                        if(!DecodeData(com_buffer[i], &data))
                        {
                          event.eventType = EVT_UART_GET_CHAR;
                          event.eventID = 0;
                          event.para1 = (WORD) data;
                          event.para2 = 0;
                          event.evtPBP = NULL;
                          EvtAppend(event, &evtQueue);
                        }
                   }
                   break;
		case COM_MODE_BUFFER:
                        while ((i<buf_size) && ( uart_port_ctrl[port].rec_buffer_size > uart_port_ctrl[port].rec_buffer_count))
                        {
                               if(!DecodeData(com_buffer[i++], &data))
                               {
                                  uart_port_ctrl[port].rec_buffer[(uart_port_ctrl[port].rec_buffer_count++)] = data;
                               }
                        } 
                        if (uart_port_ctrl[port].rec_buffer_size > uart_port_ctrl[port].rec_buffer_count)
                        {
				/* buffer not full */
				break;
                         }
			/* buffer full, post an event for app. to get the buffer */
                        uart_port_ctrl[port].rec_buffer_size = 0;
                        uart_port_ctrl[port].rec_buffer_count = 0;
                        event.eventType = EVT_UART_GET_BUFFER;
			event.eventID = 0;
			event.para1 = 0;
			event.para2 = 0;
                        event.evtPBP = uart_port_ctrl[port].rec_buffer;  /* buffer address */
			EvtAppend(event, &evtQueue);
			break;
		default:
                #ifdef DEBUG
                printf(" illegal buffer mode ");
                #endif
		break;		/* illegal mode, should never occur */
	}
	return;
}


/*********************************************************************
          Purpose: Send data through DD
            Scope: Internal
 Input parameters: port - port number                     
Output parameters: None
           Return: None                                        
          Comment: data will send out at 16-byte each time
************************************************************************/
void ComWriteOutData(WORD port)
{
	ComSendLnk *send_buffer;
	BYTE send_data[UART_BUF_SIZE];
	UWORD next_byte = 0;

	if (UartChkTxDmaBusy(port) != FALSE)  /* being used */
		return;

	send_buffer = uart_port_ctrl[port].send_buffer;
	if (send_buffer == NULL)	/* no more need to send */
	    return;

	while ((send_buffer != NULL) && (next_byte < UART_BUF_SIZE))
	{
                send_data[next_byte++] = *( ((BYTE*) *(send_buffer->data)) + (send_buffer->byte_send++));
		if (send_buffer->byte_send >= send_buffer->size)  /* this buffer chunk is completed */
		{
			  uart_port_ctrl[port].send_buffer = send_buffer->next;
			  pfree(send_buffer);  /* discard this chunk */
			  send_buffer = uart_port_ctrl[port].send_buffer;
		}
	}
	UartWriteBuf(port, send_data, next_byte);  /* send data through DD */
}

/*********************************************************************
          Purpose: DD interface
            Scope: Message Manager 
 Input parameters: msg - message from message manager                     
Output parameters: None
           Return: None                                        
          Comment: UART DD call this routine
************************************************************************/
void UartMgr(MsgType *msg)
{
        EvtType event;
        BYTE data;

        switch(msg->msgID)
        {     
		case UART_RX_FIRST_BYTE:	/* first byte in buffer */
                        ComGetUartBuffer(msg->para1);
			break;
                case UART_RX_BUF_FULL:          /* uart buffer full */
                        ComGetUartBuffer(msg->para1);
                        break;
		case UART_TX_COMPLETE:		/* tx complete, from DD */
                        ComWriteOutData(msg->para1);
                        break;
		case UART_START_TX:             /* start tx, from ComSendData */
                        ComWriteOutData(msg->para1);  /* send data */
			break;
		case UART_RX_BUF_OVERFLOW:      /* rx buf. overflow */
                   if( uart_port_ctrl[msg->para2].status != COM_RX_ENABLE)
                        break;  /* disabled at some time before */
                    if (uart_port_ctrl[msg->para2].mode == COM_MODE_BUFFER)
                    {
                        ComGetUartBuffer(msg->para2);  /* get bytes in buffer */
                        if (uart_port_ctrl[msg->para2].rec_buffer_size < uart_port_ctrl[msg->para1].rec_buffer_count)
                        {
                          if(!DecodeData(msg->para1, &data))
                          {
				/* buffer not full */
                                /* copy the byte */
                                uart_port_ctrl[msg->para2].rec_buffer[(uart_port_ctrl[msg->para2].rec_buffer_count++)] = data;
                                if (uart_port_ctrl[msg->para2].rec_buffer_count >= uart_port_ctrl[msg->para2].rec_buffer_size )
                                {
                                  /* buffer full, post an event for app. to get the buffer */
        //todo                          uart_port_ctrl[msg->para2].status = COM_RX_DISABLE;
                                  uart_port_ctrl[msg->para1].rec_buffer_size = 0;
                                  uart_port_ctrl[msg->para1].rec_buffer_count = 0;
                                  event.eventType =EVT_UART_GET_BUFFER;
                                  event.eventID = 0;
                                  event.para1 = 0;
                                  event.para2 = 0;
                                  event.evtPBP = uart_port_ctrl[msg->para2].rec_buffer;  /* buffer address */
                                  EvtAppend(event, &evtQueue);
                                }
                             }
                         }

                        }
                        else if (uart_port_ctrl[msg->para2].mode == COM_MODE_CHAR)
                        {                          
                                ComGetUartBuffer(msg->para2);  /* get char in buffer */
                                if(!DecodeData(msg->para1, &data))
                                {
                                event.eventType = EVT_UART_GET_CHAR;
                                event.eventID = 0;
                                event.para1 = (WORD) data;
                                event.para2 = 0;
                                event.evtPBP = NULL;
                                EvtAppend(event, &evtQueue);
                                }
                        }

			break;
		case UART_RX_OVERRUN:		/* rx overrun, cannot slove */
			break;
		case UART_RX_PRTY_ERR:		/* rx parity error, cannot slove */
			break;
/*              case UART_TX_OVERRUN:              tx overrun, cannot slove 
                      break;                    */
		default:
			break;
	}
	return;
}


/*********************************************************************
          Purpose: Init. UartMgr
            Scope: OS
 Input parameters: None
Output parameters: None
           Return: None
          Comment: Call when system reboot
************************************************************************/
void UartMgrInit()
{
    BYTE *x;
        UartARxBuf   =UartARxBuf1;
        UartARxWrkBuf=UartARxBuf2;
        UartARxReadPtr = 0;
        UartARxWritePtr = 0;
	uart_port_ctrl[0].send_buffer = NULL;
	uart_port_ctrl[1].send_buffer = NULL;
        uart_port_ctrl[0].rec_buffer = NULL;
        uart_port_ctrl[1].rec_buffer = NULL;
        ComReset();
        x=pmalloc(10*4092);
        pfree(x);
}




/* put the following code in syshandleevent */

/*
BOOLEAN SysHandleEvent(EvtType *event)
{
        switch (event->eventType)
        {
                case EVT_UART_GET_CHAR:
                AppApp(event->para1);
                return TRUE;
                break;

                case EVT_UART_GET_BUFFER:
                AppBuf((char*)event->evtPBP);
                return TRUE;
                break;
        }
        return FALSE;
}
*/
